/**
 @addtogroup groupLedLine
 @{
 @file ledline.h
 @brief Управление светодиодной лентой RGB
 по однопроводному интерфейсу.
 @}
 */

/*
 @author Матяш А.В. (a.matiash@set-code.ru)
 @date 11.05.2023

 */

#ifndef _LEDLINE_H_
#define _LEDLINE_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>
#include <time.h>

/**
 *  @name Управление драйвером
 *  @{
 */

/**
 *  @brief Старт драйвера.
 *  @details Настройка аппаратной части.
 *  
 *  @param pixelCount Количество светодиодов в ленте.
 *  @param period Период обновления светодиодов в мс.
 *  Минимальное актуальное значение --- время передачи одного пакета 
 *  ~2мс (зависит от длины ленты). Значение 0 соответствует минимально
 *  возможному периоду обновления данных, определяемому аппаратно.
 *  Расчёт длительности эффектов в этом случае искажается.
 *  @param signalInversion Инверсия сигнала: \ref true ---
 *  применяется в схемах с инвертирующим усилителем, либо \ref false --- 
 *  схема подключения сигнала без инверсии.
 *  @return \ref OK при успешном прохождении настройки, иначе \ref ERROR.
 */
STATUS ledLineInit (uint32_t pixelCount, uint32_t period, bool signalInversion);

/**
 *  @brief Остановка драйвера.
 *  @details Остановка задачи, освобождение ресурсов, сброс параметров.
 */
void ledLineStop(void);

/// @}

/**
 *  @name Запуск эффектов
 *  @{
 */


/**
 * @brief Направление распространения эффекта.
 */
typedef enum
{
    LLD_FromEndToPower = -1,    //!< Эффект распространяется от конца ленты к её началу (к месту подключения).
    LLD_NoDirection = 0,        //!< Эффект распространяется равномерно по всей ленте сразу.
    LLD_FromPowerToEnd = 1,     //! <Эффект распространяется от начала ленты (от места подключения) к её концу.
}eLedLineDirection;


/**
 *  @brief Возможные виды эффектов перехода цвета.
 */
typedef enum
{
    leffectNone = -1,           ///< Эффект отсутствует.
    leffectSimple = 0,          ///< Простое переключение на нужный цвет (направление и время не имеют значения).
    leffectSmoothGradient,      ///< Плавный переход в течение заданного времени с градиентом.
                                ///  Скорость изменения цветов точек может быть неравномерна
                                ///  и задаётся с помощью параметра направления. При времени равном нулю
                                ///  вырождается в leffect_simple.
    leffectWave,                ///  Цвет изменяется бегущей волной.
    leffectRunningLight,        ///< Бегущий огонёк (пока только один пиксель без параметров для теста ленты).
    leffectDottedLine,
} ELedlineEffect;


/**
 *  @brief Запуск эффекта светодиодной ленты. Блокирует исполнение.
 *  @details Расчёт параметров - запуск эффекта.
 *  Сама работа эффекта происходит в ledline_work().
 *  Исполнение будет заблокировано до конца выполнения эффекта - обычно на период duration.
 *  
 *  @param color Основной цвет эффекта - XRGB формат.
 *  @param additionalColor Дополнительный цвет эффекта - XRGB формат.
 *  @param effect Номер эффекта - значение из перечисления \ref ELedlineEffect.
 *  @param windowWidth Ширина окна эффекта.
 *  @param direction Направление эффекта - значение из перечисления \ref eLedLineDirection.
 *  @param duration Время выполнения эффекта в мс.
 */
void ledLineStartEffect (uint32_t color, uint32_t additionalColor, ELedlineEffect effect, unsigned int windowWidth, eLedLineDirection direction, unsigned int duration);

/// @}

#ifdef __cplusplus
}
#endif

#endif
