/**
 *  @addtogroup groupStdTypes
 *  @{
 * @file assert.h
 * @brief Механизмы диагностики и проверки.
 * @details См. стандарт C11 7.2.
 * @see <a href="https://www.iso.org/standard/57853.html">C11 standard 7.2</a>.
 *  @}
 */

/*
 * @author Sergey Sokolov
 * @author Dubovich Timofey
 */
#ifndef ASSERT_H
#define ASSERT_H

//-------------------------------------------
// Asserts.

/**
 * Макрос-заглушка.
 */
#define __ASSERT_NOOP ((void) 0)

#if defined(NDEBUG)
    #define assert(ignore) __ASSERT_NOOP
#else

    /**
     * Обработать сработавший ассерт.
     * @param __assertion Текст ассерта.
     * @param __file Имя файла с ассертом.
     * @param __line Номер строки с ассертом.
     * @param __function Имя функции с ассертом.
     */
    void __assert_fail (__const char *__assertion, __const char *__file, unsigned int __line, __const char *__function);

    /**
     * Механизм проверки. В случае ложности утверждения x процесс исполнения будет прерван, а в stderr будет распечатано сообщение о сработавшей диагностике.
     * @param expr Утверждение для проверки.
     */
    #define assert(expr)                            \
      ((expr) ? __ASSERT_NOOP : __assert_fail (#expr, __FILE__, __LINE__, __func__))
#endif


//-------------------------------------------
// Static asserts.

#if __STDC_VERSION__  >= 201112L    // C11 and higher.
    /**
     * Механизм статической пред-компиляционной проверки. В случае ложности утверждения x программа не скомпилируется.
     * @param x Утверждение для проверки.
     */
    #define static_assert(x...) _Static_assert(x, #x)
#else
    #define static_assert(x...) // Not supported in non c11 modes.
#endif

// ONLY for Eclipse IDE parser.
#if defined(__CDT_PARSER__)
    // Eclipse IDE parser doesn't support static assert syntax (outside fuctions).
    #undef static_assert
    #define static_assert(x...)
#endif



#endif //ASSERT_H
