/**
    @addtogroup groupCore 
    @{
    @file cache.h
    @brief Методы работы с КЭШ-памятью.
    @details Файл содержит объявления функций работы с КЭШ-памятью процессора.
    @see Общее описание см. в главе \ref sectionCache.
    @}

    @page pageCore

    @section sectionCache Работа с встроенной КЭШ-памятью процессора
    @see Описание методов работы с КЭШ-памятью см. в файле cache.h.
    
    Встроенная КЭШ-память процессора обеспечивает существенное повышение
    быстродействия вычислительной системы. Ядро \b MULTEX при запуске размещает в
    памяти таблицу виртуализации памяти и активирует \b MMU (модуль управления
    памятью) в соответствии с этой таблицей. При этом для каждой страницы
    размером в \b 1М память может быть настроена как некэшируемая, кэшируемая с
    записью, или кэшируемая с отложенной записью. Для этого в составе \b BSP для
    конкретного модуля назначается специальная структура \ref SYS_MEM_MAP,
    описывающая то, как нужно настраивать память. Так как \b MULTEX работает с
    использованием кэш, то в некоторых случаях возможно рассогласование между
    содержимым памяти и данными в кэш. Это бывает, например, когда какой-либо
    блок заносит данные в память прямым доступом.
    
 */

#ifndef CACHE_H_
#define CACHE_H_

#ifdef __cplusplus
extern "C" {
#endif


/**
 *  @brief Аннулировать кэш данных.
 *  @details Функция аннулирует весь кэш данных.
 */
void invalidate_dcache_all(void);

/**
 *  @brief Сбросить кэш данных.
 *  @details Функция сбрасывает все данные из кэш в память.
 */
void flush_dcache_all(void);

/**
 *  @brief Аннулировать кэш в области памяти.
 *  @details Функция аннулирует кэш для блока в указанном месте памяти.
 *  
 *  @param start Адрес начала блока памяти данных.
 *  @param stop Адрес конца блока памяти данных.
 */
void invalidate_dcache_range(unsigned long start, unsigned long stop);

/**
 *  @brief Сбросить в кэш указанный блок памяти.
 *  @details Функция сбрасывает в память кэш для блока в указанном месте.
 *  
 *  @param start Адрес начала блока памяти данных.
 *  @param stop Адрес конца блока памяти данных.
 */
void flush_dcache_range(unsigned long start, unsigned long stop);

/**
 *  @brief Отключить кэш данных.
 *  @details Функция отключает кэширование данных.
 *  Возможно использовать в ходе выполнения программы.
 */
void dcache_disable (void);

/**
 *  @brief Включить кэш данных.
 *  @details Функция включает кэширование данных.
 *  Возможно использовать в ходе выполнения программы.
 */
void dcache_enable (void);

/**
 *  @brief Статус кэша данных.
 *  @details Проверка состояния кэша данных.
 *  
 *  @return \b 0 - кэш отключен, \b иначе - кэш включен.
 */
int dcache_status (void);

/**
 *  @brief Аннулировать весь кэш команд.
 *  @details Функция аннулирует весь кэш команд.
 */
void invalidate_icache_all(void);

/**
 *  @brief Статус кэша команд.
 *  @details Проверка состояния кэша команд.
 *  
 *  @return \b 0 - кэш отключен, \b иначе - кэш включен.
 */
int icache_status (void);



/// @privatesection

void arm_init_before_mmu(void);

/*
 * Flush range from all levels of d-cache/unified-cache used:
 * Affects the range [start, start + size - 1]
 */
void  flush_cache(unsigned long start, unsigned long size);

void  inval_cache(unsigned long start, unsigned long size);


#ifdef __cplusplus
}
#endif
#endif // ifndef CACHE_H_
