/**
 * @addtogroup groupCore
 * @{
 * @file crt.h
 * @brief Функции для работы с терминалом и клавиатурой,
 * а также заглушки для обратной совместимости.
 * @}
 */

#ifndef CRT_H_
#define CRT_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "semlib.h"  // NO_WAIT, WAIT_FOREVER

//--------------------------------------------------

/**
 *  @name Коды цветов.
 *  @{
 */
#define clNone 0      ///< @details Синий.
#define clBlue 34     ///< @details Синий.
#define clGreen 32    ///< @details Зеленый.
#define clBlack 30    ///< @details Черный.
#define clRed 31      ///< @details Красный.
#define clCyan 36     ///< @details Циан.
#define clMagenta 35  ///< @details Маджента.
#define clYellow 33   ///< @details Желтый.
#define clWhite 37    ///< @details Белый.

#define clBrightBlack 90    ///< @details Серый.
#define clBrightRed 91      ///< @details Светло-красный.
#define clBrightGreen 92    ///< @details Светло-зелёный.
#define clBrightYellow 93   ///< @details Светло-жёлтый.
#define clBrightBlue 94     ///< @details Светло-синий.
#define clBrightMagenta 95  ///< @details Светлый маджента.
#define clBrightCyan 96     ///< @details Светлый циан.
#define clBrightWhite 97    ///< @details Яркий белый.

#define bgBrightBlack 100    ///< @details Серый фон.
#define bgBrightRed 101      ///< @details Светло-красный фон.
#define bgBrightGreen 102    ///< @details Светло-зелёный фон.
#define bgBrightYellow 103   ///< @details Светло-жёлтый фон.
#define bgBrightBlue 104     ///< @details Светло-синий фон.
#define bgBrightMagenta 105  ///< @details Светлый маджента (фон).
#define bgBrightCyan 106     ///< @details Светлый циан (фон).
#define bgBrightWhite 107    ///< @details Яркий белый фон.

/// @}

/**
 *  @name Атрибуты текста.
 *  @{
 */
#define taNormal 0      ///< @details Выключение всех атрибутов.
#define taLight 1       ///< @details Увеличение яркости текста.
#define taBgLight 2     ///< @details Увеличение яркости фона.
#define taUnderlined 4  ///< @details Подчеркивание текста
#define taInverse 8     ///< @details Инвертирование цветов текста.
/// @}

/**
 *  @name Управление режимом вывода
 *  @{
 */

/**
 *  @brief Включить поддержку \b ESC последовательностей.
 *  @details Функция активирует управляющие последовательности.
 *  По умолчанию управляющеи последовательности в отладочном выводе
 *  включены.
 */
void crtOn ();

/**
 *  @brief Выключить поддержку \b ESC последовательностей.
 *  @details Функция отключает управляющие последовательности 
 *  в выводе на консоль.
 *  По умолчанию управляющеи последовательности в отладочном выводе
 *  включены.
 */
void crtOff ();
/// @}

/**
 * @name Функции и макросы для работы с терминалом.
 * @note Терминал должен быть подключен к \b stdout.
 * @{
 */

/**
 * Отправить ESC-последовательность.
 * @details СSI будет отправлен перед управляющей строкой.
 * @param s Управляющая строка (без CSI).
 */
void CSI (const char *s);

/**
 * Переместить курсор в терминале.
 * @private
 * @param x Позиция по горизонтали.
 * @param y Позиция по вертикали.
 * @deprecated x,y - перепутаны местами!
 * Использовать cursorMove ().
 */
void __attribute__((deprecated)) gotoxy (char x, char y);

/**
 *  @brief Установить курсор в терминале на заданную позицию.
 *  @details Устанавливает курсор в заданную строку на заданный символ.
 *  Начало отсчёта (позиция 1:1) --- левый верхний угол.
 *  @param row Номер строки.
 *  @param column Номер символа (колонки).
 *
 */
void cursorMove (unsigned char row, unsigned char column);

/**
 * Сделать курсор видимым.
 */
void cursorOn (void);

/**
 * Скрыть курсор.
 */
void cursorOff (void);

/**
 * Сохранить положение курсора.
 */
void cursorStore (void);

/**
 * Восстановить положение курсора.
 */
void cursorRestore (void);

/**
 * Очистить экран терминала.
 */
void clrscr (void);

/**
 * Установить цвет текста.
 * @param C Код цвета.
 */
void textColor (char C);

/**
 * Установить цвет фона.
 * @param C Код цвета.
 */
void textBackground (char C);

/**
 * Установить атрибут текста.
 * @param TA Атрибут текста.
 */
void textAttr (char TA);
/// @}

//--------------------------------------------------
/**
 * @name Функции для работы с клавиатурой.
 * @{
 */

/**
 * Проверить, есть ли зажатая клавиша на клавиатуре.
 * @return 0, если клавиши или клавиатуры нет, не-0, если есть.
 */
int keyPressed (void);

/**
 * Вернуть следующую нажатую клавишу.
 * @return Следующая нажатая клавиша, либо 0 при отсутствии клавиатуры или ошибке.
 * @note Функция блокирует исполнение до тех пор, пока не будет получена клавиша.
 */
char readKey (void);

/**
 * Вернуть следующую нажатую клавишу, при этом ждать не более установленного времени.
 * @param ticks Количество тиков, которое следует прождать (см. \see sysClkRateGet()), либо \ref NO_WAIT для отсутствия ожидания, либо \ref WAIT_FOREVER для вечного ожидания.
 * @return Следующая нажатая клавиша, либо 0 при отсутствии клавиатуры или ошибке.
 * @note Функция блокирует исполнение до тех пор, пока не будет получена клавиша.
 */
char readKey_Timeout (int ticks);
/// @}

//--------------------------------------------------
/**
 * @name Заглушки функций для работы с пищалкой.
 * @{
 */

/**
 * ЗАГЛУШКА: Включить пищалку на некоторое время.
 * @param F Частота звука.
 * @param t Длительность писка в тиках (\see sysClkRateGet()).
 * @return Всегда возвращает 0.
 * @note Это функция-заглушка, она фактически не отрабатывает.
 */
int soundt (int F, int t);

/**
 * ЗАГЛУШКА: Включить пищалку.
 * @param F Частота звука.
 * @return Всегда возвращает 0.
 * @note Это функция-заглушка, она фактически не отрабатывает.
 */
int sound (int F);

/**
 * ЗАГЛУШКА: Выключить пищалку.
 * @return Всегда возвращает 0.
 * @note Это функция-заглушка, она фактически не отрабатывает.
 */
int nosound (void);
/// @}

/**
 *  @name Вывод прогресс-бара в консоли
 *  @{
 */

/**
 *  @brief Запуск вывода прогресс-бара в консоли.
 *  @details Функция подготавливает место для вывода прогресс-бара
 *  и рисует пустой прогресс бар в текущей строке.
 *  Текущая строка должна быть пустой --- для этого предыдущий вывод
 *  в консоль должен содержать символ перевода строки. В процессе
 *  вывода прогресс-бара не должно быть другого вывода в консоль.
 *  
 *  @param legend Подпись --- короткая строка, выводимая перед прогресс-баром.
 *  @return \ref true если прогресс-бар успешно инициализирован, иначе \ref false.
 */
bool barStart (const char *legend);

/**
 *  @brief Следующий шаг прогресс-бара.
 *  @details Функция перерисует текущую строку прогресс-бара
 *  и выведет обновлённое значение. Для корректной работы
 *  прогресс-бар должен быть запущен с помощью barStart().
 *  
 *  @param percent Значение шкалы прогресс-бара в процентах.
 */
void barGo (int percent);

/**
 *  @brief Завершение вывода прогресс-бара.
 *  @details Функция завершает вывод прогресс-бара
 *  и возвращает консоль в нормальный режим работы.
 *  
 */
void barStop ();

/// @}

#ifdef __cplusplus
}
#endif
#endif  // ifndef CRT_H_
