/**
 * @addtogroup groupCore
 * @{
 * @file datetime.h
 * @brief Дополнительные функции для работы с датой/временем.
 * @}
 */

#ifndef DATETIME_H_
#define DATETIME_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "time.h"

/**
 *  Структура формата Дата / Время.
 */
typedef struct date_time
{
    int Hour;   ///< Часы 0-23.
    int Minute; ///< Минуты 0-59.
    int Second; ///< Секунды 0-59.
    int Day;    ///< Дни. Первый день = 1.
    int Month;  ///< Месяцы. Январь = 1.
    int Year;   ///< Годы.
} DATE_TIME;

/**
 *  @brief Структура формата Дата / Время (DOS-совместимая).
 *  @details Компактное представление даты / времени совместимое с DOS.
 */
typedef struct dtcompact
{
    UINT Sec2 :5;      ///< @details Пары секунд (значение 1 обозначает 2 секунды).
    UINT Minute :6;    ///< @details Минуты.
    UINT Hour :5;      ///< @details Часы.
    UINT Day :5;       ///< @details Дни.
    UINT Month :4;     ///< @details Месяцы.
    UINT Year :7;      ///< @details Годы.
} DT_COMPACT;

/**
 * Распечатать или/и установить дату.
 * @details Функция потенциально устанавливает новую дату, после чего распечатывает текущую/новую дату в stdout.
 * @param newDate Указатель на строку с новой датой в формате "ДД/ММ/ГГГГ" или NULL, если установка даты не требуется.
 * @return Всегда возвращает 0.
 */
int date(const char *newDate);


/**
 * Распечатать или/и установить время.
 * @details Функция потенциально устанавливает новое время, после чего распечатывает текущее/новое время в stdout.
 * @param newTime Указатель на строку с новой датой в формате "ЧЧ:ММ:СС" или NULL, если установка даты не требуется.
 * @return Всегда возвращает 0.
 */
int setTime(const char *newTime);


/**
 * Получить кол-во секунд, прошедших со времени pDateTime0 и до времени pDateTime1.
 * @param pDateTime1 Конечная временная точка.
 * @param pDateTime0 Начальная временная точка.
 * @return Количество секунд разницы.
 * @note Работает только для дат моложе 1970 года.
 */
int diffdate(const DATE_TIME* pDateTime1, const DATE_TIME* pDateTime0);

/**
 * Увеличить значение переменной DATE_TIME на одну секунду.
 * @param pDateTime Переменная, значение которой нужно увеличить.
 * @return OK при успехе, ERROR иначе.
 */
STATUS increaseDateTimeBySecond(DATE_TIME* pDateTime);


/**
 * Преобразовать структуру DATE_TIME в абсолютное время.
 * @param pDateTime Cтруктура DATE_TIME.
 * @return Кол-во секунд, прошедших с 1 января 1970 года для представленного времени или -1 при ошибке.
 */
time_t convertDateTimeTime(const DATE_TIME *pDateTime);



/// @privatesection               
//DATE_TIME *dt могут быть (и будут) изменены!
//Возвращают OK и ERROR
extern int (*getDateTime)(DATE_TIME *dt);                 
extern int (*setDateTime)(const DATE_TIME *dt);

DT_COMPACT getCompactDT();
DT_COMPACT convertDateTime(const DATE_TIME* pDateTime);
void convertCompactDT(DT_COMPACT dtc, DATE_TIME *dt);
void assignDTFunc(int (*get)(DATE_TIME *dt), int (*set)(const DATE_TIME *dt));

#ifdef __cplusplus
}
#endif
#endif // ifndef DATETIME_H_
