/**
 *  @addtogroup groupStdTypes
 *  @{
 * @file errno.h
 * @brief Обработка причин ошибок в библиотечных функциях.
 * @details См. стандарт C11 7.5.
 * @see <a href="https://www.iso.org/standard/57853.html">C11 standard 7.5</a>.
 * @note В основном используется 'внешними' библиотеками, например, FFMPEG или LIBZ.
 *  @}
 */

/*
 * @author Sergey Sokolov
 * @author Dubovich Timofey
*/

#ifndef __ERRNO_H_
#define __ERRNO_H_

/**
 * Переменная, хранящая в себе код ошибки.
 */
extern int errno;

/**
 *  @name Коды ошибок
 *  @{
 */
#define EPERM             1  ///< @details Operation not permitted
#define ENOENT            2  ///< @details No such file or directory
#define ESRCH             3  ///< @details No such process
#define EINTR             4  ///< @details Interrupted system call
#define EIO               5  ///< @details I/O error
#define ENXIO             6  ///< @details No such device or address
#define E2BIG             7  ///< @details Argument list too long
#define ENOEXEC           8  ///< @details Exec format error
#define EBADF             9  ///< @details Bad file number
#define ECHILD           10  ///< @details No child processes
#define EAGAIN           11  ///< @details Try again
#define ENOMEM           12  ///< @details Out of memory
#define EACCES           13  ///< @details Permission denied
#define EFAULT           14  ///< @details Bad address
#define ENOTBLK          15  ///< @details Block device required
#define EBUSY            16  ///< @details Device or resource busy
#define EEXIST           17  ///< @details File exists
#define EXDEV            18  ///< @details Cross-device link
#define ENODEV           19  ///< @details No such device
#define ENOTDIR          20  ///< @details Not a directory
#define EISDIR           21  ///< @details Is a directory
#define EINVAL           22  ///< @details Invalid argument
#define ENFILE           23  ///< @details File table overflow
#define EMFILE           24  ///< @details Too many open files
#define ENOTTY           25  ///< @details Not a typewriter
#define ETXTBSY          26  ///< @details Text file busy
#define EFBIG            27  ///< @details File too large
#define ENOSPC           28  ///< @details No space left on device
#define ESPIPE           29  ///< @details Illegal seek
#define EROFS            30  ///< @details Read-only file system
#define EMLINK           31  ///< @details Too many links
#define EPIPE            32  ///< @details Broken pipe
#define EDOM             33  ///< @details Math argument out of domain of func
#define ERANGE           34  ///< @details Math result not representable
#define EDEADLK          35  ///< @details Resource deadlock would occur
#define ENAMETOOLONG     36  ///< @details File name too long
#define ENOLCK           37  ///< @details No record locks available
#define ENOSYS           38  ///< @details Function not implemented
#define ENOTEMPTY        39  ///< @details Directory not empty
#define ELOOP            40  ///< @details Too many symbolic links encountered
#define EWOULDBLOCK      EAGAIN  ///< @details Operation would block
#define ENOMSG           42  ///< @details No message of desired type
#define EIDRM            43  ///< @details Identifier removed
#define ECHRNG           44  ///< @details Channel number out of range
#define EL2NSYNC         45  ///< @details Level 2 not synchronized
#define EL3HLT           46  ///< @details Level 3 halted
#define EL3RST           47  ///< @details Level 3 reset
#define ELNRNG           48  ///< @details Link number out of range
#define EUNATCH          49  ///< @details Protocol driver not attached
#define ENOCSI           50  ///< @details No CSI structure available
#define EL2HLT           51  ///< @details Level 2 halted
#define EBADE            52  ///< @details Invalid exchange
#define EBADR            53  ///< @details Invalid request descriptor
#define EXFULL           54  ///< @details Exchange full
#define ENOANO           55  ///< @details No anode
#define EBADRQC          56  ///< @details Invalid request code
#define EBADSLT          57  ///< @details Invalid slot
#define EDEADLOCK        EDEADLK
#define EBFONT           59  ///< @details Bad font file format 
#define ENOSTR           60  ///< @details Device not a stream 
#define ENODATA          61  ///< @details No data available 
#define ETIME            62  ///< @details Timer expired 
#define ENOSR            63  ///< @details Out of streams resources 
#define ENONET           64  ///< @details Machine is not on the network 
#define ENOPKG           65  ///< @details Package not installed 
#define EREMOTE          66  ///< @details Object is remote 
#define ENOLINK          67  ///< @details Link has been severed 
#define EADV             68  ///< @details Advertise error 
#define ESRMNT           69  ///< @details Srmount error 
#define ECOMM            70  ///< @details Communication error on send 
#define EPROTO           71  ///< @details Protocol error 
#define EMULTIHOP        72  ///< @details Multihop attempted 
#define EDOTDOT          73  ///< @details RFS specific error 
#define EBADMSG          74  ///< @details Not a data message 
#define EOVERFLOW        75  ///< @details Value too large for defined data type 
#define ENOTUNIQ         76  ///< @details Name not unique on network 
#define EBADFD           77  ///< @details File descriptor in bad state 
#define EREMCHG          78  ///< @details Remote address changed 
#define ELIBACC          79  ///< @details Can not access a needed shared library 
#define ELIBBAD          80  ///< @details Accessing a corrupted shared library 
#define ELIBSCN          81  ///< @details .lib section in a.out corrupted 
#define ELIBMAX          82  ///< @details Attempting to link in too many shared libraries 
#define ELIBEXEC         83  ///< @details Cannot exec a shared library directly 
#define EILSEQ           84  ///< @details Illegal byte sequence 
#define ERESTART         85  ///< @details Interrupted system call should be restarted 
#define ESTRPIPE         86  ///< @details Streams pipe error 
#define EUSERS           87  ///< @details Too many users 
#define ENOTSOCK         88  ///< @details Socket operation on non-socket 
#define EDESTADDRREQ     89  ///< @details Destination address required 
#define EMSGSIZE         90  ///< @details Message too long 
#define EPROTOTYPE       91  ///< @details Protocol wrong type for socket 
#define ENOPROTOOPT      92  ///< @details Protocol not available 
#define EPROTONOSUPPORT  93  ///< @details Protocol not supported 
#define ESOCKTNOSUPPORT  94  ///< @details Socket type not supported 
#define EOPNOTSUPP       95  ///< @details Operation not supported on transport endpoint 
#define EPFNOSUPPORT     96  ///< @details Protocol family not supported 
#define EAFNOSUPPORT     97  ///< @details Address family not supported by protocol 
#define EADDRINUSE       98  ///< @details Address already in use 
#define EADDRNOTAVAIL    99  ///< @details Cannot assign requested address 
#define ENETDOWN        100 ///< @details Network is down 
#define ENETUNREACH     101 ///< @details Network is unreachable 
#define ENETRESET       102 ///< @details Network dropped connection because of reset 
#define ECONNABORTED    103 ///< @details Software caused connection abort 
#define ECONNRESET      104 ///< @details Connection reset by peer 
#define ENOBUFS         105 ///< @details No buffer space available 
#define EISCONN         106 ///< @details Transport endpoint is already connected 
#define ENOTCONN        107 ///< @details Transport endpoint is not connected 
#define ESHUTDOWN       108 ///< @details Cannot send after transport endpoint shutdown 
#define ETOOMANYREFS    109 ///< @details Too many references: cannot splice 
#define ETIMEDOUT       110 ///< @details Connection timed out 
#define ECONNREFUSED    111 ///< @details Connection refused 
#define EHOSTDOWN       112 ///< @details Host is down 
#define EHOSTUNREACH    113 ///< @details No route to host 
#define EALREADY        114 ///< @details Operation already in progress 
#define EINPROGRESS     115 ///< @details Operation now in progress 
#define ESTALE          116 ///< @details Stale NFS file handle 
#define EUCLEAN         117 ///< @details Structure needs cleaning 
#define ENOTNAM         118 ///< @details Not a XENIX named type file 
#define ENAVAIL         119 ///< @details No XENIX semaphores available 
#define EISNAM          120 ///< @details Is a named type file 
#define EREMOTEIO       121 ///< @details Remote I/O error 
#define EDQUOT          122 ///< @details Quota exceeded 
#define ENOMEDIUM       123 ///< @details No medium found 
#define EMEDIUMTYPE     124 ///< @details Wrong medium type 
#define ECANCELED       125 ///< @details Operation Canceled 
#define ENOKEY          126 ///< @details Required key not available 
#define EKEYEXPIRED     127 ///< @details Key has expired 
#define EKEYREVOKED     128 ///< @details Key has been revoked 
#define EKEYREJECTED    129 ///< @details Key was rejected by service 
#define EOWNERDEAD      130 ///< @details Robust mutex: Owner died 
#define ENOTRECOVERABLE 131 ///< @details Robust mutex: State not recoverable 
#define ERFKILL         132 ///< @details Operation not possible due to RF-kill 
#define EHWPOISON       133 ///< @details Memory page has hardware error 
#define ENOTSUP         200
/// @}



#endif /* __ERRNO_H_ */
