/**
 *  @file a20graph-example.c
 *  @author Матяш А.В. (a.matiash@set-code.ru)
 *  @brief Файл примеров и тестов для a20graph.
 *  @date 13.01.2023
 */

#include <multimedia/a20graph.h>
#include <multimedia/fonts.h>
#include <iolib.h>

/**
 * Создание скриншота видимой области экрана.
 * Будет создан файл вида C:/scr/scr_0000.bmp с заданным номером.
 */
static int makeSS (char *prefix, int number) {
    char fileName[50];
    if(!dirExists("scr/"))
    {
        mkdir((char*)"scr/");
    }
    sprintf (fileName, "%s%s_%04i.bmp", "scr/", prefix, number);

    // Видимая область экрана
    HDC buffer = SCREEN;
    int height = buffer->h, width = buffer->w;

    int fdWr = open (fileName, O_RDWR | O_CREAT);
    if (fdWr < 3) {
        printf ("Screenshot error: cannot open file %s\n", fileName);
        return -1;
    }

    // Структура файла:
    int bytePerPx = 4;
    unsigned short bfType = (((short int)'M') << 8) | ((short int)'B');
    unsigned int bfSize = height * width * bytePerPx;
    unsigned int bfOffBits = 14 + 40;  // Смещение до начала массива данных
    bfSize += bfOffBits;
    bfSize += height * (width % 4) * bytePerPx;
    unsigned short int filler = 0;

    unsigned int biSize = 40;
    int biWidth = width;
    int biHeight = height;
    unsigned short biPlanes = 1;
    unsigned short biBitCount = 32;
    unsigned int biCompression = 0;  // BI_RGB
    unsigned int biSizeImage = 0;
    int biXPelsPerMeter = 800;
    int biYPelsPerMeter = 600;
    unsigned int biClrUsed = 0;
    unsigned int biClrlImportant = 0;

    write (fdWr, (void*)&bfType, sizeof (short));
    write (fdWr, (void*)&bfSize, sizeof (int));
    write (fdWr, (void*)&filler, sizeof (short));
    write (fdWr, (void*)&filler, sizeof (short));
    write (fdWr, (void*)&bfOffBits, sizeof (int));

    write (fdWr, (void*)&biSize, sizeof (int));
    write (fdWr, (void*)&biWidth, sizeof (int));
    write (fdWr, (void*)&biHeight, sizeof (int));
    write (fdWr, (void*)&biPlanes, sizeof (short));
    write (fdWr, (void*)&biBitCount, sizeof (short));
    write (fdWr, (void*)&biCompression, sizeof (int));
    write (fdWr, (void*)&biSizeImage, sizeof (int));
    write (fdWr, (void*)&biXPelsPerMeter, sizeof (int));
    write (fdWr, (void*)&biYPelsPerMeter, sizeof (int));
    write (fdWr, (void*)&biClrUsed, sizeof (int));
    write (fdWr, (void*)&biClrlImportant, sizeof (int));

    // Простая запись
    // write(fdWr, (void*)buffer->addr[0], buffer->w * buffer->h * 4);

    // Запись с инверсией по H
    for (int h = buffer->h - 1; h >= 0; h--) {
        unsigned int *addr = (unsigned int *)buffer->addr[0];
        addr += h * buffer->w;
        write (fdWr, (void*)addr, buffer->w * 4);
    }
    close (fdWr);
    return 0;
}

static int ssCnt = 0;  //Счётчик текущих скринов

void screenShot () {
    makeSS ("ss", ssCnt++);
}

static char testString[] = "Test Print!";

void fontTest () {
    // Инициализация графического адаптера в режиме HDMI.
    if ((int) getLFB () == 0) 
        setVideoMode (MODE_1920x1080, 32);
    // Инициализация 2D акселератора.
    init_2D_engine ();

    // Заполнить всю конструируемую поверхность белым цветом.
    fillRect (CONSTR, 0, 0, CONSTR->w, CONSTR->h, 0xFF, 0xFF3A3A3A, G2D_FIL_NONE);


    /// [ttf test]
    // Загрузка шрифта
    pTtfFont font  = ttf_LoadFont ("fnt/nsans-md.ttf", 64, 0x3A3A3A, ttf_NoGlyphSaving);

    // Расчёт ограничивающего прямоугольника
    sTextRect rect;
    ttf_GetTextRect (&rect.w, &rect.h, font, testString);
    rect.x = (CONSTR->w - rect.w) / 2;
    rect.y = (CONSTR->h - rect.h) / 2;

    // Показать ограничивающий прямоугольник
    fillRect (CONSTR, rect.x, rect.y, rect.w, rect.h, 0xFF, 0xFFFFFFFF, G2D_FIL_NONE);

    // Вывести надпись по центру 
    ttf_PrintRect (CONSTR, (pTextRect)&rect, alignHCenter | alignVMiddle, font, testString);

    // Удалить шрифт после использования
    ttf_FreeFont (font);
    /// [ttf test]



    // Смена видимой и конструируемой поверхностей.
    FlipScreenAndConstr ();

    // Ожидание окончания процесса вывода новой поверхности.
    waitVerticalRetrace ();

    // Сохранение снимка экрана с номером режима в имени.
    char str[10];
    sprintf (str, "%s", "fnt");
    makeSS (str, 0);


}


/// [bitBlt test func]
void bitBltTest (int blt_flags) {
    // Инициализация графического адаптера в режиме HDMI.
    if ((int) getLFB () == 0) 
        setVideoMode (MODE_1920x1080, 32);
    init_2D_engine ();

    // Начало подготовки нового экрана.

    // Размеры для вывода тестовых поверхностей.
    int dw = CONSTR->w / 5;
    int dh = CONSTR->h / 5;

    // Создание тестовых поверхностей.
    HDC surfR = newSurface (dw * 3, dh * 3, G2D_FMT_ARGB8888);
    HDC surfG = newSurface (dw * 3, dh * 3, G2D_FMT_ARGB8888);
    HDC surfB = newSurface (dw * 3, dh * 3, G2D_FMT_ARGB8888);

    // Заполнить всю конструируемую поверхность белым цветом.
    fillRect (CONSTR, 0, 0, CONSTR->w, CONSTR->h, 0xFF, 0xFFFFFFFF, G2D_FIL_NONE);

    // Заполнение тестовых поверхностей
    // (просто копирование как есть).
    fillRect (surfR, 0, 0, surfR->w, surfR->h, 0, 0x40FF0000, G2D_BLT_NONE);
    fillRect (surfG, 0, 0, surfG->w, surfG->h, 0, 0x4000FF00, G2D_BLT_NONE);
    fillRect (surfB, 0, 0, surfB->w, surfB->h, 0, 0x400000FF, G2D_BLT_NONE);

    // Ключевой цвет для тестирования Color Key
    unsigned int colorKey = 0xFF00FF00;

    // Копирование созданных поверхностей
    // в конструируемую поверхность.

    // Простое копирование поверхностей.
    bitBlt (CONSTR, dw * 0, dh * 2, surfR->w, surfR->h, surfR, 0, 0, 0x80, colorKey, blt_flags);
    bitBlt (CONSTR, dw * 2, dh * 0, surfB->w, surfB->h, surfB, 0, 0, 0x80, colorKey, blt_flags);
    bitBlt (CONSTR, dw * 1, dh * 1, surfG->w, surfG->h, surfG, 0, 0, 0x80, colorKey, blt_flags);

    // Смена видимой и конструируемой поверхностей.
    FlipScreenAndConstr ();

    // Ожидание окончания процесса вывода новой поверхности.
    waitVerticalRetrace ();

    // Сохранение снимка экрана с номером режима в имени.
    char str[10];
    sprintf (str, "%s", "bbt");
    makeSS (str, blt_flags);

    // Удаление созданных объектов из памяти
    deleteSurface (surfR);
    deleteSurface (surfG);
    deleteSurface (surfB);
}
/// [bitBlt test func]

void stretchBltTest (int blt_flags) {

    // Инициализация графического адаптера в режиме HDMI.
    if ((int) getLFB () == 0) 
        setVideoMode (MODE_1920x1080, 32);
    // Инициализация 2D акселератора.
    init_2D_engine ();

    // Начало подготовки нового экрана.

    /// [stretchBlt test]
    // Загрузка поверхностей из файлов.

    HDC surfBG = loadPNGSurface ("surf/bkgr1080.png");
    HDC surfImg = loadPNGSurface ("surf/mario.png");

    // Размеры накладываемой поверхности (1/2 высоты экрана).
    int dstA = CONSTR->h / 2;
    // Координаты, куда положить накладываемую поверхность.
    int dstX = (CONSTR->w - dstA) / 2;
    int dstY = (CONSTR->h - dstA) / 2;

    // Копирование созданных поверхностей
    // в конструируемую поверхность с масштабированием.
    // Подложка
    stretchBlt (CONSTR, 0, 0, CONSTR->w, CONSTR->h, surfBG, 0, 0, surfBG->w, surfBG->h, 0, 0, G2D_BLT_NONE);
    // Изображение
    stretchBlt (CONSTR, dstX, dstY, dstA, dstA, surfImg, 0, 0, surfImg->w, surfImg->h, 0, 0, blt_flags);
    /// [stretchBlt test]

    // Смена видимой и конструируемой поверхностей.
    FlipScreenAndConstr ();

    // Ожидание окончания процесса вывода новой поверхности.
    waitVerticalRetrace ();

    // Сохранение снимка экрана с номером режима в имени.
    char str[10];
    sprintf (str, "%s", "sbt");
    makeSS (str, blt_flags);

    // Удаление созданных объектов из памяти
    deleteSurface (surfBG);
    deleteSurface (surfImg);
}

void graphTest () {
    bitBltTest (G2D_BLT_NONE);
    bitBltTest (G2D_BLT_PIXEL_ALPHA);
    bitBltTest (G2D_BLT_PLANE_ALPHA);
    bitBltTest (G2D_BLT_MULTI_ALPHA);
    bitBltTest (G2D_BLT_PIXEL_ALPHA | G2D_BLT_SRC_COLORKEY);

    stretchBltTest (G2D_BLT_PIXEL_ALPHA);
    stretchBltTest (G2D_BLT_PIXEL_ALPHA | G2D_BLT_FLIP_HORIZONTAL);
    stretchBltTest (G2D_BLT_PIXEL_ALPHA | G2D_BLT_FLIP_VERTICAL);
    stretchBltTest (G2D_BLT_PIXEL_ALPHA | G2D_BLT_ROTATE90);
    stretchBltTest (G2D_BLT_PIXEL_ALPHA | G2D_BLT_ROTATE180);
    stretchBltTest (G2D_BLT_PIXEL_ALPHA | G2D_BLT_ROTATE270);
    stretchBltTest (G2D_BLT_PIXEL_ALPHA | G2D_BLT_MIRROR45);
    stretchBltTest (G2D_BLT_PIXEL_ALPHA | G2D_BLT_MIRROR135);
}

