/**
 * @file aviExample.c
 * @brief Файл примеров использования библиотеки avilib.
 * @details Воспроизведение \b AVI файла через оверлей.
 * @date Created on: 9 Jan 2023
 */

#include <multex.h>
#include <tasklib.h>
#include <multimedia/avilib.h>
#include <multimedia/a20graph.h>
#include <multimedia/mpeg4codec.h>

static AVI_HANDLE aviFile;
static int frameCount, currentFrameNumber;
static int mpegDecoder = 0;
static unsigned char* pInptBuf;
static unsigned char* pOutFrame = 0;
static unsigned int videoBuff [3];


// Запуск / остановка теста
static bool continueViewing = false;


static int aviPlayerOpen (const char *fileName, VideoModes screenVideoMode, int screenWidth, int screenHeight) {

    int videoWidth, videoHeight;

    // Открыть AVI файл на чтение
    /// [avi overlay open]
    aviFile = openAVIFile (fileName, &videoWidth, &videoHeight, &frameCount);
    /// [avi overlay open]
    if (aviFile == 0) {
        return ERROR;
    }

    // Выбор видео режима - HDMI
    if (!getLFB ())
        /// [avi overlay videmode]
        setVideoMode (screenVideoMode, 32);
        /// [avi overlay videmode]

    // Инициализация декодера MP4
    if (mpegDecoder) {
        // В случае повторного вызова - освободить память
        freeMpeg4FrameMem (mpegDecoder);
        mpegDecoder = 0;
    }
    /// [avi overlay decoder]
    mpegDecoder = mpeg4InitDecoder (videoWidth, videoHeight);
    pInptBuf = getMpeg4InptFrameBuff (mpegDecoder);
    pOutFrame = (unsigned char*) getMpeg4OutFrame (mpegDecoder);
    /// [avi overlay decoder]

    /// [avi overlay start]
    // Настройка области оверлей, запуск вывода на экран.
    int lSizeM = ALIGN (videoWidth, 32) * ALIGN (videoHeight, 32);
    videoBuff[0] = (int) pOutFrame;
    videoBuff[1] = videoBuff[0] + lSizeM;
    videoBuff[2] = videoBuff[1] + lSizeM / 4;

    OverlayInit ();
    // Поднять приоритет вывода - выводить поверх основного экрана.
    setOverlayPriority (2);
    OverlayOpen ((int*)videoBuff, 0, 0,
                 videoWidth, videoHeight,
                 screenWidth, screenHeight,
                 OT_MB_UV_COMBINED);
    /// [avi overlay start]



    return OK;
}

static int aviPlayerClose () {
    /// [avi overlay close]
    closeAVIFile (aviFile);
    freeMpeg4FrameMem (mpegDecoder);
    OverlayClose ();
    /// [avi overlay close]

    mpegDecoder = 0;
    return OK;
}

static int aviTestTask () {

    aviPlayerOpen ("avi/countdwn.avi", MODE_1920x1080, 1920, 1080);
    
    while (continueViewing) {
        /// [avi overlay readAvi]
        int readLen;
        while ((readLen = readAVIFrame (aviFile, (char*)pInptBuf)) == -1) {
            seekToFirstVideoFrame (aviFile);
        }
        /// [avi overlay readAvi]

        currentFrameNumber++;
        // printf ("decode frame - %04d/%04d\n", currentFrameNumber, frameCount);

        // pOutData = NULL
        // tiled кадр идёт сразу в overlay, дополнительное преобразование не нужно.
        /// [avi overlay decode]
        if (mpeg4DecodeBlock (mpegDecoder, NULL, readLen) != OK) {
            printf ("decode error\n");
            return ERROR;
        }
        /// [avi overlay decode]
        
    }
    aviPlayerClose ();
    return OK;
}

void aviTest () {
    if (continueViewing) {
        return;
    }
    continueViewing = true;
    taskSpawn ("aviTest", 10, 0, 0, (FUNCPTR)aviTestTask, 0);
}

void aviTestStop () {
    continueViewing = false;
}
