/**
    @addtogroup groupDrivers
    @{
    @file gpio.h
    @brief Порты ввода/вывода (GPIO).
    @details Настройка портов \b GPIO, подключение линий ввода/вывода
    к аппаратным модулям процессора.

    <b>Подключение</b>: @verbatim #include <gpio.h> @endverbatim
    
    @see Общее описание работы с портами ввода/вывода в главе \ref sectionGpio.
    @}

    @page pageDrivers

    @section sectionGpio GPIO -- Порты ввода/вывода
    @see Описание методов работы с портами ввода/вывода в файле gpio.h.

    Для работы с портами общего назначения как с простыми линиями ввода/вывода
    их следует сконфигурировать --- настроить мультиплексор используемой линии,
    подключив её к регистру чтения или записи. Такая настройка выполняется с
    помощью функций gpio_direction_input() и gpio_direction_output(). После
    настройки уровень сигнала на линии может быть считан с помощью
    gpio_get_value() или выставлен с помощью gpio_set_value(). Линии,
    сконфигурированные как входные, могут быть \em подтянуты к нулю или плюсу
    питания с помощью функций gpio_pull_up() и gpio_pull_down(). Выбор
    конфигурируемой линии осуществляется как сумма имени порта из набора \ref
    ancGpioNames "макросов" и номера линии.

    В следующем примере \b PE1 настраивается как выход с установкой высокого
    уровня на линии. Далее уровень изменяется на низкий.

    @code
    gpio_direction_output (P_E+1, 1);
    // ...
    gpio_set_value (P_E+1, 0);
    @endcode

    Пример настройки \b PD5 как линии ввода с \em подтяжкой к плюсу питания.

    @code
    gpio_direction_input (P_D+5);
    gpio_pull_up (P_D+5);
    @endcode

    Для подключения линий портов ввода/вывода к аппаратным модулям процессора
    используется функция gpio_set_mux(). Эта же функция может использоваться
    для настройки линий как входных и выходных, так как предоставляет
    возможность управления мультиплексором в общем виде. Для настройки
    мультиплексора каждой линии порта используется 4 бита, а следовательно
    линия может иметь до 16 вариантов подключения. Возможные значения
    мультиплексора каждой линии описаны в документации на используемый
    процессор.

    Пример подключения линии \b PB4 к выходу \b PWM0 модуля \b ШИМ.
    В данном примере используется значение мультиплексора \b PWM_MUX,
    определённое как \b 2.

    @code
    gpio_set_mux (P_B+4, PWM_MUX);
    @endcode

 */

#ifndef _GPIO_H_
#define _GPIO_H_

#include <multex.h>
#include <intlib.h>



/**
 *  @anchor ancGpioNames
 *  @name Макросы определения имён портов
 *  @details Для выбора одного из пинов порта следует использовать
 *  сумму имени порта и номера пина. Например <b>P_B+4</b>.
 *  @{
 */

#define P_A (0)         ///< @details Порт ввода/вывода \b A.
#define P_B (32 * 1)    ///< @details Порт ввода/вывода \b B.
#define P_C (32 * 2)    ///< @details Порт ввода/вывода \b C.
#define P_D (32 * 3)    ///< @details Порт ввода/вывода \b D.
#define P_E (32 * 4)    ///< @details Порт ввода/вывода \b E.
#define P_F (32 * 5)    ///< @details Порт ввода/вывода \b F.
#define P_G (32 * 6)    ///< @details Порт ввода/вывода \b G.
#define P_H (32 * 7)    ///< @details Порт ввода/вывода \b H.
#define P_I (32 * 8)    ///< @details Порт ввода/вывода \b I.

/// @}



/**
 *  @name Управление выходным мультиплексором
 *  @{
 */
/**
 *  @brief Сконфигурировать пин порта.
 *  @details Установить значение мультиплексора 
 *  указанного пина выбранного порта для
 *  подключения к аппаратным модулям процессора.
 *  Функция может использоваться так же для настройки
 *  пина как обычного \b GPIO.
 *  
 *  @param gpio	Имя и номер порта
 *  из группы \ref ancGpioNames "макросов" описания имён портов.
 *  @param mux Записываемое значение мультиплексора --- одно из значений, описанных
 *  в документации на используемый процессор.
 *  @return Всегда OK.
 */
int gpio_set_mux (unsigned int gpio, int mux);


/**
 *  @brief Сконфигурировать пин порта как вход.
 *  @details Функция конфигурирует выбранный пин указанного порта
 *  как вход. Функция является обёрткой функции gpio_set_mux()
 *  и записывает \b 0 в мультиплексор пина.
 *
 *  @param gpio	Имя и номер порта
 *  из группы \ref ancGpioNames "макросов" описания имён портов.
 *  @return Всегда OK.
 */
int gpio_direction_input (unsigned gpio);

/**
 *  @brief Сконфигурировать пин порта как выход и выставить заданное значение.
 *  @details Функция конфигурирует выбранный пин указанного порта
 *  как выход. Функция является обёрткой функции gpio_set_mux()
 *  и записывает \b 1 в мультиплексор пина. После чего сразу устанавливает
 *  указанное значение на выходе пина с помощью gpio_set_value().
 *
 *  @param gpio	Имя и номер порта
 *  из группы \ref ancGpioNames "макросов" описания имён портов.
 *  @param value \b 0 устанавливает низкий уровень на выходе,
 *  \b 1 -- высокий.
 *  @return Всегда OK.
 */
int gpio_direction_output (unsigned gpio, int value);
/// @}

/**
 *  @name Управление линией ввода/вывода
 *  @{
 */
/**
 *  @brief Получить значение на линии ввода.
 *  @details Функция возвращает значение, соответствующее уровню сигнала
 *  на входе выбранного пина указанного порта. 
 *  Выбранная линия должна быть сконфигурирована как вход 
 *  с помощью gpio_direction_input().
 *
 *  @param gpio	Имя и номер порта
 *  из группы \ref ancGpioNames "макросов" описания имён портов.
 *  @return \b 0 если на входе присутствует низкий уровень,
 *  \b 1 -- если высокий.
 */
int gpio_get_value (unsigned gpio);

/**
 *  @brief Выставить значение на линии вывода.
 *  @details Функция позволяет установить заданное значение
 *  на выходе выбранного пина указанного порта. Выбранная линия
 *  должна быть сконфигурирована как выход с помощью gpio_direction_output().
 *
 *  @param gpio	Имя и номер порта
 *  из группы \ref ancGpioNames "макросов" описания имён портов.
 *  @param value \b 0 устанавливает низкий уровень на выходе линии,
 *  \b 1 --- высокий уровень.
 *  @return Всегда OK.
 */
int gpio_set_value (unsigned gpio, int value);
/// @}

/**
 *  @name Настройка подтягивающих резисторов
 *  @{
 */
/**
 *  @brief Подтянуть линию к плюсу питания.
 *  @details Функция \em подтягивает выбранную линию заданного порта к 
 *  плюсу питания. 
 *  Выбранная линия должна быть сконфигурирована как вход 
 *  с помощью gpio_direction_input().
 * 
 *  @param gpio	Имя и номер порта
 *  из группы \ref ancGpioNames "макросов" описания имён портов.
 *  @return Всегда OK.
 */
int gpio_pull_up (unsigned gpio);

/**
 *  @brief Подтянуть линию к минусу питания.
 *  @details Функция \em подтягивает выбранную линию заданного порта к 
 *  минусу питания. 
 *  Выбранная линия должна быть сконфигурирована как вход 
 *  с помощью gpio_direction_input().
 * 
 *  @param gpio	Имя и номер порта
 *  из группы \ref ancGpioNames "макросов" описания имён портов.
 *  @return Всегда OK.
 */
int gpio_pull_down (unsigned gpio);

/**
 *  @brief Отпустить линию.
 *  @details Функция отключает \em подтягивающие резисторы от выбранной линии
 *  заданного порта. 
 *  Выбранная линия должна быть сконфигурирована как вход 
 *  с помощью gpio_direction_input().
 * 
 *  @param gpio	Имя и номер порта
 *  из группы \ref ancGpioNames "макросов" описания имён портов.
 *  @return Всегда OK.
 */
int gpio_pull_disable (unsigned gpio);
/// @}



/**
 *  @name Настройка внешних прерываний
 *  @{
 */
typedef enum {
    eintPositiveEdge = 0x0,  ///< Внешнее прерывание сработает по переднему фронту импульса.
    eintNegativeEdge = 0x1,  ///< Внешнее прерывание сработает по заднему  фронту импульса.
    eintHighLevel    = 0x2,  ///< Внешнее прерывание сработает по высокому уровню сигнала.
    eintLowLevel     = 0x3,  ///< Внешнее прерывание сработает по низкому уровню сигнала.
    eintDoubleEdge   = 0x4   ///< Внешнее прерывание сработает по обоим фронтам сигнала.
} eGpioInterruptMode;


/**
 *  @brief Подключение обработчика прерывания к линии \b GPIO.
 *  @details Функция подключает процедуру пользователя
 *  в качестве обработчика аппаратного прерывания
 *  линии ввода/вывода процессора с помощью функции interruptConnect().
 *  В драйвере реализовано раздельное подключение процедур пользователя
 *  на каждую линию (аппаратно для всех линий вызывается один и тот же обработчик драйвера).
 *  
 *  @param gpio	Имя и номер порта
 *  из группы \ref ancGpioNames "макросов" описания имён портов.
 *  @param mode Режим генерации прерывания --- выбор фронта сигнала, 
 *  по которому будет сгенерировано прерывание.
 *  @param group Группа, к которой относится прерывание. Рекомендуется выбирать значение
 *  из макросов соответствующей \ref ancIrqGroupPriority "группы".
 *  @param priority Приоритет прерывания внутри группы.  Рекомендуется выбирать значение
 *  из макросов соответствующей \ref ancIrqGroupPriority "группы".
 *  @param routine Процедура, которую требуется подключить
 *  в качестве обработчика.
 *  @param parameter Параметр пользовательской процедуры обработчика прерывания.
 *  @return \ref OK при успешном выполнении, иначе \ref ERROR.
 */
STATUS gpioInterruptConnect (
    unsigned int gpio,
    eGpioInterruptMode mode,
    int group, int priority,
    usr_int_proc routine, int parameter);

/**
 *  @brief Отключение обработки прерывания для заданной линии.
 *  @details Отключается только обработка данной линии в контроллере
 *  внешних прерываний. Само прерывание \b IRQ остаётся активным, так как 
 *  в контроллере к нему могут быть подключены другие линии.
 *  
 *  @param gpio	Имя и номер порта
 *  из группы \ref ancGpioNames "макросов" описания имён портов.
 *  @return \ref OK при успешном выполнении, иначе \ref ERROR.
 */
STATUS gpioInterruptDisconnect (unsigned int gpio);


/// @}


/**
 *  @name Прочее
 *  @{
 */

/**
 *  @brief Получить номер \b GPIO из строки.
 *  @details Функция получает номер \b GPIO из 
 *  строк, содержащих имя порта и номер пина.
 *  Стока может быть записана без пробелов, как
 *  в документации на процессор, либо в стиле @osName:
 *  @code
 *  bool ok;
 *  gpio = gpio_from_string ("PG1", &ok);
 *  gpio = gpio_from_string ("P_G + 1", &ok);
 *  @endcode
 *  
 *  @param str Строка содержащая имя линии \b GPIO.
 *  @param ok Проверка успешного преобразования. Если в строке
 *  не обнаружено название лини -- возвращает \ref false.
 *  @return Номер \b GPIO.
 */
unsigned gpio_from_string (const char *str, bool *ok);

/// @}


/// @privatesection


/**
 *  @brief Инициализация модуля.
 *  @details Считывание информации об используемом
 *  процессоре. Функция используется системой --- пользователю
 *  вызывать не нужно.
 *  
 *  @return \ref OK в случае успешного выполнения, иначе \ref ERROR.
 */
STATUS gpioArchInit ();

#endif /* _ASM_GENERIC_GPIO_H_ */
