/**
    @addtogroup groupCore 
    @{
    @file inifiles.h
    @brief Методы работы с ini-файлами.
    @details Файл содержит описание методов работы с <b>ini</b>-файлами в ОС @osName.
    @see Общее описание работы с <b>ini</b>-файлами в главе \ref sectionIniFiles.
    @}

    @page pageCore

    @section sectionIniFiles Работа с ini–файлами
    @see Описание методов работы с <b>ini</b>-файлами в файле inifiles.h.
    
    Часто пользователю приходится конфигурировать конкретную
    систему, целевое \b ПО, работающее в среде @osName, производя его настройку с
    помощью каких-либо переменных. Эти переменные определяют общее поведение
    системы. Их значение не должно теряться при отключении питания, а напротив
    — при включении питания и запуске системы конфигурационные переменные
    должны принимать те значения, которые были заданы при конфигурировании. С
    другой стороны, при проведении программных настроек пользователь должен
    иметь возможность изменять значения каких-либо конфигурационных переменных
    таким образом, чтобы после повторных включений системы вновь присвоенные
    значения не терялись. Для этих целей в @osName разработан единый механизм
    хранения (и модификации при необходимости) данных в специальных ini–файлах,
     по аналогии с тем, как это было сделано в ранних версиях \b ОС
    \b Windows. <b>Ini</b>-файл - это обыкновенный текстовый файл, который может быть
    подготовлен как средствами @osName, так и обыкновенным текстовым редактором,
    а затем записан на диск, доступный операционной системе. Для работы с такими файлами
    разработана специальная библиотека, входящая в состав ядра @osName.
    
    Эта библиотека позволяет пользователю отыскивать в <b>ini</b>-файле значения
    переменных по их именам. При этом значения могут представляться как в виде
    текстовых строк, так и в десятичном виде, либо в виде булевых значений
    (типа да-нет). Кроме этого, ряд функций позволяет записывать новые значения
    в этот файл по имени переменной. После закрытия файла на диске @osName будет
    создан новый <b>ini</b>-файл со всеми внесенными изменениями и дополнениями.
    Старый <b>ini</b>-файл будет при этом удален.
    
    В <b>ini</b>-файле переменные сгруппированы в секции. Каждая секция также имеет
    уникальное имя. Для того, чтобы отыскать, или записать требуемое значение,
    необходимо задать имя секции и имя переменной. При чтении также задается
    значение по умолчанию, а при записи — то значение, которое требуется
    присвоить.

 */


#ifndef INIFILES_H_
#define INIFILES_H_

#ifndef _WIN32
#include "multex.h"
#else
#include "\winMltx.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

/// @privatesection

typedef struct if_item
{
    char *Name;
    char *Value;
    struct if_item *Next;
} IF_ITEM;
typedef struct if_section
{
    char *Name;
    IF_ITEM *Items, *LastItem;
    struct if_section *Next;
} IF_SECTION;

/**
 * Родительская структура <b>ini</b>-файла.
 */
typedef struct
{
    char *Name;
    IF_SECTION *Sections, *LastSection;
    IF_SECTION *WS; // inter used
    int FMod; //FileWasModified
} T_INI_FILE;

/// @publicsection

/**
 *  Указатель на структуру <b>ini</b>-файла.
 */
typedef T_INI_FILE *INI_FILE;

//! @brief Возвраты некоторых функций чтения значений из файла.
//! @details Возвраты некоторых функций чтения значений из файла.
//! @private
typedef enum
{
    IRR_Ok,      //!< Значение было успешно считано и установлено.
    IRR_DefValue,//!< Значение не было найдено/было некорректно и было установлено значение по-умолчанию.
    IRR_Error,   //!< Произошла ошибка, значение не было установлено.
}eIniReadReturn;
			 
//---------------------------------------------------
/**
 *  @name Работа с ini-файлом
 *  @{
 */

/**
 *  @brief Создать новый идентификатор <b>ini</b>-файла.
 *  @details Создать идентификатор <b>ini</b>-файла,
 *  но не открывать файл и не заполнять идентификатор данными.
 *  @param ifName Указатель на строку имени <b>ini</b>-файла.
 *  @return Указатель, который следует использовать как идентификатор <b>ini</b>-файла.
 *  @warning Даже при отсутствии файла лучше использовать функцию iniFileOpen().
 */
INI_FILE iniFileCreate(const char *ifName);

/**
 *  @brief Открыть и считать <b>ini</b>-файл.
 *  @details Функция открывает имеющийся (либо создает новый) <b>ini</b>-файл.
 *  @param ifName Указатель на строку имени <b>ini</b>-файла.
 *  @return Указатель, который следует использовать как идентификатор <b>ini</b>-файла.
 *  @warning Сам файл не держится открытым, он закрывается после прочтения.
 */
INI_FILE iniFileOpen(const char *ifName);

/**
 *  @brief Сохранить все изменения в <b>ini</b>-файле.
 *  @details Сохранить все изменения в <b>ini</b>-файле.
 *  Файл будет сохранён в том же режиме обфускации, в котором он был открыт.
 *  @param IF Идентификатор <b>ini</b>-файла.
 *  @return \ref OK при успехе.
 *  @return \ref ERROR при ошибках записи и т.п.
 */
STATUS iniFileFlush(INI_FILE IF);

/**
 *  @brief Изменить имя <b>ini</b>-файла.
 *  @details Функция изменяет имя <b>ini</b>-файла. При закрытии он будет сохранен с новым именем.
 *  Старый файл при этом не будет удален и останется со своим изначальным именем и содержимым.
 *  @param IF Указатель на открытый <b>ini</b>-файла.
 *  @param newName Указатель на строку нового имени файла.
 *  @return \ref OK при успехе.
 *  @return \ref ERROR в случае ошибки.
 */
STATUS iniFileChangeName(INI_FILE IF, const char *newName);

/**
 *  @brief Закрыть <b>ini</b>-файл с сохранением изменений.
 *  @details Функция закрывает <b>ini</b>-файл,
 *  освобождает все занятые ресурсы и сохраняет все изменения.
 *  @param IF Указатель на открытый <b>ini</b>-файл.
 *  @return \ref OK при успехе.
 *  @return \ref ERROR в случае ошибки.
 */
STATUS iniFileClose(INI_FILE IF);


/**
 *  @brief Закрыть <b>ini</b>-файл без сохранения изменений.
 *  @details Функция закрывает файл, освобождает все ресурсы, при этом ничего не сохраняет на диск.
 *  @param IF Указатель на открытый <b>ini</b>-файл.
 */
void iniFileFree(INI_FILE IF);

/**
 * @brief Печать файла.
 * @details Распечатать <b>ini</b>-файл в \b stdout.
 * @param IF Идентификатор <b>ini</b>-файла.
 * @return OK при успехе, ERROR иначе.
 */
STATUS iniFilePrint(INI_FILE IF);
/// @}

//---------------------------------------------------


/**
 *  @name Общие функции работы с содержимым файла
 *  @{
 */

/**
 *  @brief Проверка наличия параметра.
 *  @details Функция проверять наличие параметра в <b>ini</b>-файле.
 *  
 *  @param IF Идентификатор <b>ini</b>-файла.
 *  @param Section Указатель на имя секции.
 *  @param Name Указатель на имя переменной.
 *  @return \ref true, если параметр содержится в файле, иначе \ref false.
 */
bool iniFileCheckIfItemExists (INI_FILE IF, const char *Section, const char *Name);

/**
 * @brief Проверка наличия секции в <b>ini</b>-файле.
 * @param iniFileDesc Идентификатор <b>ini</b>-файла.
 * @param sectionName Название секции, в которой расположен параметр.
 * @return \ref true если секция существует, иначе \ref false.
 */
bool iniFileCheckIfSectionExists (INI_FILE iniFileDesc, const char *sectionName);

/**
 *  @brief Поиск параметра по порядковому номеру.
 *  @details Функция позволяет получить имя параметра
 *  с номером \b ItemNum из секции \b Section.
 *  Нумерация начинается с \b 0.
 *  
 *  @param IF Идентификатор <b>ini</b>-файла.
 *  @param Section Указатель на имя секции.
 *  @param ItemNum Порядковый номер параметра.
 *  @return Указатель на строку, содержащую имя параметра.
 */
char *iniFileItemName (INI_FILE IF, const char *Section, int ItemNum);

/**
 *  @brief Поиск секции по порядковому номеру.
 *  @details Функция позволяет получить имя секции
 *  с номером \b SectionNum в структуре \ref INI_FILE.
 *  Нумерация секций начинается с \b 0.
 *  
 *  @param IF Идентификатор <b>ini</b>-файла.
 *  @param SectionNum Порядковый номер секции.
 *  @return Указатель на строку, содержащую имя параметра.
 */
char *iniFileSectionName (INI_FILE IF, int SectionNum);


/**
 * Переименовать секцию в ини-файле.
 * @param iniFileDesc Идентификатор <b>ini</b>-файла.
 * @param oldSectionName Старое название секции.
 * @param newSectionName Новое название секции.
 * @return \ref OK, если секция была переименована, \ref ERROR если секция не была найдена, если секция с newSectionName уже существует или в случае других ошибок.
 */
STATUS iniFileRenameSection(INI_FILE iniFileDesc, const char* oldSectionName, const char* newSectionName);

/**
 * Переименовать параметр в ини-файле.
 * @param iniFileDesc Идентификатор <b>ini</b>-файла.
 * @param sectionName Название секции, в которой расположен параметр.
 * @param oldItemName Старое название параметра.
 * @param newItemName Новое название параметра.
 * @return \ref OK, если параметр был переименована, \ref ERROR если параметр не был найден, если секция с newSectionName уже существует или в случае других ошибок.
 */
STATUS iniFileRenameItem(INI_FILE iniFileDesc, const char* sectionName, const char* oldItemName, const char* newItemName);

/**
 * Удалить секцию и все её параметры из ини-файла.
 * @param iniFileDesc Идентификатор <b>ini</b>-файла.
 * @param sectionName Название секции.
 * @return \ref OK, если секция была успешно удалена, \ref ERROR иначе.
 */
STATUS iniFileDeleteSection(INI_FILE iniFileDesc, const char* sectionName);

/**
 * Удалить параметр из ини-файла.
 * @param iniFileDesc Идентификатор <b>ini</b>-файла.
 * @param sectionName Название секции, в которой расположен параметр.
 * @param itemName Название параметра.
 * @return \ref OK, если параметр был успешно удален, \ref ERROR иначе.
 */
STATUS iniFileDeleteItem(INI_FILE iniFileDesc, const char* sectionName, const char* itemName);


/// @}

//----------------------------------------------------

/**
 *  @name Целые числа
 *  @{
 */

/// @private
eIniReadReturn iniFileReadInteger_Extended (INI_FILE IF, const char *Section, const char *Name, int *buffer, int defVal);

/**
 *  @brief Считать целое число из <b>ini</b>-файла.
 *  @details Функция считывает из <b>ini</b>-файла значение переменной как целое число.
 *  
 *  @param IF Указатель на открытый <b>ini</b>-файл.
 *  @param Section Указатель на имя секции.
 *  @param Name Указатель на имя переменной.
 *  @param defVal Значение, возвращаемое при неудачном поиске.
 *  @return Значение указанной переменной, либо \b defVal, если переменную найти не удалось.
 */
int iniFileReadInteger (INI_FILE IF, const char *Section, const char *Name, int defVal);

/**
 *  @brief Записать целое число в <b>ini</b>-файл.
 *  @details Функция записывает в <b>ini</b>-файл значение переменной как целое число.
 *  
 *  @param IF Указатель на открытый <b>ini</b>-файл.
 *  @param Section Указатель на имя секции.
 *  @param Name Указатель на имя переменной.
 *  @param Val Значение, которое требуется записать.
 */
void iniFileWriteInteger(INI_FILE IF,const char *Section,const char *Name,int Val);
/// @}

//----------------------------------------------------
/**
 *  @name Целые числа в hex-формате
 *  @{
 */

/**
 *  @brief Считать целое число в формате \b HEX из <b>ini</b>-файла.
 *  @details Функция считывает из <b>ini</b>-файла значение переменной
 *  как целое число в формате \b HEX.
 *  
 *  @param IF Указатель на открытый <b>ini</b>-файл.
 *  @param Section Указатель на имя секции.
 *  @param Name Указатель на имя переменной.
 *  @param defVal Значение, возвращаемое при неудачном поиске.
 *  @return Значение указанной переменной, либо \b defVal,
 *  если переменную найти не удалось.
 */
int iniFileReadHex(INI_FILE IF,const char *Section,const char *Name,int defVal);

/**
 *  @brief Записать целое число в формате \b HEX в <b>ini</b>-файл.
 *  @details Функция записывает в <b>ini</b>-файл значение переменной
 *  как целое число в формате \b HEX.
 *  
 *  @param IF Указатель на открытый <b>ini</b>-файл.
 *  @param Section Указатель на имя секции.
 *  @param Name Указатель на имя переменной.
 *  @param Val Значение, которое требуется записать.
 */
void iniFileWriteHex(INI_FILE IF,const char *Section,const char *Name,int Val);

/// @private
eIniReadReturn iniFileReadHex_Extended(INI_FILE IF, const char *Section, const char *Name, int *buffer, int defVal);
/// @}

//----------------------------------------------------
/**
 *  @name Логическое значение
 *  @{
 */

/**
 *  @brief Считать логическое значение из <b>ini</b>-файла.
 *  @details Функция считывает из <b>ini</b>-файла значение переменной
 *  как логическое значение.
 *  
 *  @param IF Указатель на открытый <b>ini</b>-файл.
 *  @param Section Указатель на имя секции.
 *  @param Name Указатель на имя переменной.
 *  @param defVal Значение, возвращаемое при неудачном поиске.
 *  @return Значение указанной переменной, либо \b defVal,
 *  если переменную найти не удалось.
 */
int iniFileReadBoolean(INI_FILE IF,const char *Section,const char *Name,int defVal);

/**
 *  @brief Записать логическое значение в <b>ini</b>-файл.
 *  @details Функция записывает в <b>ini</b>-файл значение переменной как логическое.
 *  
 *  @param IF Указатель на открытый <b>ini</b>-файл.
 *  @param Section Указатель на имя секции.
 *  @param Name Указатель на имя переменной.
 *  @param Val Значение, которое требуется записать.
 */
void iniFileWriteBoolean(INI_FILE IF,const char *Section,const char *Name,int Val);

/// @private
eIniReadReturn iniFileReadBoolean_Extended(INI_FILE IF, const char *Section, const char *Name, bool *buffer, bool defVal);
/// @}

//---------------------------------------------------
/**
 *  @name Строки
 *  @{
 */

/**
 *  @brief Считать текстовую строку из <b>ini</b>-файла.
 *  @details Функция считывает из <b>ini</b>-файла значение переменной
 *  как текстовую строку. Значение по указателю должно быть использовано/скопировано
 *  ДО закрытия <b>ini</b>-файла.
 *  @warning Необходимо освободить полученную строку после использования.
 * 
 *  @param IF Указатель на открытый <b>ini</b>-файл.
 *  @param Section Указатель на имя секции.
 *  @param Name Указатель на имя переменной.
 *  @param defVal Указатель на строку, возвращаемую при неудачном поиске.
 *  @return Указатель на строку символов, либо на \b defVal,
 *  если переменную найти не удалось.
 */
char *iniFileReadString (INI_FILE IF, const char *Section, const char *Name, const char *defVal);

/**
 *  @brief Считать текстовую строку из <b>ini</b>-файла.
 *  @details Функция аналогична iniFileReadString ().
 *  
 *  @return Константный указатель на строку символов, либо на \b defVal,
 *  если переменную найти не удалось.
 */
const char *iniFileReadConstString (INI_FILE IF, const char *Section, const char *Name, const char *defVal);

/**
 *  @brief Записать строку в <b>ini</b>-файл.
 *  @details Функция записывает в <b>ini</b>-файл значение переменной
 *  как строку символов.
 *  
 *  @param IF Указатель на открытый <b>ini</b>-файл.
 *  @param Section Указатель на имя секции.
 *  @param Name Указатель на имя переменной.
 *  @param Val Указатель на строку, которую требуется записать.
 */
void iniFileWriteString (INI_FILE IF, const char *Section, const char *Name, const char *Val);

/// @private
eIniReadReturn iniFileReadString_Extended (INI_FILE IF, const char *Section, const char *Name, char **buffer, const char *defVal);  //Нужно будет вызвать free() для буфера

/// @}

//---------------------------------------------------

/**
 *  @name Прямоугольные области
 *  @{
 */


/**
 *  Структура хранения размеров прямоугольной
 */
typedef struct {
 int x; //!< @details X-координата.
 int y; //!< @details Y-координата.
 int w; //!< @details Ширина прямоугольника.
 int h; //!< @details Высота прямоугольника.
} iniRect;

/**
 *  @brief Считать данные о прямоугольной области.
 *  @details Функция получает данные о прямоугольной области из <b>ini</b>-файла.
 *  
 *  @param iniFile Указатель на открытый <b>ini</b>-файл.
 *  @param section Указатель на имя секции.
 *  @param name Указатель на имя переменной.
 *  @param defVal Указатель на структуру, возвращаемую при неудачном поиске.
 *  @return Указатель на структуру полученных данных \ref iniRect.
 *  @warning Необходимо освободить полученную структуру после использования!
 */
iniRect *iniFileReadTextRect (INI_FILE iniFile, const char *section, const char *name, const iniRect *defVal);

/**
 *  @brief Записать данные о прямоугольной области.
 *  @details Функция записывает данные о прямоугольной
 *  области в <b>ini</b>-файл.
 *  
 *  @param iniFile Указатель на открытый <b>ini</b>-файл.
 *  @param section Указатель на имя секции.
 *  @param name Указатель на имя переменной.
 *  @param val Указатель на записываемые данные.
 */
void iniFileWriteTextRect (INI_FILE iniFile, const char *section, const char *name, const iniRect *val);

/// @private
eIniReadReturn iniFileReadTextRect_Extended (INI_FILE IF, const char *Section, const char *Name, iniRect *buffer, const iniRect *defVal);
/// @}

//---------------------------------------------------
/**
 *  @name Пары координат
 *  @{
 */

/**
 *  Структура данных для сохранения пары координат.
 */
typedef struct {
 int x; //!< @details X-координата
 int y; //!< @details Y-координата
} iniCoords;

/**
 *  @brief Считать пару координат.
 *  @details Функция считывает пару координат из <b>ini</b>-файла.
 *  
 *  @param iniFile Указатель на открытый <b>ini</b>-файл.
 *  @param section Указатель на имя секции.
 *  @param name Указатель на имя переменной.
 *  @param defVal Указатель на структуру, возвращаемую при неудачном поиске.
 *  @return Указатель на структуру полученных данных \ref iniCoords.
 *  @warning Необходимо освободить полученную структуру после использования!
 */
iniCoords* iniFileReadCoords(INI_FILE iniFile, const char *section, const char *name, const iniCoords *defVal);

/**
 *  @brief Записать пару координат.
 *  @details Функция записывает пару координат в <b>ini</b>-файл.
 *  
 *  @param iniFile Указатель на открытый <b>ini</b>-файл.
 *  @param section Указатель на имя секции.
 *  @param name Указатель на имя переменной.
 *  @param val Указатель на записываемые данные.
 */
void iniFileWriteCoords(INI_FILE iniFile, const char *section, const char *name, const iniCoords *val);

/// @private
eIniReadReturn iniFileReadCoords_Extended(INI_FILE IF, const char *Section, const char *Name, iniCoords *buffer, const iniCoords *defVal);
/// @}

//---------------------------------------------------

/**
 *  @name Массивы целых чисел
 *  @{
 */

/**
 *  @brief Структура для сохранения массива целых чисел в <b>ini</b>-файле.
 *  @details Массивы целых чисел сохраняются с указанием длины массива.
 *  @note Данная структура должна освобождаться через два free() - для поля \b Array и для самой структуры. *
 */
typedef struct{
    int ArrayLength;    ///< @details Длина массива.
    int* Array;         ///< @details Указатель на данные.
}iniIntArray;

//Нужно освободить полученную структуру после использования!

/**
 *  @brief Считать массив целых чисел.
 *  @details Функция считывает массив целых чисел из <b>ini</b>-файла.
 *  
 *  @param iniFile Указатель на открытый <b>ini</b>-файл.
 *  @param section Указатель на имя секции.
 *  @param name Указатель на имя переменной.
 *  @param defVal Указатель на структуру, возвращаемую при неудачном поиске.
 *  @return Указатель на структуру полученных данных \ref iniIntArray.
 *  @warning Необходимо освободить полученную структуру после использования!
 */
iniIntArray* iniFileReadIntArray(INI_FILE iniFile, const char *section, const char *name, const iniIntArray *defVal);

/**
 *  @brief Записать массив целых чисел.
 *  @details Функция записывает массив целых чисел в <b>ini</b>-файл.
 *  
 *  @param iniFile Указатель на открытый <b>ini</b>-файл.
 *  @param section Указатель на имя секции.
 *  @param name Указатель на имя переменной.
 *  @param val Указатель на записываемые данные.
 */
void iniFileWriteIntArray(INI_FILE iniFile, const char *section, const char *name, const iniIntArray *val);

/// @private
eIniReadReturn iniFileReadIntArray_Extended(INI_FILE IF, const char *Section, const char *Name, iniIntArray **buffer, const iniIntArray *defVal); //Нужно будет вызвать free() для поля Array и для буфера
/// @}

//---------------------------------------------------
// Binary array
/**
 *  @name Массивы бинарных данных.
 *  @brief Формат данных в <b>ini</b>-файле: #5:AB78223109
 *  @{
 */

/**
 *  @brief Структура для сохранения массива бинарных данных в <b>ini</b>-файле.
 *  @details сохраняются с указанием длины массива.
 *  @note Данная структура должна освобождаться через два free() - для поля \b Array и для самой структуры.
 */
typedef struct{
    size_t ArrayLength;
    uint8_t* Array;
}iniBinaryArray;

/**
 *  @brief Считать массив бинарных данных.
 *  @details Функция считывает массив бинарных данных из <b>ini</b>-файла.
 *
 *  @param iniFile Указатель на открытый <b>ini</b>-файл.
 *  @param section Указатель на имя секции.
 *  @param name Указатель на имя переменной.
 *  @param defVal Указатель на структуру, возвращаемую при неудачном поиске.
 *  @return Указатель на структуру полученных данных \ref iniBinaryArray.
 *  @note Необходимо освободить полученную структуру после использования!
 */
iniBinaryArray* iniFileReadBinaryArray(INI_FILE iniFile, const char *section, const char *name, const iniBinaryArray *defVal);

/**
 *  @brief Записать массив бинарных данных.
 *  @details Функция записывает массив бинарных данных в <b>ini</b>-файл.
 *
 *  @param iniFile Указатель на открытый <b>ini</b>-файл.
 *  @param section Указатель на имя секции.
 *  @param name Указатель на имя переменной.
 *  @param val Указатель на записываемые данные.
 */
void iniFileWriteBinaryArray(INI_FILE iniFile, const char *section, const char *name, const iniBinaryArray *val);

/// @}

/// @private
eIniReadReturn iniFileReadBinaryArray_Extended(INI_FILE IF, const char *Section, const char *Name, iniBinaryArray **buffer, const iniBinaryArray *defVal);



#ifdef __cplusplus
}
#endif

#endif // ifndef INIFILES_H_

