/**
 *  @addtogroup groupStdTypes
 *  @{
 * @file limits.h
 * @brief Характеристики общих типов.
 * @details См. стандарт C11 7.10.
 * @see <a href="https://www.iso.org/standard/57853.html">C11 standard 7.10</a>.
 *  @}
 */

/*
 * @author Sergey Sokolov
 * @author Dubovich Timofey
*/
#ifndef LIMITS_H
#define LIMITS_H

/**
 * Количество бит в типе \b char.
 */
#define CHAR_BIT (8)

/**
 *  @name signed char
 *  @brief Минимальное и максимальное значения типа:
 *  @{
 */
#define SCHAR_MIN   (-128)  // -(2^7 - 1) - 1
#define SCHAR_MAX   (127)   // 2^7 - 1
/// @}

/**
 *  @name unsigned char
 *  @brief Максимальное значение типа (минимальное - 0):
 *  @{
 */
#define UCHAR_MAX   (255) // 2^8 - 1
/// @}

/**
 *  @name char
 *  @brief Минимальное и максимальное значения типа:
 *  @{
 */
#define CHAR_MIN   0
#define CHAR_MAX   UCHAR_MAX
/// @}

/**
 *  @name signed short int
 *  @brief Минимальное и максимальное значения типа:
 *  @{
 */
#define SHRT_MAX   (32767)  // 2^15 - 1
#define SHRT_MIN   (-32768) // -(2^15 - 1) - 1
/// @}

/**
 *  @name unsigned short int
 *  @brief Максимальное значение типа (минимальное - 0):
 *  @{
 */
#define USHRT_MAX   (65535) // 2^16 - 1
/// @}

/**
 *  @name signed int.
 *  @brief Минимальное и максимальное значения типа:
 *  @{
 */
#define INT_MAX   (2147483647)   // 2^31 - 1
#define INT_MIN   (-INT_MAX - 1) // -(2^31 - 1) - 1
/// @}

/**
 *  @name unsigned int
 *  @brief Максимальное значение типа (минимальное - 0):
 *  @{
 */
#define UINT_MAX   4294967295U // 2^32 - 1
/// @}

/**
 *  @name signed long int
 *  @brief Минимальное и максимальное значения типа:
 *  @{
 */
#define LONG_MAX   (2147483647L)    // 2^31 - 1
#define LONG_MIN   (-LONG_MAX - 1L) // -(2^31 - 1) - 1
/// @}

/**
 *  @name unsigned long int
 *  @brief Максимальное значение типа (минимальное - 0):
 *  @{
 */
#define ULONG_MAX   4294967295UL // 2^32 - 1
/// @}

/**
 *  @name unsigned long long int.
 *  @brief Минимальное и максимальное значения типа:
 *  @{
 */
#define LLONG_MAX   9223372036854775807LL // 2^63 - 1
#define LLONG_MIN   (-LLONG_MAX - 1LL)    // -(2^63 - 1) - 1
/// @}

/**
 *  @name unsigned long long int
 *  @brief Максимальное значение типа (минимальное - 0):
 *  @{
 */
#define ULLONG_MAX   18446744073709551615ULL // 2^64 - 1
/// @}

/**
 * Максимальная длинна мультибайтного символа во всех возможных локалях.
 */
#define MB_LEN_MAX  6

// Проверка корректности заголовочника.
// #include "assert.h"
// static_assert((sizeof(int) == 4));
#if !defined(__CHAR_UNSIGNED__)
    #error "limits.h is incorrect for target device!"
#endif

#endif
