/**
    @addtogroup groupCore
    @{
    @file mapstr.h
    @brief Список пар типа \b строка-значение.
    @details Разработано специально для описания
    аппаратной части проектов. Под каждую запись выделяется минимально
    возможное место в ОЗУ, кратное блоку в 512 байт. Записи в списке
    находятся по уникальному ключу, заданному при создании записи.
    Если ключи некоторых записей совпадают - будет возвращено значение
    первой найденной записи.
    @}
 */

/*
    @author Матяш А.В. (a.matiash@set-code.ru)
    @date 28.03.2023
*/


#ifndef _MAPSTR_H_
#define _MAPSTR_H_


#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>

/** @details Просто число, отличное от нуля с пустым последним байтом. */
#define MAPSTR_SIGNATURE 0x75AADD00

/**
 *  @brief Типы данных, используемые в качестве значений.
 */
typedef enum {
    mapTypeUnknown = 0,                         ///< Значение не определено.
    mapTypeString   = MAPSTR_SIGNATURE | 1,     ///< Значение - строка. 
    mapTypeInt      = MAPSTR_SIGNATURE | 2,     ///< Значение (массив значений) - целое со знаком.
} eMapstrValueType;


/**
 *  @brief Набор указателей для работы со списками.
 *  @details Набор указателей нужен для добавления
 *  записей и поиска по списку.
 */
typedef struct {
    void *start;    ///< Итератор начала списка.
    void *end;      ///< Итератор конца списка.
} tMapIterators;

/**
 *  @name Создание и удаление списков
 *  @{
 */


/**
 *  @brief Создание нового списка.
 *  @details Создание и инициализация набора итераторов
 *  для нового списка.
 *  
 *  @return tMapIterators* Готовый набор итераторов.
 */
tMapIterators *newMap ();

/**
 *  @brief Удаление списка --- очистка памяти, отведённой под список.
 *  @details Функция освобождает всю память выделенную под список
 *  и его итераторы.
 *  
 *  @param iter Итераторы списка.
 */
void mapFree (tMapIterators *iter);

/// @}


/**
 *  @name Работа со списком
 *  @{
 */
/**
 *  @brief Добавить значение типа \b строка к списку.
 *  @details Функция добавляет строку к имеющемуся списку
 *  о обновляет значение итераторов.
 *  
 *  @param iter Итераторы списка.
 *  @param key Ключ, по которому будет осуществляться поиск значения.
 *  @param value Значение, соответствующее ключу.
 *  @param description Строковое описание параметра может быть использовано
 *  для вывода списка на печать. Если не используется, может быть NULL.
 */
void mapAppendString (tMapIterators *iter,
                      const char *key,
                      const char *value,
                      const char *description);

/**
 *  @brief Добавить значение типа \b целое к списку.
 *  @details Функция добавляет значение к имеющемуся списку
 *  и обновляет значение итераторов.
 *  
 *  @param iter Итераторы списка.
 *  @param key Ключ, по которому будет осуществляться поиск значения.
 *  @param value Значение, соответствующее ключу.
 *  @param description Строковое описание параметра может быть использовано
 *  для вывода списка на печать. Если не используется, может быть NULL.
 */
void mapAppendInt (tMapIterators *iter,
                   const char *key,
                   int value,
                   const char *description);

/**
 *  @brief Добавить массив значений типа \b целое к списку.
 *  @details Функция добавляет массив значений к имеющемуся списку
 *  и обновляет значение итераторов.
 *  
 *  @param iter Итераторы списка.
 *  @param key Ключ, по которому будет осуществляться поиск значения.
 *  @param values Добавляемый массив. 
 *  @param count Количество элементов массива.
 *  @param description Строковое описание параметра может быть использовано
 *  для вывода списка на печать. Если не используется, может быть NULL.
 */
void mapAppendIntArray (tMapIterators *iter,
                        const char *key,
                        int *values,
                        int count,
                        const char *description);

/**
 *  @brief Поиск по списку.
 *  @details Функция осуществляет поиск по ключу в списке
 *  и возвращает найденное значение, либо \b NULL.
 *  
 *  @param[in] iter Итераторы списка.
 *  @param[in] key Ключ, по которому будет осуществляться поиск значения.
 *  @param[out] count Количество элементов, содержащихся в массиве значений.
 *  Для строк это значение всегда равно 1. Если параметр не требуется,
 *  указатель может быть равен NULL.
 *  @param[out] vType Указатель на переменную, в которую будет помещён
 *  тип найденного значения. Если тип указывать не нужно -- следует
 *  передать \b NULL вместо указателя.
 *  @return Указатель на данные. Если тип данных заранее не известен
 *  можно ориентироваться по возвращаемому параметру \b vType.
 */
const void *mapFind (const tMapIterators *iter,
                     const char *key,
                     int *count,
                     eMapstrValueType *vType);


/**
 *  @brief Проверка строкового значения на соответствие заданному значению.
 *  @details Функция осуществляет поиск значения по ключу в списке
 *  и сравнивает найденное с заданным.
 *  
 *  @param iter Итераторы списка.
 *  @param key Ключ, по которому будет осуществляться поиск значения.
 *  @param value Проверяемое значение.
 *  @return true Если значение найдено и соответствует заданному.
 *  @return false Во всех остальных случаях.
 */
bool mapCheckString (const tMapIterators *iter,
                     const char *key, 
                     const char *value);



/**
 *  @brief Печать списка в консоль.
 *  @details Функция выводит в консоль все элементы списка
 *  в две колонки. Может использоваться для отладки.
 *  
 *  @param iter Итераторы списка.
 *  @param header Заголовок выводимый в начале печати.
 */
void mapPrint (const tMapIterators *iter, const char *header);

/// @}


/**
 *  @name Работа с файлами
 *  @brief Списки можно сохранять и читать из файлов.
 *  @{
 */

/**
 *  @brief Сохранить список.
 *  @details Функция сохраняет список в текстовый файл
 *  по указанному пути. Если файл существует он будет
 *  перезаписан.
 *  
 *  @param path Полный путь к файлу.
 *  @param iter Итераторы списка.
 */
void mapStore (const char *path, const tMapIterators *iter);

/**
 *  @brief Прочитать список из файла.
 *  @details Элементы прочитанные из файла будут добавлены к указанному списку.
 *  
 *  @param path Полный путь к файлу.
 *  @param iter Итераторы уже существующего списка. 
 *  \b ВАЖНО! Если список ещё не создан его следует создать с помощью newMap().
 */
void mapRestore (const char *path, tMapIterators *iter);

/**
 *  @brief Найти все возможные файлы конфигурации и прочитать их.
 *  @details Функция ищет файлы с указанным расширением (обычно \b arc)
 *  в корневых каталогах
 *  дисков и пытается извлечь из них списки. Каждый следующий
 *  найденный список будет добавлен в конец имеющегося.
 *  
 *  @param ext Расширение файлов со списками (обычно \b arc).
 *  @param iter Итераторы уже существующего списка. 
 *  \b ВАЖНО! Если список ещё не создан его следует создать с помощью newMap().
 *  
 */
void mapRestorefFromEverywhere (const char *ext, tMapIterators *iter);

/// @}


#ifdef __cplusplus
}
#endif

#endif
