#ifndef _MMC_H_
#define _MMC_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>
#include <iolib.h>


/**
 *  @brief Подключить диск \b MMC к текущей файловой системе.
 *  @details Функция создаёт устройство с помощью createMMCDev()
 *  и подключает к файловой системе. Файловая система уже должна быть
 *  создана, например с помощью dosInit(). Функция вызывается в ядре ОС
 *  при старте для всех дисков, указанных в файле конфигурации.
 *  
 *  @param n Номер физического диска \b MMC из описания на процессор (плату).
 *  @param p Номер монтируемого раздела на диске, начиная с нулевого. 
 *  Для дисков с одним разделом -- \b 0.
 * 
 *  @return \ref OK при удачном подключении диска, иначе \ref ERROR.
 */
STATUS mmcMount (int n, int p);

/**
 *  @brief Поиск имени раздела \b MMC диска в списке подключенных устройств.
 *  @details Функция позволяет найти имя уже подключенного к файловой системе
 *  раздела \b MMC по номеру физического диска и номеру раздела. Функция является
 *  обёрткой функции iosTypedVolumeLabelFind(), предназначенной для поиска именно
 *  диска \b MMC.
 *  
 *  @param n Номер физического диска \b MMC из описания на процессор (плату).
 *  @param p Номер монтируемого раздела на диске, начиная с нулевого. 
 *  Для дисков с одним разделом -- \b 0.
 *  @return Указатель на имя раздела диска (например \b С: или \b D: ) 
 *  если указанный раздел найден, иначе \ref NULL.
 */
const char *mmcLabelGet (int n, int p);


/**
 *  @brief Отключение заданного раздела указанного диска \b MMC.
 *  @details Функция отключает заданный раздел указанного диска и освобождает 
 *  выделенные ресурсы.
 *  
 *  @param n Номер физического диска \b MMC из описания на процессор (плату).
 *  @param p Номер смонтированного раздела на диске, начиная с нулевого. 
 *  Для дисков с одним разделом -- \b 0.
 *  @return \ref OK при успешном отключении, иначе \ref ERROR.
 */
STATUS mmcUmount (int n, int p);



/// @privatesection

// Здесь известен только тип процессора - файлы читать ещё не возможно!
STATUS mmcArchInit ();

/**
 *  @brief Создать дисковое устройство.
 *  @details Функция подключает контроллер диска и создаёт дисковое устройство.
 *  Функция используется системой. Для подключения дисков \b MMC 
 *  используйте mmcMount ().
 *  
 *  @param devNum Номер подключаемого устройства, начиная с 0.
 *  Номер конкретного диска следует брать из описания процессора (платы).
 *  @return Указатель на созданное устройство.
 */
BLK_DEV *createMMCDev (int devNum);


#ifdef __cplusplus
}
#endif

#endif
