/**
    @addtogroup groupCore 
    @{
    @file msgQLib.h
    @brief Создание очередей сообщений.
    @details Очереди сообщений – удобный механизм межзадачного взаимодействия.
    В файле собраны методы работы с очередями сообщений.

    @see Общее описание очередей см. в главе \ref sectionMsgQLib.

    Пример использования очереди сообщений в простой задаче:
    @code
    MSG_Q_ID queue = NULL;
    bool stop;

    // Задача в отдельной функции
    int taskSimple (int dummy) {
        // Инициализация переменных задачи (если есть)
        void *p = malloc (size);

        while (!stop) {
            // Ожидание получения сообщений
            int data;
            msgQReceive (queue, &data, sizeof (int), WAIT_FOREVER);
            // ... тело задачи
            p[0] = data;
            // ...
        }

        // Освобождение выделенной памяти
        free (p);
        // Завершение задачи
        return 0;
    }

    // Создание и запуск задачи
    void testSimple () {
        int size = 10;
        queue = msgQCreate (size, sizeof (int), MSG_Q_FIFO);
        stop = false;
        taskSpawn ("task simple", 10, 0, 0, taskSimple, 0);
    }  

    // Некое внешнее событие
    void testRelease (int data) {
        // Отправить сообщение
        msgQSend (queue, &data, sizeof (int), NO_WAIT, MSG_PRI_NORMAL);
    }
    @endcode

    @}
 */

#ifndef _MSGQLIB
#define _MSGQLIB

#ifdef __cplusplus
extern "C" {
#endif

#include "semlib.h"

/**
 *  @name Порядок получения данных из очереди
 *  @{
 */
/**
 *  @brief Порядок получения данных из очереди \b FIFO.
 *  @details Задачи получают сообщения из очереди в порядке обращения.
 */
#define MSG_Q_FIFO          (0x00)

/**
 *  @brief Порядок получения данных из очереди \b PRIORITY.
 *  @details Задачи получают сообщения из очереди в соответствии с их приоритетами.
 */
#define MSG_Q_PRIORITY      (0x01)
/// @}

/**
 *  @name Приоритеты сообщений в очереди
 *  @{
 */
#define MSG_PRI_NORMAL (0)  ///< @details Нормальный приоритет - сообщения помещаются в конец очереди.
#define MSG_PRI_URGENT (1)  ///< @details Повышенный приоритет - сообщения помещаются в начало очереди.
/// @}


/**
 *  @brief Структура блока управления очереди.
 */
typedef struct 
        {
         int PW_Id;         ///< @details Сигнатура очереди.
         SEM_ID Sem_W;      ///< @details Семафор записи.
         SEM_ID Sem_R;      ///< @details Семафор чтения.
         int count;         ///< @details Максимальное число сообщений.
         int size;          ///< @details Размер сообщений.
         int F;             ///< @details Опции очереди.
         char *p_rd;        ///< @details Указатель чтения.
         char *p_wr;        ///< @details Указатель записи.
         char *first;       ///< @details Указатель на первый элемент.
         char *last;        ///< @details Указатель на последний эл-т.
        } msgQID;


/**
 *  @brief Указатель на идентификатор очереди.
 *  @details Указатель на структуру блока управления (идентификатора) очереди \ref msgQID.
 *  
 */
typedef msgQID *MSG_Q_ID;                
        
/**
 *  @brief Создать очередь сообщений.
 *  @details Функция создает очередь сообщений.
 *  
 *  @param maxMsgs Максимальное число сообщений, буферизуемых очередью.
 *  Если задача будет пытаться поместить в полную очередь сообщение,
 *  она будет задержана до тех пор, пока в очереди не освободится место
 *  для этого сообщения. Впрочем, задача может ждать заданное время или не ждать вовсе.
 *  @param maxMsgLength Максимальный размер одного сообщения. Сообщения могут
 *  иметь и меньший размер, но память под очередь будет выделяться из расчета
 *  \b maxMsgs*maxMsgLength, поэтому рекомендуется задавать эти величины в соответствии
 *  с конкретным применением очереди.
 *  @param options Способ доступа задач к очереди. Возможные значения
 *  \ref MSG_Q_FIFO и \ref MSG_Q_PRIORITY.
 *  @return Идентификатор созданной очереди.
 *  @return \b NULL при неудаче, вызванной нехваткой динамической памяти.
 */
MSG_Q_ID msgQCreate(int maxMsgs, int maxMsgLength, int options); 

/**
 *  @brief Удалить очередь.
 *  @details Функция удаляет очередь и высвобождает занятую под нее динамическую память.
 *  @warning Следует соблюдать осторожность при удалении очереди – в этот
 *  момент отдельные задачи могут ждать сообщений из нее.
 *  
 *  @param msgQId Идентификатор удаляемой очереди.
 *  @return \ref OK, или \ref ERROR при задании неверного идентификатора очереди.
 */
STATUS msgQDelete(MSG_Q_ID msgQId);

/**
 *  @brief Поместить сообщение в очередь.
 *  @details Функция помещает сообщение в очередь. Если имеется задача,
 *  ждущая сообщения из этой очереди, и ее приоритет выше той,
 *  которая поместила сообщение, произойдет немедленное переключение на эту задачу.
 * 
 *  @warning Не следует использовать очереди сообщений в обработчиках прерываний!
 *  В обработчиках следует использовать \ref ringbuffer.h "кольцевой буфер".
 *  
 *  @param msgQId Идентификатор очереди.
 *  @param buffer Указатель на буфер, из которого сообщение будет помещено в очередь.
 *  После вызова функции буфер может быть использован задачей для своих целей –
 *  сообщение копируется во внутренний буфер очереди.
 *  @param nBytes Размер сообщения. Он не должен превышать максимальный размер сообщения,
 *  отводимый при создании очереди
 *  @param timeout Время ожидания в тиках таймера в случае, если очередь переполнена.
 *  Допустимы также значения \ref NO_WAIT и \ref WAIT_FOREVER.
 *  @param priority Приоритет помещаемого сообщения:
 *  - \ref MSG_PRI_NORMAL - для обычных сообщений, помещаемых в конец очереди.
 *  - \ref MSG_PRI_URGENT - для внеочередных сообщений, помещаемых в начало очереди.
 *  @return \ref OK при успешном выполнении, либо значение меньше нуля -- код ошибки.
 */
STATUS msgQSend(MSG_Q_ID msgQId, const void * buffer, size_t nBytes, int timeout, int priority);

/**
 *  @brief Прочитать сообщение из очереди.
 *  @details Функция читает сообщение из очереди. Если в очереди нет сообщений,
 *  то задача переводится в состояние ожидания.
 *  @warning Если задача ждет неограниченное время (\ref WAIT_FOREVER),
 *  а очередь будет удалена, то задача навсегда останется заблокированной!
 * 
 *  @warning Не следует использовать очереди сообщений в обработчиках прерываний!
 *  В обработчиках следует использовать \ref ringbuffer.h "кольцевой буфер".
 *  
 *  @param msgQId Идентификатор очереди.
 *  @param buffer Указатель на буфер, в который будет помещено полученное сообщение.
 *  @param maxNBytes Размер сообщения. Он должен соответствовать размеру сообщения,
 *  помещенного в очередь. Если размер указан большим, чем фактический, то к сообщению
 *  будет добавлена случайная информация.
 *  @param timeout Время ожидания в тиках таймерах в случае, если очередь пуста.
 *  Допустимы также значения \ref NO_WAIT и \ref WAIT_FOREVER.
 *  @return \ref OK при успешном выполнении, либо значение меньше нуля -- код ошибки.
 */
STATUS msgQReceive(MSG_Q_ID msgQId, void * buffer, size_t maxNBytes, int timeout);

/**
 *  @brief Число сообщений в очереди.
 *  @details Функция возвращает число сообщений, находящихся в данный момент в очереди.
 *  
 *  @param msgQId Идентификатор очереди.
 *  @return Функция возвращает целое число, равное количеству сообщений,
 *  находящихся в очереди на момент вызова, или \b –1, если очередь не существует.
 */
int msgQNumMsgs(MSG_Q_ID msgQId);


/// @privatesection

/**
 *  @brief Значение счётчика семафора записи сообщений в очередь.
 *  @details Значение счётчика используется для отладки работы очередей сообщений.
 *  
 *  @param msgQId Идентификатор очереди.
 *  @return Значение счётчика, или \b –1, если очередь не существует.
 */
int msgQNumMsgsW (MSG_Q_ID msgQId);

#ifdef __cplusplus
}
#endif

#endif
