/**
    @addtogroup groupCore 
    @{
    @file multex.h
    @brief Основной подключаемый файл \b RTOS @osName.
    @details В данном файле содержатся базовые определения
    операционной системы \b RTOS @osName.
    @see \ref sectionMultexOsob.
    @}
*/


/* 
    Автор: Соколов С.В.
    2000 - 2013    
*/

#ifndef _MULTEX32
#define _MULTEX32

#define _MULTEX_    ///< @details Используемая операционная система.

#define ARCH_DMA_MINALIGN   64

#define __LITTLE_ENDIAN ///< @details Используемый в системе порядок следования бит.


#ifdef __cplusplus
extern "C" {
#endif

/// @details Встроенная функция, дающая компилятору подсказку о том, что условие x - истинно.
#define likely(x)       __builtin_expect((x),1)

/// @details Встроенная функция, дающая компилятору подсказку о том, что условие x - ложно.
#define unlikely(x)     __builtin_expect((x),0)

// Стандартные типы.
#include "stdbool.h"
#include "stddef.h"
#include "stdint.h"
#include "limits.h"

// Типы и макросы для обратной совместимости
#include "backcomp.h"

#define printk printf
#define KERN_INFO "MK_Info:"
#define KERN_DEBUG "MK_Debug:"


#define VX_SUPERVISOR_MODE 0

#define INIT_TASK_NAME      "Init"
#define SHELL_NAME          "Shell"
#define SHELL_PRIORITY      100
#define CHAR_CR             ('\r')  ///< Служебный символ "Возврат каретки" (0x0d).
#define CHAR_LF             ('\n')  ///< Служебный символ "Перевод строки" (0x0a).

/// @brief Указатель на функцию, которая используется как точка входа для процессов.
typedef int (*FUNCPTR)(int);

/**
 *  @brief Результат выполнения.
 *  @details Результат выполнения действия (функции).
 */
typedef enum {
    OK = 0,     ///< Успешное выполнение.
    ERROR = -1  ///< Неудача.
} STATUS;



static inline unsigned int CPU_ID (void) {
    unsigned int val;
    asm("mrc p15, 0, %0, c0, c0, 5  @ get MPIDR" : "=r" (val) : : "cc");
    return val & 3;
}

/**
 * @name Различные типы, использующиеся в некоторых модулях.
 * @{
 */
typedef unsigned int dma_addr_t;
typedef int irqreturn_t;
typedef unsigned int resource_size_t;
/// @}


/**
 *  @name Макросы типовых операций
 *  @{
 */

//! @details Не-типобезопасный макрос, возвращающий максимальное значение из 2х вариантов.
#define MAX(a,b) \
    (((a)>(b))?(a):(b))

//! @details Типобезопасный макрос, возвращающий максимальное значение из 2х вариантов.
//! @note Может быть использован только в теле функции.
#define MAX_SAFE(a,b) \
    ({ __typeof__ (a) _a = (a); \
        __typeof__ (b) _b = (b); \
      _a > _b ? _a : _b; })

//! @details Не-типобезопасный макрос, возвращающий минимальное значение из 2х вариантов.
#define MIN(a,b) \
    (((a)<(b))?(a):(b))

//! @details Типобезопасный макрос, возвращающий минимальное значение из 2х вариантов.
//! @note Может быть использован только в теле функции.
#define MIN_SAFE(a,b) \
    ({ __typeof__ (a) _a = (a); \
        __typeof__ (b) _b = (b); \
      _a < _b ? _a : _b; })


//! @details Макрос, обеспечивающий запуск функции только один раз (для функций, не имеющих возвращаемого значения).
#define START_ONCE do{static int Started = 0;if(Started == 1){return;} Started = 1;}while(0)

//! @details Макрос, обеспечивающий запуск функции только один раз (для функций, имеющих возвращаемое значение).
#define START_ONCE_R(r) do{static int Started = 0;if(Started == 1){return (r);} Started = 1;}while(0)

//! @details Макрос, возвращающий количество элементов массива.
#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))

//! @details Макрос, возвращающий размер поля типа.
#define MEMBER_SIZE(type, member) (sizeof(((type *)0)->member))


#define ROUND(a,b)		(((a) + (b) - 1) & ~((b) - 1)) ///< @brief Округление. @details Округление величины \b a до числа кратного заданному \b b.
#define DIV_ROUND(n,d)		(((n) + ((d)/2)) / (d))
#define DIV_ROUND_UP(n,d)	(((n) + (d) - 1) / (d))
#define roundup(x, y)		((((x) + ((y) - 1)) / (y)) * (y)) ///< @brief Округление в большую сторону. @details Округление величины \b x до большего числа кратного заданному \b y.


#define ALIGN(x,a)		__ALIGN_MASK((x),(typeof(x))(a)-1)

#define __ALIGN_MASK(x,mask)	(((x)+(mask))&~(mask)) 

#define DEFINE_ALIGN_BUFFER(type, name, size, align)			\
	static char __##name[roundup(size * sizeof(type), align)]	\
			__attribute__((aligned(align)));				\
	static type *name = (type *)__##name


#define DEFINE_CACHE_ALIGN_BUFFER(type, name, size)			\
	DEFINE_ALIGN_BUFFER(type, name, size, ARCH_DMA_MINALIGN) 


#define ALLOC_ALIGN_BUFFER(type, name, size, align)			\
	char __##name[ROUND(size * sizeof(type), align) + (align - 1)];	\
	type *name = (type *) ALIGN((uintptr_t)__##name, align)

#define ALLOC_CACHE_ALIGN_BUFFER(type, name, size)			\
	ALLOC_ALIGN_BUFFER(type, name, size, ARCH_DMA_MINALIGN) 
	

/// @}

/// @privatesection
static inline u16 __get_unaligned_le16(const u8 *p)
{
	return p[0] | p[1] << 8;
}

static inline u32 __get_unaligned_le32(const u8 *p)
{
	return p[0] | p[1] << 8 | p[2] << 16 | p[3] << 24;
} 
static inline u64 __get_unaligned_le64(const u8 *p)
{
	return (u64)__get_unaligned_le32(p + 4) << 32 |
	       __get_unaligned_le32(p);
}

static inline void __put_unaligned_le16(u16 val, u8 *p)
{
	*p++ = val;
	*p++ = val >> 8;
}

static inline void __put_unaligned_le32(u32 val, u8 *p)
{
	__put_unaligned_le16(val >> 16, p + 2);
	__put_unaligned_le16(val, p);
}

static inline void __put_unaligned_le64(u64 val, u8 *p)
{
	__put_unaligned_le32(val >> 32, p + 4);
	__put_unaligned_le32(val, p);
}

static inline u16 get_unaligned_le16(const void *p)
{
	return __get_unaligned_le16((const u8 *)p);
}

static inline u32 get_unaligned_le32(const void *p)
{
	return __get_unaligned_le32((const u8 *)p);
}

static inline u64 get_unaligned_le64(const void *p)
{
	return __get_unaligned_le64((const u8 *)p);
}

static inline void put_unaligned_le16(u16 val, void *p)
{
	__put_unaligned_le16(val, (u8*)p);
}

static inline void put_unaligned_le32(u32 val, void *p)
{
	__put_unaligned_le32(val, (u8*)p);
}

static inline void put_unaligned_le64(u64 val, void *p)
{
	__put_unaligned_le64(val, (u8*)p);
}

extern void __bad_unaligned_access_size(void); 
 
#define get_unaligned(ptr) ((__force typeof(*(ptr)))({			\
	__builtin_choose_expr(sizeof(*(ptr)) == 1, *(ptr),			\
	__builtin_choose_expr(sizeof(*(ptr)) == 2, get_unaligned_le16((ptr)),	\
	__builtin_choose_expr(sizeof(*(ptr)) == 4, get_unaligned_le32((ptr)),	\
	__builtin_choose_expr(sizeof(*(ptr)) == 8, get_unaligned_le64((ptr)),	\
	__bad_unaligned_access_size()))));					\
	})) 

#define put_unaligned(val, ptr) ({					\
	void *__gu_p = (ptr);						\
	switch (sizeof(*(ptr))) {					\
	case 1:								\
		*(u8 *)__gu_p = (__force u8)(val);			\
		break;							\
	case 2:								\
		put_unaligned_le16((__force u16)(val), __gu_p);		\
		break;							\
	case 4:								\
		put_unaligned_le32((__force u32)(val), __gu_p);		\
		break;							\
	case 8:								\
		put_unaligned_le64((__force u64)(val), __gu_p);		\
		break;							\
	default:							\
		break;							\
	}								\
	(void)0; }) 


#define debug_cond(cond, fmt, args...)		\
	do {					\
		if (cond)			\
			printf(fmt, ##args);	\
	} while (0) 


#define __be32_to_cpu(x) ((((x) & 0x000000ff) << 24) | \
    (((x) & 0x0000ff00) <<  8) | \
    (((x) & 0x00ff0000) >>  8) | \
    (((x) & 0xff000000) >> 24))


/*
 * Divide positive or negative dividend by positive divisor and round
 * to closest integer. Result is undefined for negative divisors and
 * for negative dividends if the divisor variable type is unsigned.
 */
#define DIV_ROUND_CLOSEST(x, divisor)(          \
{                           \
    typeof(x) __x = x;              \
    typeof(divisor) __d = divisor;          \
    (((typeof(x))-1) > 0 ||             \
     ((typeof(divisor))-1) > 0 || (__x) > 0) ?  \
        (((__x) + ((__d) / 2)) / (__d)) :   \
        (((__x) - ((__d) / 2)) / (__d));    \
}                           \
)

/*
 * clamp - return a value clamped to a given range with strict typechecking
 * @val: current value
 * @min: minimum allowable value
 * @max: maximum allowable value
 *
 * This macro does strict typechecking of min/max to make sure they are of the
 * same type as val.  See the unnecessary pointer comparisons.
 */
#define clamp(val, min, max) ({         \
    typeof(val) __val = (val);      \
    typeof(min) __min = (min);      \
    typeof(max) __max = (max);      \
    (void) (&__val == &__min);      \
    (void) (&__val == &__max);      \
    __val = __val < __min ? __min: __val;   \
    __val > __max ? __max: __val; })

typedef int boolean;

// #define msleep(msec) taskDelay(MAX(sysClkRateGet() * (msec) / 1000, 1))

#ifdef __cplusplus
}
#endif
#endif
