/**

    @addtogroup groupMultimedia
    @{
    @file fonts.h
    @brief Высокоуровневые интерфейсы для работы с TTf-шрифтами.
    @details Библиотека функций поддержки <b>TrueType</b>-шрифтов.

    <b>Подключение</b>: @verbatim #include <multimedia/fonts.h> @endverbatim
    \ref sectionMakefile "Makefile": @verbatim LIBRARIES += -l_font -l_freetype @endverbatim

    @see Общее описание функций поддержки \b TTf в главе \ref sectionTTf.
    @}
 */

#ifndef TTFFONTS_H_
#define TTFFONTS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "fontsdefines.h"

//---------------------------------------------------------------------------------------
//       Набор универсальных TTF-шрифтов, объединяющих в себе возможности
//   интерфейсов выше (юникод, пререндер, вписывание в прямоугольник, поворот и т.д.).
//---------------------------------------------------------------------------------------



/**
 * Опции функционирования ttf-шрифта.
 * @details Наборы опций {NoGlyphSaving, SaveGlyphs}, {NoPrerender, PrerenderASCII, ttf_PrerenderDecNumbers}, {NormalOrientation, RotatedOrientation}, {KeepFontOpen, CloseFontAfterPrerender} допускают
 *  выбор только одной опции из каждого набора.
 * Если из набора не будет выбрана ни одна из опций, то по-умолчанию будет использоваться одна из них (см. детальное описание опций).
 * Опции из разных наборов можно совмещать через оператор '|'.
 */
typedef enum {
    // Настройки сохранения глифов "на ходу"
    ttf_NoGlyphSaving = 0x00,  //!< Не кешировать глифы в памяти, опция по-умолчанию.
    ttf_SaveGlyphs = 0x01,     //!< Каждый раз при рендере нового глифа он будет кеширован в память (пока только ASCII-глифы).

    // Настройки пререндера глифов
    ttf_NoPrerender = 0x00,          //!< Не пререндерить глифы при инициализации шрифта, опция по-умолчанию.
    ttf_PrerenderASCII = 0x10,       //!< Пререндерить все ASCII-глифы при инициализации шрифта.
    ttf_PrerenderDecNumbers = 0x20,  //!< Пререндерить все глифы десятичных цифр (0-9).

    // Ориентация глифов
    ttf_NormalOrientation = 0x000,   //!< Глифы будут иметь обычную ориентацию, опция по-умолчанию.
    ttf_RotatedOrientation = 0x100,  //!< Глифы будут развернуты на 270 градусов (про координатные перерасчеты см. описание интерфейса).

    // Нужно ли держать шрифт "открытым", то есть нужна ли возможность выводить любые символы на лету, или пререндера достаточно
    ttf_KeepFontOpen = 0x0000,             //!< Держать шрифт открытым, это позволит рендерить символы "на лету", опция по-умолчанию.
    ttf_CloseFontAfterPrerender = 0x1000,  //!< Закрывать шрифт после инициализации, при этом можно будет использовать только перендеренные символы.

} eTtfFontOptions;

/**
 * Инкапсулированные поля шрифта.
 */
struct ttfPrivateFontStruct;
//! @details Инкапсулированные поля шрифта.
typedef struct ttfPrivateFontStruct sTtfPrivateFontStruct;
//! @details Указатель на инкапсулированные поля шрифта.
typedef struct ttfPrivateFontStruct *pTtfPrivateFontStruct;

/**
 * Структура шрифта.
 */
typedef struct {
    unsigned int FontSize;                       //!< @details Размер шрифта в пунктах.
    unsigned int FontPixelSize;                  //!< @details Размер шрифта в пикселях (фактически высота самой высокого символа от базовой линии).
    unsigned int FontColor;                      //!< @details Цвет шрифта в RGB-формате.
    eTtfFontOptions FontOptions;                 //!< @details Опции шрифта, являются комбинацией перечислений \ref eTtfFontOptions.
    pTtfPrivateFontStruct PrivateStructPointer;  //!< @details Инкапсулированные параметры шрифта.
} sTtfFont;

//! @details Указатель на ttf-шрифт.
typedef sTtfFont *pTtfFont;


/**
 *  @name Инициализация библиотеки
 *  @brief Настройка режимов. Выделение и освобождение ресурсов.
 *  @{
 */

/**
 * Установить \b DPI, который будет использоваться при выводе шрифтов.
 * @details Значение \b DPI по умолчанию - \b 96.
 * @param dpiHorizontal,dpiVerical Значения \b DPI по вертикальной и горизонтальной оси.
 * @return \ref OK при успехе, \ref ERROR, если выполнение функции не разрешено.
 * @warning Данную функцию разрешено использовать только перед началом взаимодействия со шрифтами;
 * после загрузки какого-либо шрифта данная функция не будет выполняться.
 */
STATUS ttf_SetDpi (int dpiHorizontal, int dpiVerical);

/**
 * Загрузить ttf-шрифт.
 * @param path Путь к ttf-файлу (настоящему или виртуальному).
 * @param size Высота шрифта в пунктах.
 * @param color Цвет шрифта в HTML-формате.
 * @param options Комбинация перечислений eTtfFontOptions, можно использовать несколько перечислений через оператор '|'.
 * @return Указатель на загруженный шрифт или \ref NULL при ошибке.
 */
pTtfFont ttf_LoadFont (const char *path, unsigned int size, unsigned int color, eTtfFontOptions options);

/**
 * Освободить ресурсы шрифта.
 * @param font Шрифт для освобождения.
 * @return \ref OK в случае успеха, \ref ERROR в противном случае.
 */
STATUS ttf_FreeFont (pTtfFont font);

/// @}


/**
 *  @name Вывод текста
 *  @{
 */

/**
 * Функция вывода ASCII-текста, аналог textOut.
 * @param outputSurface Поверхность, на которую осуществляется вывод.
 * Может отличаться в зависимости от используемой графической библиотеки.
 * Поверхность будет использована при копировании символа с помощью fntGlyphCopy() 
 * и должна иметь тот же тип данных.
 * @param x,y Позиция для вывода на сурфейсе.
 * @param font Указатель на шрифт для вывода.
 * @param text Указатель на выводимый текстовый буфер.
 * @return \ref OK в случае успеха, \ref ERROR в противном случае.
 * @details Границы вывода не проверяются, выход за границы экрана/буфера будет предотвращен функциями графической подсистемы.
 */
STATUS ttf_Print (void *outputSurface, int x, int y, pTtfFont font, const char *text);

/**
 * Вывести UTF-16-текст, аналог textOut.
 * @param outputSurface Поверхность, на которую осуществляется вывод.
 * Может отличаться в зависимости от используемой графической библиотеки.
 * Поверхность будет использована при копировании символа с помощью fntGlyphCopy() 
 * и должна иметь тот же тип данных.
 * @param x,y Позиция для вывода на сурфейсе.
 * @param font Указатель на шрифт для вывода.
 * @param text Указатель на выводимый UTF-16 буфер.
 * @param len Длинна выводимого UTF-16 буфера.
 * @return \ref OK в случае успеха, \ref ERROR в противном случае.
 * @details Границы вывода не проверяются, выход за границы экрана/буфера будет предотвращен функциями графической подсистемы.
 */
STATUS ttf_PrintUtf16 (void *outputSurface, int x, int y, pTtfFont font, const unsigned short *text, unsigned int len);

// Аналоги drawUtf16Text
/**
 * Вывести ASCII-текст в прямоугольник с проверкой границ, аналог drawUtf16Text.
 * @param outputSurface Поверхность, на которую осуществляется вывод.
 * Может отличаться в зависимости от используемой графической библиотеки.
 * Поверхность будет использована при копировании символа с помощью fntGlyphCopy() 
 * и должна иметь тот же тип данных.
 * @param dstRect Координатный прямоугольник для вывода.
 * @param align Выравнивание текста в прямоугольнике.
 * @param font Указатель на шрифт для вывода.
 * @param text Указатель на выводимый текстовый буфер.
 * @return \ref OK в случае успеха, \ref ERROR в противном случае.
 * @details Границы вывода проверяются, символы, выходящие за границы сурфейса или прямоугольника вывода НЕ будут выведены.
 */
STATUS ttf_PrintRect (void *outputSurface, const pTextRect dstRect, eAlignType align, pTtfFont font, const char *text);

/**
 * Вывести UTF-16-текст в прямоугольник с проверкой границ, аналог drawUtf16Text.
 * @param outputSurface Поверхность, на которую осуществляется вывод.
 * Может отличаться в зависимости от используемой графической библиотеки.
 * Поверхность будет использована при копировании символа с помощью fntGlyphCopy() 
 * и должна иметь тот же тип данных.
 * @param dstRect Координатный прямоугольник для вывода.
 * @param align Выравнивание текста в прямоугольнике.
 * @param font Указатель на шрифт для вывода.
 * @param text Указатель на выводимый UTF-16 буфер.
 * @param len Длинна выводимого UTF-16 буфера.
 * @return \ref OK в случае успеха, \ref ERROR в противном случае.
 * @details Границы вывода проверяются, символы, выходящие за границы сурфейса или прямоугольника вывода НЕ будут выведены.
 */
STATUS ttf_PrintRectUft16 (void *outputSurface, const pTextRect dstRect, eAlignType align, pTtfFont font, const unsigned short *text, unsigned int len);

/**
 * Вывести ASCII-текст в прямоугольник без проверки границ, аналог drawUtf16TextNoCheckBorder.
 * @param outputSurface Поверхность, на которую осуществляется вывод.
 * Может отличаться в зависимости от используемой графической библиотеки.
 * Поверхность будет использована при копировании символа с помощью fntGlyphCopy() 
 * и должна иметь тот же тип данных.
 * @param dstRect Координатный прямоугольник для вывода.
 * @param align Выравнивание текста в прямоугольнике.
 * @param font Указатель на шрифт для вывода.
 * @param text Указатель на выводимый текстовый буфер.
 * @return \ref OK в случае успеха, \ref ERROR в противном случае.
 * @details Границы вывода не проверяются, символы могут выйти за границы прямоугольника, 
 * однако выход за границы сурфейса будет предотвращен функциями графической подсистемы.
 */
STATUS ttf_PrintRectNoCheckBorder (void *outputSurface, const pTextRect dstRect, eAlignType align, pTtfFont font, const char *text);

/**
 * Вывести UTF-16-текст в прямоугольник без проверки границ, аналог drawUtf16TextNoCheckBorder.
 * @param outputSurface Поверхность, на которую осуществляется вывод.
 * Может отличаться в зависимости от используемой графической библиотеки.
 * Поверхность будет использована при копировании символа с помощью fntGlyphCopy() 
 * и должна иметь тот же тип данных.
 * @param dstRect Координатный прямоугольник для вывода.
 * @param align Выравнивание текста в прямоугольнике.
 * @param font Указатель на шрифт для вывода.
 * @param text Указатель на выводимый UTF-16 буфер.
 * @param len Длинна выводимого UTF-16 буфера.
 * @return \ref OK в случае успеха, \ref ERROR в противном случае.
 * @details Границы вывода не проверяются, символы могут выйти за границы прямоугольника, 
 * однако выход за границы сурфейса будет предотвращен функциями графической подсистемы.
 */
STATUS ttf_PrintRectUft16NoCheckBorder (void *outputSurface, const pTextRect dstRect, eAlignType align, pTtfFont font, const unsigned short *text, unsigned int len);

/// @}

/**
 *  @name Получение характеристик текста
 *  @{
 */

/**
 * Получить ширину выводимой ASCII-надписи для шрифта нормальной ориентации или высоту надписи для шрифта повернутой ориентации.
 * @param font Указатель на шрифт.
 * @param text Указатель на ASCII-буфер.
 * @return Ширина или высота (в зависимости от ориентации шрифта) в пикселях или -1 при ошибке.
 */
int ttf_GetTextPixelLength (pTtfFont font, const char *text);

/**
 * Получить ширину выводимой UTF-16-надписи для шрифта нормальной ориентации или высоту надписи для шрифта повернутой ориентации.
 * @param font Указатель на шрифт.
 * @param text Указатель на UTF-16 буфер.
 * @param len Длинна UTF-16 буфера.
 * @return Ширина или высота (в зависимости от ориентации шрифта) в пикселях или -1 при ошибке.
 */
int ttf_GetTextPixelLengthUtf16 (pTtfFont font, const unsigned short *text, unsigned int len);

/**
 * Получить высоту и ширину ASCII-надписи.
 * @param[out] width Указатель на буфер, куда будет записана ширина надписи.
 * @param[out] height Указатель на буфер, куда будет записана высота шрифта (не надписи).
 * @param font Указатель на шрифт.
 * @param text Указатель на ASCII-буфер.
 * @return \ref OK в случае успеха, \ref ERROR в противном случае.
 */
STATUS ttf_GetTextRect (int *width, int *height, pTtfFont font, const char *text);

/**
 * Получить высоту и ширину UTF-16-надписи.
 * @param[out] width Указатель на буфер, куда будет записана ширина надписи.
 * @param[out] height Указатель на буфер, куда будет записана высота шрифта (не надписи).
 * @param font Указатель на шрифт.
 * @param text Указатель на UTF-16 буфер.
 * @param len Длинна UTF-16 буфера.
 * @return \ref OK в случае успеха, \ref ERROR в противном случае.
 */
STATUS ttf_GetTextRectUtf16 (int *width, int *height, pTtfFont font, const unsigned short *text, unsigned int len);

/// @}

/**
 *  @name Прочие функции
 *  @{
 */

/**
 * Преобразовать размер шрифта (в типографских пунктах) для различных dpi.
 * @param originalSize Исходный размер шрифта в типографских пунктах.
 * @param originalDpi Исходный \b DPI.
 * @param newDpi Новый \b DPI.
 * @return Размер шрифта (в типографских пунктах) для нового \b DPI или -1, в случае ошибки.
 */
int ttf_ConvertFontSize (int originalSize, int originalDpi, int newDpi);

/**
 * Варианты текстового вывода от библиотеки. Варианты могут комбинироваться.
 */
typedef enum {
    TTOT_None = 0,               //!< Текстовый вывод отсутствует, опция по-умолчанию.
    TTOT_FontLoadingTime = 0x1,  //!< Выводить сообщения вида "Font "Font.ttf" (color = 0xFFFFFF, size = 90) was loaded in 1313 ms".
    TTOT_Errors = 0x2,           //!< Выводить сообщения при ошибках.
    TTOT_Debug = 0x8,            //!< Выводить вспомогательный вывод для отладки библиотеки.
} eTtfTextOutputType;

/**
 * Установить текстовый вывод библиотеки.
 * @param outputType Параметры текстового вывода.
 */
void ttf_SetTextOutputType (eTtfTextOutputType outputType);

/// @}



#ifdef __cplusplus
}
#endif
#endif  // ifndef TTFFONTS_H_
