/**
    @addtogroup groupMultimedia
    @{
    @file tv-decoder.h
    @brief Драйвер аппаратного TV-декодера.
    @details 

    <b>Подключение</b>: @verbatim #include <multimedia/tv-decoder.h> @endverbatim
    \ref sectionMakefile "Makefile": @verbatim LIBRARIES += -l_tvd @endverbatim
    
    Настройка аппаратной части TV-декодера.
    Рекомендуется использовать TV-декодер в составе
    Устройства приёма телевизионного сигнала, описанного 
    в tv-receiver.h.
    @}
 */

/*
    @author Матяш А.В. (a.matiash@set-code.ru)
    @date 20.11.2023
*/

#ifndef __TV_DECODER_H__
#define __TV_DECODER_H__


/**
 *  @brief Выбор входного интерфейса TV-декодера.
 */
typedef enum {
    TVD_CVBS,
    TVD_YPBPR_I,
    TVD_YPBPR_P,
} tvd_interface_t;

/**
 *  @brief Выбор стандарта видео сигнала.
 */
typedef enum {
    TVD_NTSC,
    TVD_PAL,
    TVD_SECAM,
} tvd_system_t;

/**
 *  @brief Выбор формата выходных данных TV-декодера.
 */
typedef enum {
    TVD_PL_YUV422,
    TVD_PL_YUV420,
    TVD_MB_YUV420,
} tvd_fmt_t;

/**
 *  @brief Номера каналов (аппаратных входов) TV-декодера.
 */
typedef enum {
    TVD_0 = 0,
    TVD_1,
    TVD_2,
    TVD_3
} tvd_chnl_t;



/**
 *  @brief Запуск работы TV-декодера.
 *  @details Инициализация аппаратного модуля захвата
 *  телевизионного сигнала. Модуль запускается сразу на 
 *  все 4 входа.
 *  
 *  @param interface Выбор входного интерфейса декодера.
 *  @param system Выбор стандарта сигнала для \b CVBS интерфейса.
 */
void tvd_init (tvd_interface_t interface, tvd_system_t system);

/**
 *  @brief Активировать прерывания для канала TV-декодера.
 *  @details Функция активирует прерывания для выбранного
 *  канала TV-декодера.
 *  
 *  @param id Номер канала декодера 
 *  (рекомендуется выбирать из перечисления \ref tvd_chnl_t).
 */
void tvd_irq_enable (tvd_chnl_t id);

/**
 *  @brief Деактивировать прерывания для канала TV-декодера.
 *  @details Функция деактивирует прерывания для выбранного
 *  канала TV-декодера.
 *  
 *  @param id Номер канала декодера 
 *  (рекомендуется выбирать из перечисления \ref tvd_chnl_t).
 */
void tvd_irq_disable (tvd_chnl_t id);

/**
 *  @brief Получить статус прерывания выбранного канала TV-декодера.
 *  @details Функция позволяет получить значение одного бита регистра
 *  статуса прерывания TV-декодера.
 *  
 *  @param id Номер канала декодера 
 *  (рекомендуется выбирать из перечисления \ref tvd_chnl_t).
 *  @return \b 1 -- если бит статуса выбранного канала взведён,
 *  иначе \b 0.
 */
signed int tvd_irq_status_get (tvd_chnl_t id);

/**
 *  @brief Очистить бит прерывания выбранного канала TV-декодера.
 *  @details Функция позволяет очистить один бит регистра статуса
 *  прерывания TV-декодера.
 *  
 *  @param id Номер канала декодера 
 *  (рекомендуется выбирать из перечисления \ref tvd_chnl_t).
 */
void tvd_irq_status_clear (tvd_chnl_t id);


/**
 *  @brief Получить статус прерывания всех каналов TV-декодера.
 *  @details Функция позволяет получить значение группы бит регистра
 *  статуса прерывания TV-декодера.
 *  
 *  @return Значение группы бит регистра статуса. Значащие -- первые 
 *  4 бита.
 */
int tvd_irq_all_status_get ();

/**
 *  @brief Очистить биты прерывания TV-декодера.
 *  @details Функция позволяет очистить группу бит регистра статуса
 *  прерывания TV-декодера.
 *  
 *  @param status Значение группы бит регистра статуса, 
 *  полученное с помощью tvd_irq_all_status_get().
 */
void tvd_irq_all_status_clear (int status);

/**
 *  @brief Запуск захвата сигнала для выбранного канала.
 *  @details Функция запускает захват входного сигнала
 *  для выбранного канала TV-декодера.
 *  
 *  @param id Номер канала декодера 
 *  (рекомендуется выбирать из перечисления \ref tvd_chnl_t).
 */
void tvd_capture_on (tvd_chnl_t id);

/**
 *  @brief Остановка захвата сигнала для выбранного канала.
 *  @details Функция останавливает захват входного сигнала
 *  для выбранного канала TV-декодера.
 *  
 *  @param id Номер канала декодера 
 *  (рекомендуется выбирать из перечисления \ref tvd_chnl_t).
 */
void tvd_capture_off (tvd_chnl_t id);

/**
 *  @brief Задать адрес буфера для выходного буфера 
 *  цветоразностного сигнала.
 *  @details Функция позволяет задать адрес области памяти
 *  в которую будут помещены данные цветоразностного сигнала 
 *  декодированного кадра.
 *  
 *  @param id Номер канала декодера 
 *  (рекомендуется выбирать из перечисления \ref tvd_chnl_t).
 *  @param addr Адрес области памяти для цветоразностного сигнала.
 */
void tvd_set_luma (tvd_chnl_t id, unsigned int addr);

/**
 *  @brief Задать адрес буфера для выходного буфера 
 *  яркостного сигнала.
 *  @details Функция позволяет задать адрес области памяти
 *  в которую будут помещены данные яркостного сигнала 
 *  декодированного кадра.
 *  
 *  @param id Номер канала декодера 
 *  (рекомендуется выбирать из перечисления \ref tvd_chnl_t).
 *  @param addr Адрес области памяти для яркостного сигнала.
 */
void tvd_set_chroma (tvd_chnl_t id, unsigned int addr);

/**
 *  @brief Задать ширину кадра входного видео сигнала.
 *  @details Функция задаёт ширину кадра входного видео сигнала.
 *  
 *  @param id Номер канала декодера 
 *  (рекомендуется выбирать из перечисления \ref tvd_chnl_t).
 *  @param w Ширина кадра в пикселях.
 */
void tvd_set_width (tvd_chnl_t id, unsigned int w);

/**
 *  @brief Задать высоту кадра входного видео сигнала.
 *  @details Функция задаёт высоту кадра входного видео сигнала.
 *  
 *  @param id Номер канала декодера 
 *  (рекомендуется выбирать из перечисления \ref tvd_chnl_t).
 *  @param w Высота кадра в пикселях.
 */
void tvd_set_height (tvd_chnl_t id, unsigned int h);

/**
 *  @brief Задать формат декодированного кадра.
 *  @details Функция позволяет выбрать формат 
 *  выходных данных TV-декодера.
 *  
 *  @param id Номер канала декодера 
 *  (рекомендуется выбирать из перечисления \ref tvd_chnl_t).
 *  @param fmt Формат данных из перечисления \ref tvd_fmt_t.
 */
void tvd_set_fmt (tvd_chnl_t id, tvd_fmt_t fmt);

/**
 *  @brief Получить статус TV-декодера.
 *  @details Функция возвращает статус аппаратного TV-декодера.
 *  
 *  @param id Номер канала декодера 
 *  (рекомендуется выбирать из перечисления \ref tvd_chnl_t).
 *  @return Битовую маску. Используются следующие биты:
 *  бит 0 -- detect (признак обнаружения входного сигнала);
 *  бит 4 -- system.
 */
unsigned int tvd_get_status (tvd_chnl_t id);

/**
 *  @brief Настройка параметров передачи цвета TV-декодера.
 *  @details Настройка параметров передачи цвета TV-декодера
 *  при отличаи от значений по умолчанию.
 *  
 *  @param id Номер канала декодера 
 *  (рекомендуется выбирать из перечисления \ref tvd_chnl_t).
 *  @param luma 
 *  @param contrast 
 *  @param saturation 
 *  @param hue 
 */
void tvd_set_color (tvd_chnl_t id, unsigned int luma, 
    unsigned int contrast, unsigned int saturation, 
    unsigned int hue);





/// @privatesection

void tvd_uv_swap (unsigned char uv_swap);

// Функция внесена в состав tvd_set_width()
void tvd_set_width_jump (tvd_chnl_t id, unsigned int j);


#endif