/**
    @addtogroup groupMultimedia
    @{
    @file tv-receiver.h
    @brief Устройство захвата телевизионного сигнала.
    @details 
    
    <b>Подключение</b>: @verbatim #include <multimedia/tv-receiver.h> @endverbatim
    \ref sectionMakefile "Makefile": @verbatim LIBRARIES += -l_tvd @endverbatim

    
    Устройство приёма телевизионного сигнала на базе драйвера
    аппаратного декодера tv-decoder.h. Для начала приёма сигнала необходимо
    запустить аппаратную часть с помощью функции инициализации и создать
    устройство приёма данных. При создании устройства ему выделяется
    тройной кольцевой буфер данные из которого можно брать в любой момент
    времени. Для синхронизации с обновлением данных от камеры можно использовать
    функцию tvrDataChanged(). Приём данных от всех источников входного сигнала
    ведётся независимо.

    Пример запуска устройства приёма телевизионного сигнала, получения
    кадров и вывода изображения на конструируемую поверхность приведён ниже:

    @code
    HDC surf = newSurface (720, 576, G2D_FMT_PYUV420UVC);
    tvrInit (TVD_CVBS, TVD_PAL);
    tvrDevCreate (TVD_0, TVD_PL_YUV420, 720, 576);
    while (true) {
        tvrWaitFrame (TVD_0, WAIT_FOREVER);
        tvrGetData (TVD_0, surf);
        bitBlt (CONSTR, 0, 0, FRAME_W, FRAME_H, surf, 0, 0, 0, 0, 0);
    }
    @endcode
    @}
 */

/*
    @author Матяш А.В. (a.matiash@set-code.ru)
    @date 20.11.2023
*/

#ifndef _TV_RECEIVER_H_
#define _TV_RECEIVER_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>
#include <multimedia/a20graph.h>
#include <multimedia/tv-decoder.h>

/**
 *  @brief Настройка и запуск аппаратного декодера.
 *  @details Функция настраивает и запускает аппаратную часть
 *  TV-декодера.
 *  
 *  @param interface Выбор входного интерфейса декодера.
 *  @param system Выбор стандарта сигнала для \b CVBS интерфейса.
 *  @return \ref true при удачной настройке, иначе \ref false.
 */
bool tvrInit (tvd_interface_t interface, tvd_system_t system);

/**
 *  @brief Создание устройства захвата декодированного
 *  видео потока.
 *  @details Функция создаёт устройство захвата и выделяет
 *  память для его работы.
 *  
 *  @param n Номер канала от 0 до 4. 
 *  Рекомендуется выбирать значение из перечисления \ref tvd_chnl_t.
 *  @param fmt Формат данных из перечисления \ref tvd_fmt_t.
 *  @param width Ширина захватываемого кадра.
 *  @param height Высота захватываемого кадра.
 *  @return \ref true при удачном создании устройства, иначе \ref false.
 */
bool tvrDevCreate (int n, tvd_fmt_t fmt, unsigned int width, unsigned int height);

/**
 *  @brief Удаление устройства захвата декодированного
 *  видео потока.
 *  @details Функция удаляет устройство захвата, если таковое было создано.
 *  
 *  @param n Номер канала от 0 до 4. 
 *  Рекомендуется выбирать значение из перечисления \ref tvd_chnl_t.
 */
void tvrDevDelete (int n);

/**
 *  @brief Ожидание захвата одного кадра в выбранном канале.
 *  @details Функция ожидает окончания захвата и декодирования
 *  одного кадра в выбранном канале в течение заданного промежутка
 *  времени. Ожидание реализовано с использованием функции
 *  taskDelay() и предназначено для использования в отдельной задаче.
 *  
 *  @param n Номер канала от 0 до 4. 
 *  Рекомендуется выбирать значение из перечисления \ref tvd_chnl_t.
 *  @param timeout Максимальное время ожидания в тиках системного таймера.
 *  Возможны так же значения \ref NO_WAIT и \ref WAIT_FOREVER.
 *  @return \ref true если кадр захвачен раньше указанного времени, 
 *  иначе \ref false.
 */
bool tvrWaitFrame (int n, int timeout);


/**
 *  @brief Получить данные последнего декодированного кадра.
 *  @details Функция подключает буфер с последним декодированным
 *  кадром к указанной поверхности. Физического копирования
 *  данных не происходит -- функция только заменяет адреса
 *  областей памяти поверхности. Рекомендуется получать указатели
 *  на последний декодированный кадр сразу после окончания декодирования,
 *  о котором можно узнать с помощью tvrWaitFrame(). Декодированные
 *  данные следует сразу скопировать, например, в конструируемую 
 *  поверхность с помощью одной из функций копирования из раздела
 *  \ref ancA20GraphSurfaces "Работа с поверхностями".
 *  
 *  @param n Номер канала от 0 до 4. 
 *  Рекомендуется выбирать значение из перечисления \ref tvd_chnl_t.
 *  @param surf Поверхность, к которой будут подключены буферы
 *  декодированного кадра.
 *  @return \ref true при удачном получении данных, иначе \ref false.
 */
bool tvrGetData (int n, HDC surf);


/// @privatesection

/// @brief  Сброс флага готовности кадра.
/// @param n Номер канала от 0 до 4. 
void tvrDataFlush (int n);

/// @brief Проверка готовности кадра.
/// @param n Номер канала от 0 до 4.
bool tvrDataChanged (int n);



#ifdef __cplusplus
}
#endif

#endif
