/**
    @addtogroup groupCore
    @{
    @file pll.h
    @brief Работа с PLL.
    @details Проверка и настройка конфигурации распределения частот в процессоре \b ARM.

    <b>Подключение</b>: @verbatim #include <pll.h> @endverbatim
    
    @see Общее описание \b PLL в разделе \ref sectionPll.
    @}

    @page pageCore

    @section sectionPll PLL -- распределение тактовых частот
    @see Описание методов работы с \b PLL в файле pll.h.

    Опорная частота 24 МГц, получаемая с помощью внутренней схемы генерации и
    стабилизируемая с помощью внешнего кварцевого резонатора, повышается
    внутри процессора и далее распределяется по внутренним шинам и
    аппаратным модулям. Как именно происходит распределение частот зависит
    от конфигурации конкретного процессора. Управление умножением и делением
    частот и распределение их между шинами и модулями осуществляется с
    помощью регистров конфигурации \b PLL. 
    
    Функции по настройке \b PLL регистров полностью берёт на себя операционная
    система. Однако при проектировании новых драйверов устройств, а так же
    для проверки правильной работы уже подключенных аппаратных модулей имеет
    смысл воспользоваться функциями, описанными в файле pll.h. Для вывода в
    консоль таблицы используемых тактовых частот можно воспользоваться
    функцией pll(). Для получения значения частоты конкретного аппаратного
    модуля или шины в программе можно воспользоваться функциями pllGet(),
    pllAhbGet() и им подобными. Следует учесть, что драйверы устройств могут
    изменять частоты общих шин, поэтому для получения актуальных значений
    следует перед использованием \b get-функций вызвать функцию сбора данных
    pllUpdate().


 */

/*
    @author Матяш А.В. (a.matiash@set-code.ru)
    @date 20.05.2023
*/

#ifndef _PLL_H_
#define _PLL_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>

/**
 *  @anchor ancPllDefines
 *  @name Макросы обозначения PLL
 *  @{
 */

#define PLL_1       0   ///< @details Индекс для \b PLL1.
#define PLL_CPU     0   ///< @details Синоним для \b PLL1.
#define PLL_2       1   ///< @details Индекс для \b PLL2.
#define PLL_AUDIO   1   ///< @details Синоним для \b PLL2.
#define PLL_3       2   ///< @details Индекс для \b PLL3.
#define PLL_4       3   ///< @details Индекс для \b PLL4.
#define PLL_VE      3   ///< @details Синоним для \b PLL4.
#define PLL_5       4   ///< @details Индекс для \b PLL5.
#define PLL_6       5   ///< @details Индекс для \b PLL6.
#define PLL_PERIPH0 5   ///< @details Синоним для \b PLL6.
#define PLL_7       6   ///< @details Индекс для \b PLL7.
#define PLL_8       7   ///< @details Индекс для \b PLL8.
#define PLL_9       8   ///< @details Индекс для \b PLL9.

/// @}


/**
 *  @anchor ancAhbApbDefines
 *  @name Макросы обозначения шин AHB, APB
 *  @{
 */

#define AHB_1       0   ///< @details Индекс для шины \b AHB1.
#define AHB_2       1   ///< @details Индекс для шины \b AHB2.
#define APB_1       0   ///< @details Индекс для шины \b APB1.
#define APB_2       1   ///< @details Индекс для шины \b APB2.

/// @}



/**
 *  @brief Расчёт частот всех \b PLL и шин на основе заданной конфигурации.
 *  @details Рекомендуется рассчитывать заново всю конфигурацию частот
 *  перед получением данных о работе какой либо \b PLL или шины.
 *  Это обусловлено тем, что каждый модуль при настройке может изменить
 *  частоты некоторых модулей под себя. Окончательную конфигурацию
 *  частот можно посмотреть с помощью команды pll().
 *  
 *  @return \ref true Модуль настроен, данные о частотах обновлены.
 *  @return \ref false Модуль не настроен, не прошла инициализация.
 *  Скорее всего не указан процессор в модуле \b arch.
 */
bool pllUpdate ();

/**
 *  @brief Получить текущую частоту заданной \b PLL.
 *  @details Функция возвращает рассчитанную частоту для выбранной \b PLL.
 *  Для получения актуального значения частоты имеет смысл вызвать pllUpdate().
 *  
 *  @param n Номер \b PLL из группы \ref ancPllDefines "макросов".
 *  @param out Номер выхода \b PLL. Обычно в \b PLL существует только один выход (0).
 *  В некоторых случаях \b PLL имеет 2 выхода с разными частотами --- 0 и 1.
 *  
 *  @param ok Признак нормальной работы \b PLL. Стабильная работа в заданном 
 *  диапазоне частот.
 *  @return Частота выхода \b PLL в Гц. 
 */
unsigned int pllGet (unsigned int n, unsigned int out, bool *ok);

/**
 *  @brief Установить частоту для \b PLL.
 *  @details Функция рассчитывает и устанавливает коэффициенты выбранной
 *  \b PLL таким образом, чтобы итоговая частота была как можно ближе к заданной.
 *  @experimental Функция реализована с ограниченным функционалом и находится в стадии тестирования.
 *  
 *  @param n Номер \b PLL из группы \ref ancPllDefines "макросов".
 *  @param out Номер выхода \b PLL. Обычно в \b PLL существует только один выход (0).
 *  В некоторых случаях \b PLL имеет 2 выхода с разными частотами --- 0 и 1.
 *  Установка выполняется только для выхода 0.
 *  @param frequency Заданная частота в Гц.
 *  @return Установленное значение частоты в Гц если настройка прошла успешно, иначе 0.
 */
unsigned int pllSet (unsigned int n, unsigned int out, unsigned int frequency);

/**
 *  @brief Получить текущую частоту шины \b AXI.
 *  @details Функция возвращает рассчитанную частоту шины \b AXI.
 *  Для получения актуального значения частоты имеет смысл вызвать pllUpdate().
 *  
 *  @param ok \ref true --- шина присутствует и значение частоты вычислено. 
 *  @return Частота шины в Гц. 
 */
unsigned int pllAxiGet (bool *ok);

/**
 *  @brief Получить текущую частоту шины \b Ahb.
 *  @details Функция возвращает рассчитанную частоту шины \b AHB.
 *  Для получения актуального значения частоты имеет смысл вызвать pllUpdate().
 *  
 *  @param n Номер шины \b AHB из группы \ref ancAhbApbDefines "макросов".
 *  @param ok \ref true --- шина присутствует и значение частоты вычислено. 
 *  @return Частота шины в Гц. 
 */
unsigned int pllAhbGet (unsigned int n, bool *ok);

/**
 *  @brief Получить текущую частоту шины \b Apb.
 *  @details Функция возвращает рассчитанную частоту шины \b APB.
 *  Для получения актуального значения частоты имеет смысл вызвать pllUpdate().
 *  
 *  @param n Номер шины \b APB из группы \ref ancAhbApbDefines "макросов".
 *  @param ok \ref true --- шина присутствует и значение частоты вычислено. 
 *  @return Частота шины в Гц. 
 */
unsigned int pllApbGet (unsigned int n, bool *ok);


/**
 *  @brief Вывести текущую конфигурацию \b PLL.
 *  @details Функция выводить текущие настройки
 *  используемых частот в консоль.
 *  
 */
void pll ();


/// @privatesection

/**
 *  @brief Инициализация модуля.
 *  @details Считывание информации об используемом
 *  процессоре. Функция используется системой --- пользователю
 *  вызывать не нужно.
 *  
 *  @return STATUS 
 */
STATUS pllArchInit ();


#ifdef __cplusplus
}
#endif

#endif
