/**
    @addtogroup groupCore 
    @{
    @file semlib.h
    @brief Управление семафорами.
    @details Семафоры в @osName – это основной механизм синхронизации задач в
    реальном времени и организации взаимоисключающего доступа задач к общим
    ресурсам.

    @see Более подробное описание в главе \ref sectionSemLib.

    Пример использования семафора в простой задаче:
    @code
    SEM_ID sem = NULL;
    bool stop;

    // Задача в отдельной функции
    int taskSimple (int dummy) {
        // Инициализация переменных задачи (если есть)
        void *p = malloc (size);

        while (!stop) {
            // Ожидание семафора
            semTake (sem, WAIT_FOREVER);
            // ... тело задачи
        }

        // Освобождение выделенной памяти
        free (p);
        // Завершение задачи
        return 0;
    }

    // Создание и запуск задачи
    void testSimple () {
        sem = semBCreate (SEM_Q_FIFO, SEM_EMPTY);
        stop = false;
        taskSpawn ("task simple", 10, 0, 0, taskSimple, 0);
    }  

    // Некое внешнее событие
    void testRelease () {
        // Освободить семафор
        semGive (sem);
    }
    @endcode

    @}


 */

#ifndef _SEMLIB
#define _SEMLIB

#ifdef __cplusplus
extern "C" {
#endif


#include "multex.h"
#include "tasklib.h"

//! @details Не ждать.
#define NO_WAIT         (0)

//! @details Ждать до успеха.
#define WAIT_FOREVER    (-1)

//! @brief Порядок обработки задач - \b Приоритет.
//! @details Обрабатывать семафоры с учетом приоритетов задач.
//! @warning Возможно не работает.
#define SEM_Q_PRIORITY        0x1

//! @brief Порядок обработки задач - \b FIFO.
//! @details Обрабатывать семафоры с учетом модели \b FIFO задач.
//! @warning Возможно не работает.
#define SEM_Q_FIFO            0x0

//! @brief Инверсия приоритетов.
//! @details Разрешить инверсию приоритетов для мьютекса.
#define SEM_INVERSION_SAFE    0x8

//! @brief Защита задачи.
//! @details Защищать задачу от удаления, если она захватила семафор.
//! @warning Применимо только для мьютексов.
#define SEM_DELETE_SAFE       0x4

//! @details Сигнатура семафора.
#define SEM_MARKER      (0xA525E727)

/**
 * Состояние семафора.
 */
typedef enum {
 SEM_EMPTY = 0,   //!< @details Семафор закрыт (заблокирован, захвачен).
 SEM_FULL = 1     //!< @details Семафор открыт (разблокирован, свободен)
} SEM_B_STATE;

/**
 * Класс семафора.
 */
typedef enum {
 scSemB,   //!< @details Бинарный семафор.
 scSemC,   //!< @details Семафор-счетчик.
 scSemM,   //!< @details Мьютекс.
} SEM_CLASS;

/**
 * @brief Способ подъёма семафора.
 * @details Способ подъёма семафора - по какой причине задачу выдернули с ожидания семафора.
 * @note Устаревшие значения, ныне фактически не используются.
 */
typedef enum {
 sfsNoFlush = 0,    //!< @details Вышло время на ожидание захвата задачи. Семафор не считается захваченным.
 sfsFlush = 1,      //!< @details Устаревшее значение, не должно возникать.
 sfsUnblocked = 2,  //!< @details Мьютекс или семафор-счетчик был принудительно сброшен. Семафор не считается захваченным.
} SEM_FLUSH_STATE;

/**
 * @brief Структура семафора.
 * @details Структура семафора.
 */
typedef struct {
 int marker;              //!< @details Маркер-сигнатура семафора.
 SEM_CLASS semclass;      //!< @details Класс семафора.
 SEM_B_STATE state;       //!< @details Положение семафора.
 SEM_FLUSH_STATE flushed; //!< @details Устаревшее поле, на данный момент не используется.
 int count;               //!< @details Счетчик рекурсий (для мьютексов) или захватов (для счетчика).
 taskId *owner;           //!< @details Захватившая задача.
 int ownpri;              //!< @details Приоритет захватившей задачи.
 int options;             //!< @details Опции семафора.
} Sem_Id;

/**
 *  @brief Указатель на семафор.
 *  @details Указатель на структуру семафора \ref Sem_Id.
 */
typedef Sem_Id *SEM_ID;

/**
 *  @name Макросы и надстройки для быстрой инициализации семафоров.
 *  @{
 */

/**
 * Создать бинарный семафор с параметрами по-умолчанию (\b FIFO, свободный).
 * @return Указатель на семафор или \ref NULL.
 */
SEM_ID semBCreate_Default (void);

/**
 * Создать семафор-мьютекс с параметрами по-умолчанию (\b FIFO).
 * @return Указатель на семафор или \ref NULL.
 */
SEM_ID semMCreate_Default (void);

//! Инициализация мьютекса в сегменте данных. Инициализация производится в следующем виде: 'SEM_ID Mutex = INIT_STATIC_MUTEX(SEM_Q_FIFO);'.
#define INIT_STATIC_MUTEX(SemOptions) &(Sem_Id){.marker = SEM_MARKER, .semclass = scSemM, .state = SEM_FULL, \
        .options = SemOptions,  .flushed=FALSE, .count=0,  .owner=NULL}

//! Инициализация бинарного семафора в сегменте данных. Инициализация производится в следующем виде: 'SEM_ID Sem = INIT_STATIC_SEM(SEM_Q_FIFO, SEM_FULL);'.
#define INIT_STATIC_SEM(SemOptions, SemState) &(Sem_Id){.marker = SEM_MARKER, .semclass = scSemB, .state = SemState, \
        .options = SemOptions,  .flushed=FALSE, .count=0,  .owner=NULL}

//! Инициализация мьютекса в сегменте данных. Инициализация производится в следующем виде: 'SEM_ID Mutex = INIT_STATIC_MUTEX_DEFAULT();'.
#define INIT_STATIC_MUTEX_DEFAULT() &(Sem_Id){.marker = SEM_MARKER, .semclass = scSemM, .state = SEM_FULL, \
        .options = SEM_Q_FIFO,  .flushed=FALSE, .count=0,  .owner=NULL}

//! Инициализация бинарного семафора в сегменте данных. Инициализация производится в следующем виде: 'SEM_ID Sem = INIT_STATIC_SEM_DEFAULT();'.
#define INIT_STATIC_SEM_DEFAULT() &(Sem_Id){.marker = SEM_MARKER, .semclass = scSemB, .state = SEM_FULL, \
        .options = SEM_Q_FIFO,  .flushed=FALSE, .count=0,  .owner=NULL}

/// @}

/**
 *  @name Основные функции для работы с семафорами.
 *  @{
 */

/**
 *  @brief Создать двоичный семафор.
 *  @details Функция создает двоичный семафор. Такой семафор
 *  может быть использован для синхронизации задач, при этом
 *  начальное состояние семафора следует задать закрытым (\ref SEM_EMPTY).
 *  При использовании его для защиты доступа к ресурсам общего пользования
 *  (взаимоисключения) следует создавать семафор изначально открытым (\ref SEM_FULL).
 *  
 *  @param options Способ помещения в очередь задач, ждущих у семафора.
 *  Следует использовать следующие предопределенные константы \b SEM_Q_FIFO и \b SEM_Q_PRIORITY.
 *  @param initialState Начальное состояние семафора.
 *  Если семафор открыт (\ref SEM_FULL), то первая захватившая его задача закрывает и продолжает выполняться.
 *  Если задача пытается захватить закрытый семафор, то ее выполнение задерживается до тех пор,
 *  пока другая задача не откроет этот семафор.
 *  @return SEM_ID Идентификатор созданного семафора или 0 при неудаче.
 */
SEM_ID semBCreate (int options, SEM_B_STATE initialState);

/**
 *  @brief Создать семафор взаимоисключений (\b Mutex).
 *  @details \b Mutex работает так же, как двоичный с открытым начальным состоянием:
 *  первая же захватившая его задача блокирует все дальнейшие попытки его захвата
 *  до тех пор, пока захватившая задача не освободит его.
 *  
 *  Этот тип семафора имеет следующие особенности:
 *  - Он может быть открыт только той задачей, которая его закрыла.
 *  - Он не может быть открыт в обработчике прерывания.
 *  - К нему нельзя применять функцию semFlush().
 *  - Он имеет дополнительную опцию \ref SEM_INVERSION_SAFE, которая включает
 *  алгоритм наследования приоритета, заключающийся в том, что захватившая
 *  семафор задача автоматически получает приоритет, равный самому высокому
 *  приоритету из всех задач, ждущих у этого семафора.
 *  - Он имеет дополнительную опцию \ref SEM_DELETE_SAFE, которая защищает задачу от
 *  удаления на время захвата ею этого семафора.
 *  - Он имеет возможность рекурсивного захвата, при этом задача может
 *  захватить семафор несколько раз подряд. При этом для того, чтобы другая
 *  задача смогла захватить этот семафор, захватившая задача должна освободить
 *  его столько же раз, сколько раз его захватила.
 * 
 *  @param options Следует использовать следующие предопределенные константы \ref SEM_Q_FIFO
 *  или \ref SEM_Q_PRIORITY в комбинации с возможными опциями \ref SEM_INVERSION_SAFE
 *  и \ref SEM_DELETE_SAFE.
 *  @return Указатель на семафор или \ref NULL.
 */
SEM_ID semMCreate (int options);

/**
 *  @brief Создать целочисленный семафор.
 *  @details Отличие целочисленного семафора от двоичного в том,
 *  что он имеет внутренний счётчик, фиксирующий, сколько раз семафор был открыт.
 *  При очередном открытии семафора счётчик увеличивается, при закрытии – уменьшается.
 *  Когда счётчик доходит до нуля, задача, пытающаяся закрыть семафор, будет приостановлена.
 *  Такой тип семафора удобен для организации множественного доступа к буферам
 *  ограниченного размера, стекам, очередям, а также нескольким копиям каких-либо ресурсов.
 *  Начальное значение при этом задают равным максимальному размеру ресурса,
 *  что гарантирует его от переполнения. Внутренний механизм очередей @osName
 *  использует этот тип семафора при их организации.
 *  
 *  @param options Способ помещения в очередь задач, ждущих у семафора.
 *  Следует использовать следующие предопределенные константы \ref SEM_Q_FIFO и \ref SEM_Q_PRIORITY.
 *  @param initialCount Начальное значение внутреннего счётчика открываний.
 *  @return Указатель на семафор или \ref NULL.
 */
SEM_ID semCCreate (int options, int initialCount);

/**
 *  @brief Освободить семафор.
 *  @details Функция освобождает семафор, обеспечивая тем самым возможность
 *  другим задачам его захватить. Если в карусели задержанных задач имеются задачи,
 *  ждущие это событие, то первая из них становится в карусель активных задач. При этом,
 *  если ее приоритет выше выполняемой задачи, производится немедленное переключение
 *  процессора на ее выполнение.
 * 
 *  @warning Не следует использовать семафоры в обработчиках прерываний!
 *  
 *  @param semId Идентификатор семафора, с которым производится действие.
 *  @return \ref OK при успешном выполнении или \ref ERROR при неудаче.
 */
STATUS semGive (SEM_ID semId);

/**
 *  @brief Освободить все задачи, ожидающие захвата бинарного семафора.
 *  @details Функция разблокирует все задачи, ожидающие у этого семафора.
 *  Семафор при этом остается закрытым.
 *  
 *  @warning Не следует использовать семафоры в обработчиках прерываний!
 * 
 *  @param semId Идентификатор семафора, с которым производится действие.
 *  @return \ref OK при успешном выполнении или \ref ERROR при неудаче.
 *  @note Только для семафора типа \ref scSemB.
 */
STATUS semFlush (SEM_ID semId);

/**
 * Разблокировать все задачи, ожидающие захвата мьютекса или семафора-счетчика.
 * @param semId Целевой семафор.
 * @return \ref OK при успехе, \ref ERROR иначе.
 * @details semTake в таком случае вернет \ref ERROR. Нужно для удаления семафоров, на которых кто-нибудь сидит.
 * @note Только для мьютексов или семафоров-счетчиков (scSemM и scSemC).
 */
STATUS semMCUnblock (SEM_ID semId);

/**
 *  @brief Попытаться захватить семафор.
 *  @details Функция делает попытку захватить семафор. Если он на этот момент открыт,
 *  то задача захватывает его и продолжается. Если семафор был закрыт или захвачен
 *  другой задачей, то задача приостанавливается до его открытия, т.е.
 *  ожидает у этого семафора. Время ожидания может быть задано непосредственно
 *  в тиках таймера, либо быть неограниченным (\ref WAIT_FOREVER),
 *  либо отсутствовать (\ref NO_WAIT).
 * 
 *  @warning Не следует использовать семафоры в обработчиках прерываний!
 *  
 *  @param semId Идентификатор семафора, с которым производится действие.
 *  @param timeout Кол-во тиков таймера, в течении которых задача будет ждать захвата.
 *  Дополнительные возможные значения:
 *  - \ref NO_WAIT - не ждать вовсе.
 *  - \ref WAIT_FOREVER - ждать до успеха.
 *  @return \ref OK - семафор был захвачен ИЛИ если бинарный семафор был разблокирован через semFlush().
 *  @return \ref ERROR - семафор не был захвачен.
 *  @return \b -2 - задача обрабатывала прерывание и мгновенный захват не удался.
 *  @return \b -3 - установлен флаг блокировки задачи от переключения и мгновенный захват не удался.
 */
STATUS semTake (SEM_ID semId, int timeout);

/**
 *  @brief Удалить семафор.
 *  @details Функция удаляет указанный семафор и освобождает задачи,
 *  ожидающие его освобождения.
 *  
 *  @param semId Идентификатор семафора, с которым производится действие.
 *  @return \ref OK при успешном выполнении.
 *  @return Если идентификатор семафора, передаваемый функциям управления
 *  семафорами не является правильным или ссылается на удаленный семафор,
 *  функция возвращает значение \ref ERROR.
 */
STATUS semDelete (SEM_ID semId);

/**
 * Получить значение счетчика для семафора-счетчика.
 * @param semId Целевой семафор.
 * @return Значение счетчика для семафора-счетчика.
 */
int semCCount (SEM_ID semId);

/// @}


#ifdef __cplusplus
}
#endif
    
       
#endif
