/**
    @addtogroup groupMultimedia 
    @{
    @file sound.h
    @brief Работа со звуковой подсистемой.
    @details В файле собраны функции поддержки записи/воспроизведения звука.

    <b>Подключение</b>: @verbatim #include <sound.h> @endverbatim
    
    @see Общее описание работы звуковой подсистемы в главе \ref sectionSound.
    @}

    @page pageMultimedia

    @section sectionSound Звуковая подсистема
    @see Описание методов работы со звуковой подсистемой в файле sound.h.

    В составе библиотек ОС @osName имеется поддержка записи/воспроизведения
    звука. Для подключения звуковой подсистемы следует вызвать функцию инициализации
    soundInit(). Данная функция подключат аппаратный кодек используемого процессора.
    Ниже приведён пример запуска звуковой подсистемы и запуска воспроизведения файла.

    @code
    #include <sound.h>
    void play () {
        soundInit ();
        setMasterVol (70);
        playWaveFile ("sound.wav");
    }
    @endcode


 */

#ifndef SOUND_H_
#define SOUND_H_

#include <multex.h>



/**
 *  @name Настройка и инициализация
 *  @{
 */



/**
 *  @brief Инициализация звуковой системы.
 *  @details Инициализация звуковой системы выбранного процессора.
 *  Процессор указывается в файле проекта config.h.
 *  
 *  @return \ref OK при удачном запуске аппаратного кодека, иначе \ref ERROR.
 */
int soundInit ();

/**
 *  @brief Установить громкость звука.
 *  @details Функция устанавливает громкость проигрывания звука.
 *  
 *  @param vol Громкость звука от \b 0 до \b 100.
 *  @return \ref OK при удачном завершении, иначе \ref ERROR.
 */
int setMasterVol(unsigned char vol);


/// @}


/**
 *  @name Воспроизведение сырых данных
 *  @{
 */

/**
 *  @brief Воспроизведение звука из буфера.
 *  @details Функция проигрывает содержимое буфера, как звуковой файл в формате \b PCM.
 *  
 *  @param buffer Указатель на звуковой буфер.
 *  @param size Размер буфера в байтах.
 *  @return \ref OK при удачном завершении.
 *  @return \ref ERROR при ошибке.
 */
int playSndBuffer(char *buffer, int size);

/**
 *  @brief Задать формат воспроизведения буфера.
 *  @details Функция устанавливает заданный формат для воспроизведения буфера.
 *  
 *  @param Channels Количество каналов для воспроизведения (1 или 2).
 *  @param SampleSize Размер выборки в битах.
 *  @param SampleRate Частота дискретизации звука в герцах.
 *  @return \ref OK при удачном завершении.
 *  @return \ref ERROR при ошибке.
 */
int setSndFmt(int Channels, int SampleSize, int SampleRate);


/// @}



/**
 *  @name Поддержка WAV-файлов
 *  @{
 */

/**
 *  @brief Воспроизведение буфера в формате \b RIFF.
 *  @details Функция проигрывает буфер, содержащий звук в формате \b RIFF (\b Wave).
 *  Формат воспроизведения установится в соответствии с данными,
 *  записанными в заголовке буфера.
 *  
 *  @param Buffer Указатель на буфер.
 *  @param limit Размер проигрывания в байтах или \b 0, чтобы не проверять.
 *  @param BufLen Размер буфера.
 *  @return \ref OK при удачном завершении.
 *  @return \ref ERROR при ошибке.
 */
int playWaveBuffer(char *Buffer, int limit, int BufLen);

/**
 *  @brief Воспроизведение файла в формате \b WAV.
 *  @details Функция проигрывает звуковой файл типа \b WAV (\b Waveform \b Audio).
 *  
 *  @param name Имя <b>WAV</b>-файла.
 *  @return \ref OK при удачном завершении.
 *  @return \ref ERROR при ошибке.
 */
int playWaveFile(char *name);

/// @}


/**
 *  @name Поддержка MP3
 *  @{
 */

/**
 *  @brief Воспроизведение файла \b MP3.
 *  @details Функция проигрывает звуковой файл в стандарте \b MP3.
 *  Воспроизведение производится в отдельном потоке.
 *  Для работы требуется подключение библиотеки \b libMP3.a.
 *  
 *  @param name Имя <b>MP3</b>-файла.
 *  @return \ref OK при удачном завершении.
 *  @return \ref ERROR при ошибке.
 */
int playMP3File(char *name);

/**
 *  @brief Остановить воспроизведение файла \b MP3.
 *  @details Функция останавливает проигрывание файла в формате \b MP3.
 */
void stopMP3(void);

/// @}



// Звукозапись

/**
 *  @name Запись звука
 *  @{
 */

/**
 *  @brief Начало звукозаписи.
 *  @details Функция включает режим звукозаписи.
 *  
 *  @return \ref OK при удачном завершении.
 *  @return \ref ERROR при ошибке.
 */
int recStart(void);

/**
 *  @brief Остановить звукозапись.
 *  @details Функция выключает режим звукозаписи.
 *  
 *  @return \ref OK при удачном завершении.
 *  @return \ref ERROR при ошибке.
 */
int recStop(void);

/**
 *  @brief Указатель на записанный фрагмент звука.
 *  @details Функция возвращает указатель на записанный фрагмент в формате \b PCM.
 *  
 *  @param len Указатель на длину фрагмента в байтах.
 *  @return Указатель на буфер, содержащий фрагмент,
 *  записанный между двумя вызовами функции.
 */
char *getRecSndBuffer(int *len);
/// @}


/// @privatesection

/**
 *  @brief Инициализация структур данных.
 *  @details Функция готовит звуковую систему используемого процессора к запуску.
 *  
 *  @return \ref OK при удачном завершении, иначе \ref ERROR.
 */
STATUS soundArchInit ();

struct  audio_codec
        {
         char *codecName;
		 int (*playBuffer)(void *buf, int len);
         int  (*setSndFmt)(int chan_num, int sample_rate);
		 int  (*playMute)(void);
         int  (*playStop)(void);
		 int  (*setPlayVol)(int vol);
		 int  (*recStart)(void);
		 int  (*recStop)(void);
		 void *(*getRecBuffer)(int *len);
        }; 


typedef struct audio_codec *(*initSoundFunc)(void);


/**
 *  @brief Инициализация звуковой системы.
 *  
 *  @param initFunc Фунция инициализации, возвращающая массив колбеков.
 *  @deprecated Устаревшая функция инициализации -- будет удалена. Используйте soundInit().
 *  @return OK при успехе, ERROR иначе.
 */
int __attribute__((deprecated)) initSoundSystem(initSoundFunc initFunc);

char *loadSoundFile(char *name, int *size);
char *findSoundBuffer(char *name, int *buflen);


#endif
