/**
 *  @addtogroup groupStdTypes
 *  @{
 * @file stdint.h
 * @brief Целочисленные типы заданного размера.
 * @details См. стандарт C11 7.20.
 * @see <a href="https://www.iso.org/standard/57853.html">C11 standard 7.20</a>.
 *  @}
 */

/*
 * @author Sergey Sokolov
 * @author Dubovich Timofey
*/
#ifndef STDINT_H
#define STDINT_H

/**
 *  @name Типы заданного размера
 *  @{
 */
typedef signed char         int8_t;
typedef signed short        int16_t;
typedef signed int          int32_t;
typedef signed long long    int64_t;
typedef unsigned char       uint8_t;
typedef unsigned short      uint16_t;
typedef unsigned int        uint32_t;
typedef unsigned long long  uint64_t;
/// @}

/**
 *  @name Типы с минимально-заданным размером
 *  @{
 */
typedef signed char         int_least8_t;
typedef signed short        int_least16_t;
typedef signed int          int_least32_t;
typedef signed long long    int_least64_t;
typedef unsigned char       uint_least8_t;
typedef unsigned short      uint_least16_t;
typedef unsigned int        uint_least32_t;
typedef unsigned long long  uint_least64_t;
/// @}

/**
 *  @name Типы с максимальным быстродействием
 *  @{
 */
typedef signed char         int_fast8_t;
typedef signed int          int_fast16_t;
typedef signed int          int_fast32_t;
typedef signed long long    int_fast64_t;
typedef unsigned char       uint_fast8_t;
typedef unsigned int        uint_fast16_t;
typedef unsigned int        uint_fast32_t;
typedef unsigned long long  uint_fast64_t;
/// @}

/**
 *  @name Типы для хранения указателей
 *  @{
 */
typedef int intptr_t;
typedef unsigned int uintptr_t;
/// @}

/**
 *  @name Типы максимального размера
 *  @{
 */
typedef signed long long int    intmax_t;
typedef unsigned long long int  uintmax_t;
/// @}

#include "limits.h"

/**
 *  @name Максимальные и минимальные значения типов, определенных в limits.h
 *  @{
 */
# define INT8_MIN       (SCHAR_MIN)
# define INT16_MIN      (SHRT_MIN)
# define INT32_MIN      (INT_MIN)
# define INT64_MIN      (INT64_C(LLONG_MIN))
# define INT8_MAX       (SCHAR_MAX)
# define INT16_MAX      (SHRT_MAX)
# define INT32_MAX      (INT_MAX)
# define INT64_MAX      (INT64_C(LLONG_MAX))

# define UINT8_MAX      (UCHAR_MAX)
# define UINT16_MAX     (USHRT_MAX)
# define UINT32_MAX     (UINT_MAX)
# define UINT64_MAX     (UINT64_C(ULLONG_MAX))

# define INT_LEAST8_MIN       (SCHAR_MIN)
# define INT_LEAST16_MIN      (SHRT_MIN)
# define INT_LEAST32_MIN      (INT_MIN)
# define INT_LEAST64_MIN      (INT64_C(LLONG_MIN))
# define INT_LEAST8_MAX       (SCHAR_MAX)
# define INT_LEAST16_MAX      (SHRT_MAX)
# define INT_LEAST32_MAX      (INT_MAX)
# define INT_LEAST64_MAX      (INT64_C(LLONG_MAX))

# define UINT_LEAST8_MAX      (UCHAR_MAX)
# define UINT_LEAST16_MAX     (USHRT_MAX)
# define UINT_LEAST32_MAX     (UINT_MAX)
# define UINT_LEAST64_MAX     (UINT64_C(ULLONG_MAX))

# define INT_FAST8_MIN       (SCHAR_MIN)
# define INT_FAST16_MIN      (INT_MIN)
# define INT_FAST32_MIN      (INT_MIN)
# define INT_FAST64_MIN      (INT64_C(LLONG_MIN))
# define INT_FAST8_MAX       (SCHAR_MAX)
# define INT_FAST16_MAX      (INT_MAX)
# define INT_FAST32_MAX      (INT_MAX)
# define INT_FAST64_MAX      (INT64_C(LLONG_MAX))

# define UINT_FAST8_MAX      (UCHAR_MAX)
# define UINT_FAST16_MAX     (UINT_MAX)
# define UINT_FAST32_MAX     (UINT_MAX)
# define UINT_FAST64_MAX     (UINT64_C(ULLONG_MAX))

#define INTPTR_MIN  (INT_MIN)
#define INTPTR_MAX  (INT_MAX)
#define UINTPTR_MAX (UINT_MAX)

#define INTMAX_MIN  (INT64_C(LLONG_MIN))
#define INTMAX_MAX  (INT64_C(LLONG_MAX))
#define UINTMAX_MAX (UINT64_C(ULLONG_MAX))
/// @}

/**
 *  @name Максимальные и минимальные значения некоторых дополнительных типов
 *  @{
 */
#define PTRDIFF_MIN (INT_MIN)
#define PTRDIFF_MAX (INT_MAX)

#define SIG_ATOMIC_MIN (INT_MIN)
#define SIG_ATOMIC_MAX (INT_MAX)

#define SIZE_MAX (UINT_MAX)

#define WCHAR_MIN (INT_MIN)
#define WCHAR_MAX (INT_MAX)

#define WINT_MIN (INT_MIN)
#define WINT_MAX (INT_MAX)
/// @}

/**
 *  @name Макросы для приведения констант к типам с минимально-заданным размером
 *  @{
 */
#define INT8_C(c) c
#define INT16_C(c) c
#define INT32_C(c) c
#define INT64_C(c) c ## LL
#define UINT8_C(c) c
#define UINT16_C(c) c
#define UINT32_C(c) c
#define UINT64_C(c) c ## ULL

#define INTMAX_C(c)   c ## LL
#define UINTMAX_C(c)  c ## ULL
/// @}



#endif //STDINT_H
