/**
 *  @addtogroup groupStdTypes
 *  @{
 *  @file stdlib.h
 *  @brief Стандартная библиотека.
 *  @details
 *  @see <a href="https://www.iso.org/standard/57853.html">C11 standard 7.22</a>.
 *  @}
 */

/*
 *  @author Sergey Sokolov
 *  @author Dubovich Timofey
 */

#ifndef _STDLIB
#define _STDLIB

#ifdef __cplusplus
extern "C" {
#endif

#include "limits.h"   // INT_MAX.
#include "stdbool.h"  // bool, true, false.
#include "stddef.h"   // size_t, NULL, wchar_t.
#include "setjmp.h"
#include "tasklib.h"

//----------------------------------------------------

/**
 *  @name Преобразования чисел
 *  @{
 */
/**
 * Преобразовать начальную часть строки в double-представление.
 * @param nptr Указатель на строку.
 * @return Преобразованное значение.
 */
double atof (const char *nptr);

/**
 * Преобразовать начальную часть строки в int-представление.
 * @param nptr Указатель на строку.
 * @return Преобразованное значение.
 */
int atoi (const char *nptr);

/**
 * Преобразовать начальную часть строки в long-представление.
 * @param nptr Указатель на строку.
 * @return Преобразованное значение.
 */
long atol (const char *nptr);

#if __STDC_VERSION__ >= 199901L  // C99 or higher
/**
 * Преобразовать начальную часть строки в long-long-представление.
 * @param nptr Указатель на строку.
 * @return Преобразованное значение.
 */
long long atoll (const char *nptr);
#endif

/**
 * Преобразовать начальную часть строки в double-представление.
 * @param nptr Указатель на строку.
 * @param[out] endptr Опциональный буфер для возврата указателя на символ в строке,
 * следующий после преобразованной строки.
 * @return Преобразованное значение.
 */
double strtod (const char *restrict nptr, char **restrict endptr);

/**
 * Преобразовать начальную часть строки в float-представление.
 * @param nptr Указатель на строку.
 * @param[out] endptr Опциональный буфер для возврата указателя на символ в строке,
 * следующий после преобразованной строки.
 * @return Преобразованное значение.
 */
float strtof (const char *restrict nptr, char **restrict endptr);

/**
 * Преобразовать начальную часть строки в long-double-представление.
 * @param nptr Указатель на строку.
 * @param[out] endptr Опциональный буфер для возврата указателя на символ в строке,
 * следующий после преобразованной строки.
 * @return Преобразованное значение.
 */
long double strtolf (const char *restrict nptr, char **restrict endptr);

/**
 * Преобразовать начальную часть строки в long-представление.
 * @param nptr Указатель на строку.
 * @param[out] endptr Опциональный буфер для возврата указателя на символ в строке,
 * следующий после преобразованной строки.
 * @param base Основание системы исчисления.
 * @return Преобразованное значение.
 */
long strtol (const char *nptr, char **endptr, int base);

/**
 * Преобразовать начальную часть строки в long-long-представление.
 * @param nptr Указатель на строку.
 * @param[out] endptr Опциональный буфер для возврата указателя на символ в строке,
 * следующий после преобразованной строки.
 * @param base Основание системы исчисления.
 * @return Преобразованное значение.
 */
long long strtoll (const char *nptr, char **endptr, int base);

/**
 * Преобразовать начальную часть строки в unsigned-long-представление.
 * @param nptr Указатель на строку.
 * @param[out] endptr Опциональный буфер для возврата указателя на символ в строке,
 * следующий после преобразованной строки.
 * @param base Основание системы исчисления.
 * @return Преобразованное значение.
 */
unsigned long strtoul (const char *nptr, char **endptr, int base);

/**
 * Преобразовать начальную часть строки в unsigned-long-long-представление.
 * @param nptr Указатель на строку.
 * @param[out] endptr Опциональный буфер для возврата указателя на символ в строке,
 * следующий после преобразованной строки.
 * @param base Основание системы исчисления.
 * @return Преобразованное значение.
 */
unsigned long long strtoull (const char *nptr, char **endptr, int base);
/// @}

//----------------------------------------------------
/**
 *  @name Генерация псевдо-случайных чисел
 *  @{
 */
/**
 * @brief Максимально возможное значение, возвращаемое функцией rand().
 * @details Максимально возможное значение, возвращаемое функцией rand().
 */
#define RAND_MAX (INT_MAX)  // TODO check!

/**
 * Получить псевдослучайное число.
 * @return Псевдослучайное число в промежутке [0, RAND_MAX].
 */
int rand (void);

/**
 * Установить 'зерно' для псевдослучайной последовательности.
 * @param seed 'Зерно' для псевдослучайной последовательности
 */
void srand (unsigned int seed);
/// @}

//----------------------------------------------------
// Управление памятью.
#include "memlib.h"  // malloc, calloc, realloc, free, aligned_alloc

//----------------------------------------------------
/**
 *  @name Коммуникация с окружением
 *  @{
 */
/**
 * @brief Значение нештатного завершения для функции exit.
 * @details Значение успешного завершения для функции exit.
 */
#define EXIT_FAILURE (1)

/**
 * @brief Значение успешного завершения для функции exit.
 * @details Значение успешного завершения для функции exit.
 */
#define EXIT_SUCCESS (0)

/**
 * Вызвать нештатное завершение текущего процесса.
 */
void abort (void);

/**
 * Установить функцию, которая должна будет быть вызвана в случае завершения процесса.
 * @param func Указатель на функцию.
 * @return 0 при успешной регистрации функции, иное при ошибке.
 * @details Отличия от стандартной реализации: функции будут вызваны даже
 * при нештатном завершении процесса.
 */
int atexit (void (*func) (void));

/**
 * Вызывать завершение текущего процесса.
 * @param status Статус завершения процесса.
 * @details Отличия от стандартной реализации: потоки и дескрипторы,
 * открытые процессом, не будут закрыты/обновлены.
 */
void exit (int status);

/**
 * Вызывать завершение текущего процесса без подчистки дескрипторов и потоков.
 * @param status Статус завершения процесса.
 */
void _Exit (int status);

#if __STDC_VERSION__ >= 201112L  // C11 or higher
/**
 * Установить функцию, которая должна будет быть вызвана в случае быстрого завершения процесса.
 * @param func Указатель на функцию.
 * @return 0 при успешной регистрации функции, иное при ошибке.
 * @details Отличия от стандартной реализации: функции будут вызваны даже при нештатном завершении процесса.
 * @warning РЕАЛИЗАЦИЯ ФУНКЦИИ ОТСУТСТВУЕТ.
 */
int at_quick_exit (void (*func) (void));

/**
 * Вызывать быстрое завершение текущего процесса.
 * @param status Статус завершения процесса.
 * @details Отличия от стандартной реализации: потоки и дескрипторы, открытые процессом, не будут закрыты/обновлены.
 * @warning РЕАЛИЗАЦИЯ ФУНКЦИИ ОТСУТСТВУЕТ.
 */
void quick_exit (int status);
#endif

/**
 * Установить точку сохранения контекста для завершения задачи.
 * @param env Буфер для сохранения контекста.
 * @return 0x80000000, если функция установила точку, иное значение, если возврат функции был вызван вызовом exit().
 */
int setexit (jmp_buf env);

/**
 * Получить код завершения задачи.
 * @return Текущий код завершения задачи
 */
int exitcode (void);

/**
 * Получить текущий буфер возврата для задачи.
 * @return Текущий буфер возврата для задачи
 */
jmp_buf *exitbuf (void);



/**
 * Получить переменную окружения по её имени.
 * @param name Имя переменной окружения.
 * @return Указатель на переменную окружения или \ref NULL,
 * если переменная не была найдена.
 */
char *getenv (const char *name);

/**
 * Выполнить команду системе.
 * @param string Команда системе или \ref NULL.
 * @return Если string - \ref NULL, то не-0, если исполнение команд доступно и 0 иначе.
 * Если string - не \ref NULL, то функция возвращает возврат команды.
 * @warning РЕАЛИЗАЦИЯ ФУНКЦИИ ОТСУТСТВУЕТ.
 */
int system (const char *string);
/// @}

//----------------------------------------------------
/**
 *  @name Поиск и сортировка
 *  @{
 */
/**
 * Выполнить бинарный поиск по отсортированному массиву.
 * @param key Ключ поиска.
 * @param base Указатель на первый элемент массива.
 * @param nmemb Количество элементов в массиве.
 * @param size Размер одиночного элемента в массиве.
 * @param compar Функция, сравнивающая ключ поиска с элементами массива.
 * @details Функция из аргумента compar должна возвращать значение <0, если элемент меньше ключа, ==0,
 * если он равен ключу, и >0, если он больше ключа.
 */
void *bsearch (const void *key, const void *base, size_t nmemb, size_t size, int (*compar) (const void *keyPtr, const void *arrayElementPtr));

/**
 * Отсортировать элементы массива.
 * @param base Указатель на первый элемент массива.
 * @param nmemb Количество элементов в массиве.
 * @param size Размер одиночного элемента в массиве.
 * @param compar Функция сравнения.
 * @details Функция из аргумента compar должна возвращать значение <0, если первый аргумент меньше второго,
 * ==0, если первый аргумент равен второму, и >0, если первый аргумент больше второго.
 */
void qsort (void *base, size_t nmemb, size_t size, int (*compar) (const void *, const void *));
/// @}

//----------------------------------------------------
/**
 *  @name Целочисленная арифметика
 *  @{
 */
/**
 * Вычислить абсолютное значение целого.
 * @param i Целое.
 * @return Абсолютное значение целого.
 */
int abs (int i);

/**
 * Вычислить абсолютное значение целого.
 * @param i Целое.
 * @return Абсолютное значение целого.
 */
long int labs (long int i);

/**
 * Вычислить абсолютное значение целого.
 * @param i Целое.
 * @return Абсолютное значение целого.
 */
long long int llabs (long long int i);

/**
 *  @brief Результат деления с остатком.
 *  @details Результат деления с остатком.
 *  
 */
typedef struct
{
    int quot;   ///< @details Частное от деления.
    int rem;    ///< @details Остаток от деления.
} div_t;

/**
 * Деление целых чисел с остатком.
 * @param numer Делимое.
 * @param denom Делитель.
 * @return Структура из частного и остатка.
 */
div_t div (int numer, int denom);

/**
 *  @brief Результат деления чисел типа \b long \b long с остатком.
 *  @details Результат деления чисел типа \b long \b long с остатком.
 *  
 */
typedef struct
{
    long quot;  ///< @details Частное от деления.
    long rem;   ///< @details Остаток от деления.
} ldiv_t;

/**
 * Деление целых чисел \b long \b long с остатком.
 * @param numer Делимое.
 * @param denom Делитель.
 * @return Структура из частного и остатка.
 */
ldiv_t ldiv (long int numer, long int denom);

#if __STDC_VERSION__ >= 199901L  // C99 or higher
//! Возврат функции lldiv.
typedef struct
{
    long long quot;
    long long rem;
} lldiv_t;

/**
 * Разделить два целых числа и вычислить частное и остаток.
 * @param numer Делимое.
 * @param denom Делитель.
 * @return Структура из частного и остатка.
 */
lldiv_t lldiv (long long int numer, long long int denom);
#endif
/// @}

//----------------------------------------------------
// Преобразование широких символов.

//#define MB_CUR_MAX (4)

// int mblen(const char *s, size_t n);
// int mbtowc(wchar_t * restrict pwc, const char * restrict s, size_t n);
// int wctomb(char *s, wchar_t wc);

//----------------------------------------------------
// Преобразование строк из широких символов.

// size_t mbstowcs(wchar_t * restrict pwcs, const char * restrict s,size_t n);
// size_t wcstombs(char * restrict s, const wchar_t * restrict pwcs,size_t n);

//----------------------------------------------------
/**
 *  @name Нестандартные дополнительные функции
 *  @{
 */
/**
 * Установить переменную окружения.
 * @param name Имя переменной окружения.
 * @param val Значение переменной окружения.
 * @return 0 при успехе, не-0 иначе.
 */
int setenv (const char *name, const char *val);

/**
 * Распечатать целое число в строковый буфер.
 * @param N Целое.
 * @param strptr Буфер.
 */
void itoa (int N, char *strptr);

/**
 * Распечатать целое число в строковый буфер.
 * @param N Целое.
 * @param strptr Буфер.
 */
void lltoa (long long N, char *strptr);

/**
 * Распечатать целое число в строковый буфер.
 * @param N Целое.
 * @param strptr Буфер.
 */
void uitoa (unsigned int N, char *strptr);

/**
 * Распечатать целое число в строковый буфер.
 * @param N Целое.
 * @param strptr Буфер.
 */
void ulltoa (unsigned long long N, char *strptr);

/**
 * Конвертация числа с плавающей точкой в строку заданной длинны.
 * @param value Конвертируемый параметр.
 * @param ndig Количество значащих цифр, которое должно быть сохранено.
 * @param buf Буфер для сохранения результата.
 * @return Указатель на строку с результатом.
 */
char *gcvt (double value, int ndig, char *buf);

/**
 * Проверить, является ли символ цифрой в заданной системе счисления.
 * @param c Символ.
 * @param base Разрядность системы счисления (от 1 до 36).
 * @return 0, если не является, не-0 иначе.
 */
int isdigitex (char c, int base);

/**
 * Выделение слова с номером N из строки Src и помещение его в строку Dst.
 * @param n Номер слова, начиная с 1.
 * @param src Исходная строка.
 * @param dst Буфер под слово.
 * @return 0 при ошибке, 1, если слово было успешно записано в буфер, 2, если слово было успешно записано в буфер и оно изначально было обрамлено кавычками.
 */
int getWord (size_t n, const char *src, char *dst);

/**
 * Инвертированный strcmp.
 * @param S1,S2 Строки для сравнения.
 * @return true, если строки равны, false иначе.
 */
bool compareStr (char *S1, char *S2);

/**
 * Перевести число из формата BCD в обычный формат.
 * @param d Число в формате BCD.
 * @return Число в обычном формате.
 * @note BCD - binary-coded decimal, двоично-десятичный код.
 */
int bcd2d (int d);

/**
 * Перевести число из обычного формата в формат BCD.
 * @param d Число в обычном формате.
 * @return Число в формате BCD.
 * @note BCD - binary-coded decimal, двоично-десятичный код.
 */
int d2bcd (int d);
/// @}







#ifdef __cplusplus
}
#endif

#endif
