/**
 *  @addtogroup groupStdTypes
 *  @{
 *  @file string.h
 *  @brief Работа с массивами символов.
 *  @details
 *  @see <a href="https://www.iso.org/standard/57853.html">C11 standard 7.24</a>.
 *  @}
 */

/*
 *  @author Sergey Sokolov
 *  @author Dubovich Timofey
 */
#ifndef _STRING
#define _STRING

#ifdef __cplusplus
extern "C" {
#endif

#include "stddef.h"  // size_t, NULL.

//----------------------------------------------------
/**
 *  @name Стандартные функции для работы с блоками памяти
 *  @{
 */
/**
 * Скопировать n байт из объекта по указателю s1 в объект по указателю s1.
 * @param s1 Указатель на объект, в который производится копирование.
 * @param s2 Указатель на объект, из которого производится копирование.
 * @param n Количество байт для копирования.
 * @return Значение s1.
 * @warning Объекты не должны пересекаться. В случае пересечения объектов следует использовать функцию memmove.
 */
void *memcpy (void *restrict s1, const void *restrict s2, size_t n);

/**
 * Скопировать n байт из объекта по указателю s1 в объект по указателю s1. Объекты могут пересекаться.
 * @param s1 Указатель на объект, в который производится копирование.
 * @param s2 Указатель на объект, из которого производится копирование.
 * @param n Количество байт для копирования.
 * @return Значение s1.
 */
void *memmove (void *s1, const void *s2, size_t n);

/**
 * Сравнить первые n байт объекта по указателю s1 с байтами объекта по указателю s2.
 * @param s1,s2 Указатели на объекты для сравнения.
 * @param n Количество байт для сравнения.
 * @return 0, если объекты равны, >0, если s1 > s2, <0, если s2 < s1.
 */
int memcmp (const void *s1, const void *s2, size_t n);

/**
 * Найти первое вхождение символа c (приведенного к unsinged char) в первых n байт объекта по указателю s.
 * @param s Указатель на объект, в котором проводится поиск.
 * @param c Искомый символ.
 * @param n Количество байт для поиска.
 * @return Указатель на найденный символ в объекте или \ref NULL, если символ не был найден.
 */
void *memchr (const void *s, int c, size_t n);

/**
 * Скопировать символ c (приведенный к unsinged char) в каждый из первых n байтов объекта по указателю s.
 * @param s Указатель на заполняемый объект.
 * @param c Символ для копирования.
 * @param n Количество байт для заполнения.
 * @return Значение s.
 */
void *memset (void *s, int c, size_t n);

/**
 *  @brief Поиск первого вхождения значения в области памяти.
 *  @details Функция возвращает адрес первого вхождения \b c 
 *  или на 1 байт дальше области памяти, если \b c не найден.
 *  
 *  @param addr Начальный адрес.
 *  @param c Искомое значение.
 *  @param size Размер области памяти.
 *  @return void* Адрес -- указатель на найденный символ.
 */
void *memscan(void * addr, int c, size_t size);

/// @}

//----------------------------------------------------
/**
 *  @name Стандартные функции для работы с массивами символов
 *  @{
 */
/**
 * Скопировать строку по указателю s2 в буфер по указателю s1.
 * @param s1 Указатель на массив, в который будет производится копирование.
 * @param s2 Копируемая строка.
 * @return Значение s1.
 * @warning Объекты не должны пересекаться.
 */
char *strcpy (char *restrict s1, const char *restrict s2);

/**
 * Скопировать не более чем n символов из строки по указателю s2 в буфер по указателю s1.
 * @param s1 Указатель на массив, в который будет производится копирование.
 * @param s2 Копируемая строка.
 * @param n Количество символов для копирования.
 * @return Значение s1.
 * @details В случае, если размер копируемой строки меньше n, то остаток байт в буфере-массиве будет заполнен нулевыми байтами.
 * @warning Объекты не должны пересекаться.
 */
char *strncpy (char *restrict s1, const char *restrict s2, size_t n);

/**
 * Присоединить копию строки по указателю s2 (включая нулевой символ) в конец строки по указателю s1.
 * @param s1 Дополняемая строка.
 * @param s2 Копируемая строка.
 * @return Значение s1.
 * @warning Объекты не должны пересекаться.
 */
char *strcat (char *restrict s1, const char *restrict s2);

/**
 * Присоединить не более n символов из строки по указателю s2 (включая нулевой символ) в конец строки по указателю s1.
 * @param s1 Дополняемая строка.
 * @param s2 Копируемая строка.
 * @param n Количество символов для копирования.
 * @return Значение s1.
 * @warning Объекты не должны пересекаться.
 */
char *strncat (char *restrict s1, const char *restrict s2, size_t n);

/**
 * Сравнить строку по указателю s1 со строкой по указателю s2.
 * @param s1,s2 Указатели на строки для сравнения.
 * @return 0 -- если строки равны, >0 -- если s1 > s2, <0 -- если s1 < s2.
 */
int strcmp (const char *s1, const char *s2);

/**
 * Сравнить строку по указателю s1 со строкой по указателю s2, с учетом текущей локали.
 * @param s1,s2 Указатели на строки для сравнения.
 * @return 0, если строки равны, >0, если s1 > s2, <0, если s2 < s1.
 * @warning РЕАЛИЗАЦИЯ ФУНКЦИИ ОТСУТСТВУЕТ.
 */
int strcoll (const char *s1, const char *s2);

/**
 * Сравнить не более n символов из строки по указателю s1 с символами из строки по указателю s2.
 * @param s1,s2 Указатели на строки для сравнения.
 * @param n Количество символов для сравнения.
 * @return 0, если строки равны, >0, если s1 > s2, <0, если s1 < s2.
 */
int strncmp (const char *s1, const char *s2, size_t n);

/**
 * Преобразовать строку с учетом локали.
 * @details Преобразовать строку по указателю s2 таким образом, чтобы ее можно было использовать функцией strcmp(),
 * и поместить результат преобразования в строку по указателю s2. После преобразования результат вызова функции strcmp(),
 * использующей параметр s1, будет совпадать с результатом вызова функции strcoll(), использующей исходную строку,
 * на которую указывает параметр s2. В массив, адресуемый параметром s1, записывается не более n символов.
 * @param s1 Указатель на массив, в который будет производится преобразование.
 * @param s2 Преобразуемая строка.
 * @param n Количество символов для преобразования.
 * @return Длинна преобразованной строки без нулевого символа.
 * @warning РЕАЛИЗАЦИЯ ФУНКЦИИ ОТСУТСТВУЕТ.
 */
size_t strxfrm (char *restrict s1, const char *restrict s2, size_t n);

/**
 * Найти первое вхождение символа c в строке по указателю s.
 * @param s Указатель на строку, в которой проводится поиск.
 * @param c Искомый символ.
 * @return Указатель на первый найденный символ в строке или \ref NULL, если символ не был найден.
 */
char *strchr (const char *s, int c);

/**
 * Найти первое вхождение в строку по указателю s1 любого из символов из строки по указателю s2.
 * @param s1 Указатель на строку, в которой выполняется поиск.
 * @param s2 Указатель на строку, содержащую символы для поиска.
 * @return Количество символов до первого вхождения.
 */
size_t strcspn (const char *s1, const char *s2);

/**
 * Найти первое вхождение в строку по указателю s1 любого из символов из строки по указателю s2.
 * @param s1 Указатель на строку, в которой выполняется поиск.
 * @param s2 Указатель на строку, содержащую символы для поиска.
 * @return Указатель на первое вхождение в строке по указателю s1, или \ref NULL, при его отсутствии.
 */
char *strpbrk (const char *s1, const char *s2);

/**
 * Найти последнее вхождение символа c в строке по указателю s.
 * @param s Указатель на строку, в которой проводится поиск.
 * @param c Искомый символ.
 * @return Указатель на последний найденный символ в строке или \ref NULL, если символ не был найден.
 */
char *strrchr (const char *s, int c);

/**
 * Найти длинну начального участка строки по указателю s1, который состоит только из символов из строки по указателю s2.
 * @param s1 Указатель на строку, в которой проводится поиск.
 * @param s2 Указатель на строку, содержащую символы для поиска.
 * @return Длинна подходящего под условия начального участка строки.
 */
size_t strspn (const char *s1, const char *s2);

/**
 * Найти первое вхождение подстроки по указателю s2 в строке по указателю s1.
 * @param s1 Указатель на строку, в которой выполняется поиск.
 * @param s2 Указатель на строку, содержащую подстроку для поиска.
 * @return Указатель на первое вхождение подстроки в строке по указателю s1 или \ref NULL, при его отсутствии.
 */
char *strstr (const char *s1, const char *s2);

/**
 * Найти лексемы в строке по указателю s1, разделенные разделителями из строки по указателю s2.
 * @param s1 Указатель на строку, в которой выполняется поиск. Эта строка будет изменена.
 * @param s2 Указатель на строку, содержащую разделители для поиска.
 * @return Указатель на последнюю найденную лексему в строке или \ref NULL, при её отсутствии.
 */
char *strtok (char *restrict s1, const char *restrict s2);

/**
 * Получить сообщение об ошибке, соответствующее коду ошибки.
 * @param errnum Код ошибки.
 * @return Указатель на строку с кодом ошибки.
 * @warning Текущая реализация тривиальна.
 */
char *strerror (int errnum);

/**
 * Расчитать длинну строки по указателю s.
 * @param s Указатель на строку, длинна которой должна быть посчитана.
 * @return Количество символов в строке, без учета нуль-терминатора.
 */
size_t strlen (const char *s);
/// @}

//----------------------------------------------------------------------------------
/**
 *  @name Нестандартные функции для работы с массивами символов (расширения POSIX и т.п.)
 *  @{
 */
/**
 * Реентерабельная версия strtok.
 * @param s Указатель на строку, в которой выполняется поиск. Эта строка будет изменена.
 * @param delim Указатель на строку, содержащую разделители для поиска.
 * @param lasts Указатель на переменную char*, которая используется для учета контекста.
 * @return Указатель на последнюю найденную лексему в строке или \ref NULL, при её отсутствии.
 */
char *strtok_r (char *s, const char *delim, char **lasts);

/**
 * Сделать копию строки в динамической памяти.
 * @param s Указатель на строку для копирования.
 * @return Копия строки в динамичеcкой памяти.
 */
char *strdup (const char *s);

/**
 * @details Псевдоним для stricmp.
 */
#define strcmpi(x, y) stricmp (x, y)

/**
 * Сравнить строку по указателю s1 со строкой по указателю s2 без различия между заглавными и строчными символами.
 * @param s1,s2 Указатели на строки для сравнения.
 * @return 0, если строки равны, >0, если s1 > s2, <0, если s2 < s1.
 */
int stricmp (const char *s1, const char *s2);

/**
 * Найти первое вхождение подстроки по указателю s2 в строке по указателю s1 без различия между заглавными и строчными символами.
 * @param s1 Указатель на строку, в которой выполняется поиск.
 * @param s2 Указатель на строку, содержащую подстроку для поиска.
 * @return Указатель на первое вхождение подстроки в строке по указателю s1 или \ref NULL, при его отсутствии.
 */
char *stristr (const char *s1, const char *s2);

/**
 * Расчитать длинну строки по указателю s, но не более чем maxlen.
 * @param s Указатель на строку, длинна которой должна быть посчитана.
 * @param maxlen Максимальное количество символов для проверки.
 * @return Количество символов в строке, без учета нуль-терминатора, или maxlen.
 */
size_t strnlen (const char *s, size_t maxlen);

/**
 * Преобразовать все подходящие символы строки по указателю s в строчные.
 * @param s Указатель на преобразуемую строку.
 * @return Значение s.
 */
char *strlwr (char *s);

/**
 * Преобразовать все подходящие символы строки по указателю s в заглавные.
 * @param s Указатель на преобразуемую строку.
 * @return Значение s.
 */
char *strup (char *s);

/**
 * Перераспределить строку из динамической памяти, присоеденив к ней другую строку.
 * @param b Указатель на переменную char* с перераспределямым указателем на исходную строку.
 * @param s Копируемая строка.
 * @return Длинна новой строки без учета нуль-терминатора.
 */
size_t merge (char **b, const char *s);

/**
 * Извлечь элемент из строки и скорректировать указатель на строку.
 * @param s Указатель на переменную char* с указателем на исходную строку.
 * @param ct Указатель на строку, содержащую разделители для поиска.
 * @return Исходное значение s.
 */
char *strsep (char **s, const char *ct);

/**
 * Скопировать строку по указателю s2 в буфер по указателю s1.
 * @param s1 Указатель на массив, в который будет производится копирование.
 * @param s2 Копируемая строка.
 * @return Конец строки s1 (т.е. адрес нуль-терминатора).
 * @warning Объекты не должны пересекаться.
 */
char *stpcpy (char *restrict s1, const char *restrict s2);

// Нестандартные безопасные аналоги функций для работы со строками
// Желательно при использовании проверять усечние в стиле (retval >= dsize)

/**
 * Безопасный аналог strcpy. По-сути strncpy без заполнения нулями, который ВСЕГДА ставит нуль-терминатор (кроме строки 0го размера).
 * @param dst Указатель на массив, в который будет производится копирование.
 * @param src Копируемая строка.
 * @param dsize Размер буфера dst.
 * @return Размер копируемой строки. Если возврат >= dsize, то строка не была скопирована полностью.
 */
size_t strlcpy (char *dst, const char *src, size_t dsize);

/**
 * Безопасный аналог strcat. По-сути strncat, который ВСЕГДА ставит нуль-терминатор (кроме строки 0го размера).
 * @param dst Указатель на массив, в который будет производится копирование.
 * @param src Копируемая строка.
 * @param dsize Размер буфера dst.
 * @return  Размер копируемой строки + MIN(dsize, размер начальной строки). Если возврат >= dsize, то строка не была скопирована полностью.
 */
size_t strlcat (char *dst, const char *src, size_t dsize);

/**
 * @details Альтернатива memset из BCD.
 */
#define bzero(addr, size) memset (addr, 0, size);

/**
 * Псевдоним для toupper.
 * @param c Символ для преобразования.
 * @return Преобразованный символ.
 */
char upcase (unsigned char c);

/**
 * Псевдоним для tolower.
 * @param c Символ для преобразования.
 * @return Преобразованный символ.
 */
char lowcase (unsigned char c);
/// @}

//----------------------------------------------------------------------------------

#include "uchar.h"  // char16_t.
// String-functions analogs for char16_t
// Name format str*** -> str16***

/**
 *  @name Функции для работы с char16_t нуль-терминированными строками
 *  @{
 */
/**
 * strlcpy для char16_t-строк.
 * @param dst Указатель на массив, в который будет производится копирование.
 * @param src Копируемая строка.
 * @param dsize Размер буфера dst в char16_t символах.
 * @return Размер копируемой строки. Если возврат >= dsize, то строка не была скопирована полностью.
 */
size_t str16lcpy (char16_t *dst, const char16_t *src, size_t dsize);

/**
 * strlcat для char16_t-строк.
 * @param dst Указатель на массив, в который будет производится копирование.
 * @param src Копируемая строка.
 * @param dsize Размер буфера dst в char16_t символах.
 * @return  Размер копируемой строки + MIN(dsize, размер начальной строки). Если возврат >= dsize, то строка не была скопирована полностью.
 */
size_t str16lcat (char16_t *dst, const char16_t *src, size_t dsize);

/**
 * strcmp для char16_t-строк.
 * @param s1,s2 Указатели на строки для сравнения.
 * @return 0, если строки равны, >0, если s1 > s2, <0, если s2 < s1.
 */
int str16cmp (const char16_t *s1, const char16_t *s2);

/**
 * strlen для char16_t-строк.
 * @param str Указатель на строку, длинна которой должна быть посчитана.
 * @return Количество символов в строке, без учета нуль-терминатора.
 */
size_t str16len (const char16_t *str);

/**
 * strdup для char16_t-строк.
 * @param str Указатель на строку для копирования.
 * @return Копия строки в динамичеcкой памяти.
 */
char16_t *str16dup (const char16_t *str);

/**
 * strchr для char16_t-строк.
 * @param str Указатель на строку, в которой проводится поиск.
 * @param chr Искомый символ.
 * @return Указатель на первый найденный символ в строке или \ref NULL, если символ не был найден.
 */
char16_t *str16chr (const char16_t *str, char16_t chr);
/// @}

/**
 *  @name Преобразования чисел в строки и обратно
 *  @{
 */

/**
 *  @brief Преобразование 32-разрядного числа в строку в 16-тиричном виде.
 *  @details Функция преобразует число в строку в 16-тиричном представлении.
 *  Для строки должно быть отведено достаточное для преобразования место
 *  в памяти.
 *
 *  @param D Исходное число.
 *  @param S Указатель на строку, в которую будет помещено число в текстовом виде.
 */
void intToHex (int D, char *S);

/**
 *  @brief Преобразование 32-разрядного числа
 *  в строку в 16-тиричном виде с форматированием.
 *  @details Функция преобразует число в строку в 16-тиричном представлении.
 *  В отличае от intToHex() функция предоставляет дополнительные возможности
 *  по форматированию итоговой строки.
 *  Для строки должно быть отведено достаточное для преобразования место
 *  в памяти.
 *
 *  @param D Исходное число.
 *  @param S Указатель на строку, в которую будет помещено число в текстовом виде.
 *  @param len Длина итоговой строки. Если длина значащей части меньше ---
 *  строка будет дополнена символами \b spacer.
 *  @param spacer Символ, дополняющий строку спереди до указанной длины \b len.
 *  @param hexSetToUse Набор символов для отображения чисел. В функции
 *  intToHex() используется набор, содержащий заглавные латинские буквы, такое
 *  поведение можно изменить, задав набор символов с прописными буквами.
 */
void intToHexUniversal (int D, char *S, size_t len, char spacer, const char hexSetToUse[static 16]);

/**
 *  @brief Преобразование 16-разрядного числа в строку в 16-тиричном виде.
 *  @details Функция преобразует число в строку в 16-тиричном представлении.
 *  Для строки должно быть отведено достаточное для преобразования место
 *  в памяти.
 *
 *  @param D Исходное число.
 *  @param S Указатель на строку, в которую будет помещено число в текстовом виде.
 */
void shortToHex (unsigned short D, char *S);

/**
 *  @brief Преобразование 8-разрядного числа в строку в 16-тиричном виде.
 *  @details Функция преобразует число в строку в 16-тиричном представлении.
 *  Для строки должно быть отведено достаточное для преобразования место
 *  в памяти.
 *
 *  @param D Исходное число.
 *  @param S Указатель на строку, в которую будет помещено число в текстовом виде.
 */
void charToHex (unsigned char D, char *S);

/**
 *  @brief Преобразование 64-разрядного числа в строку в 16-тиричном виде.
 *  @details Функция преобразует число в строку в 16-тиричном представлении.
 *  Для строки должно быть отведено достаточное для преобразования место
 *  в памяти.
 *
 *  @param D Исходное число.
 *  @param S Указатель на строку, в которую будет помещено число в текстовом виде.
 */
void longlongToHex (long long D, char *S);

/**
 *  @brief Преобразование строки, содержащей 32-разрядное беззнаковое число
 *  в 16-тиричном виде, в значение \b unsigned \b int.
 *  @details Функция преобразует строку в число. Знак игнорируется.
 *  Количество символов не должно превышать 8. Принимаются цифры от \b 0 до \b 9
 *  и латинские буквы от \b a до \b f и от \b A до \b F.
 *
 *  @param S Исходная строка.
 *  @return Результат преобразования.
 */
unsigned int hexToInt (const char *S);

/// @}

/**
 * Расчитать длинну UTF-8 строки по указателю s.
 * @param s Указатель на строку, длинна которой должна быть посчитана.
 * @return Количество символов в строке, без учета нуль-терминатора.
 */
size_t str8len (const char *s);

#ifdef __cplusplus
}
#endif

#endif
