/**
    @addtogroup groupCore 
    @{
    @file tasklib.h
    @brief Управление задачами.
    @details Приведенные в данном файле функции
    пользователь @osName может применять для создания,
    удаления, приостановки, возобновления и задержки задач в многозадачной среде.
    
    @see \ref sectionTaskLibDesc.

    Пример запуска простой задачи:
    @code
    bool stop;

    // Задача в отдельной функции
    int taskSimple (int dummy) {
        // Инициализация переменных задачи (если есть)
        void *p = malloc (size);

        while (!stop) {
            // ... тело задачи
            taskDelay (1);
        }

        // Освобождение выделенной памяти
        free (p);
        // Завершение задачи
        return 0;
    }

    void testSimple () {
        stop = false;
        taskSpawn ("task simple", 10, 0, 0, taskSimple, 0);
    }    
    @endcode

    @}

 */

#ifndef _TASKLIB
#define _TASKLIB

#ifdef __cplusplus
extern "C" {
#endif

// Предварительное объявление структуры. Выполняется ДО подключения прочих заголовочных файлов.
//! @private
typedef struct TCB *TASK_ID;


#include "multex.h"
#include "signal.h"
#include "setjmp.h"

/**
 * Структура элемента списка функций-обработчиков, вызываемых при завершении задачи при помощи \ref exit() и \ref abort().
 */
typedef struct exit_st
{
    void (*exit_fun)();
    struct exit_st *next;
} exit_proc;


/**
 *  @brief Маркер задачи.
 */
#define TASK_MARKER 0x5AA78627
//! @details Флаг создания задачи, укзывающий что в задаче используется математический сопроцессор (вещественные числа).
#define MX_FP_TASK        0x0008
//! @details Алиас MX_FP_TASK для обратной совместимости.
#define VX_FP_TASK        MX_FP_TASK


/**
 * Как именно задача была отпущена с семафора.
 */
typedef enum
{
    tseTimeoutOrNone = 0,       //!< Задача не предпринимала никаких действий с семафорами или семафор не был взят в желаемое время. semTake возвращает \ref ERROR.
    tseTakenFromAnotherTask,    //!< Семафор был отдан освободившей его задачей. semTake возвращает \ref OK.
    tseFlushed,                 //!< Бинарный семафор был отпущен, но не захвачен. semTake возвращает \ref OK.
    tseMutexOrCounterError,     //!< Семафор-счетчик или мьютекс был отпущен, но не захвачен. semTake возвращает \ref ERROR.
}TASK_SEM_EXIT;

/**
 *  @brief Размер служебного стека задачи.
 *  @details Размер служебного стека задачи.
 *  
 */
#define TASK_PS_STACK_SIZE 32

/**
 *  @brief Структура блока управления задачей.
 */
typedef struct TCB {
    int marker;     ///< Маркер-сигнатура задачи.
    int cursp;      ///< Адрес вершины стека.
    int stackSize;  ///< Размер стека.
    void *stack;    ///< Указатель на область памяти, выделенную под стек.
    void *vfparea;  ///< Указатель на данные сопроцессора.
    char *name;     ///< Имя задачи.
    int priority;   ///< Приоритет задачи (0 - 255), чем меньше, тем приоритетнее.
    int delay;      ///< Значение ожидания в тиках.
    int safe;       ///< Флаг защиты от удаления/приостановки.
    void *sem;         ///< Указатель на использующийся семафор.
    int flags;         ///< Служебные флаги задачи.
    unsigned intCounter; ///< Счетчик вложенных прерываний, исполняющихся в контексте задачи.
    int ps_stack[TASK_PS_STACK_SIZE];  ///< Служебный стек.
    int ps_sp;         ///< Счетчик служебного стека.
    int s_in, s_out, s_err;  ///< Стандартные потоки: stdin, stdout, stderr.
    jmp_buf exitbuf;         ///< Точка выхода задачи.
    exit_proc *exit_list;    ///< Список функций-обработчиков, вызываемых по завершении задачи.
    int exit_code;           ///< Код выхода.

    /* Поддержка сигналов */
    int signal;         ///< Полученный задачей сигнал.
    sig_handle sh[32];  ///< Обработчики сигналов.
    sigset_t sa_mask;   ///< Маскированные сигналы.

    /* Поддержка семафоров */
    TASK_SEM_EXIT taskSemExit; ///< Признак снятия задачи с семафора.

    /* Мониторинг активности */
    unsigned long startSecond;            ///< Секунда с момента включения, в которую была запущена данная задача (при запуске через 3 месяца после включения может возникнуть путаница).
    unsigned long workTime;               ///< Микросекунды, в течении которых задача была активна (переполняется за 71 минуту!).
    unsigned long workTimeOverflowCount;  ///< Кол-во переполнений поля workTime.

    /* Передача статуса завершения родительской задаче */
    struct TCB *child;  ///< Дочерняя задача.
    int childstatus;    ///< Статус завершения дочерней задачи.

    struct TCB *parent;  ///< Родительский процесс.

    struct TCB *Next;   ///< Указатель для обслуживания списка задач.
    struct TCB *Prev;   ///< Указатель для обслуживания списка задач.

} taskId;



/**
 *  @brief Отложить выполнение задачи.
 *  @details Функция откладывает выполнение текущей задачи на заданный интервал времени.
 *  
 *  @param ticks Число тиков системного таймера, на которое следует отложить
 *  выполнение текущей задачи. Для задания временных интервалов, не связанных
 *  с частотой системного таймера, воспользуйтесь функцией sysClkRateGet(),
 *  которая вернет количество тиков таймера за одну секунду. Если частота системного
 *  таймера не менялась, то это 1000.
 *  @return STATUS - при вызове в контексте пользовательских задач функция
 *  всегда возвращает значение \ref OK.
 */
STATUS taskDelay(int ticks);

/**
 * @brief Принудительно переключить текущую задачу на другую активную и более приоритетную.
 * @details Текущая задача не будет отложена или задержана. В случае отсутствия более приоритетных активных задач текущая задача продолжит свое выполнение.
 */
void taskSwitch(void);

/**
 *  @brief Создать новую задачу.
 *  @details Функция создает и активизирует новую задачу в среде @osName.
 *  Если создаваемая задача имеет более высокий приоритет, чем та задача,
 *  в контексте которой производится вызов этой функции,
 *  то производится немедленное переключение на эту задачу.
 *  В противном случае выполнение задачи, вызвавшей эту функцию, продолжается.
 *  
 *  @param name Указатель на имя задачи. Имя задачи – произвольная последовательность символов,
 *  заканчивающаяся нулем. Длина имени не ограничена, однако не рекомендуется использовать
 *  имена длиннее 10 символов. После вызова функции строка может быть удалена из памяти.
 *  @param priority Приоритет задачи. Нулевое значение соответствует самому высокому приоритету.
 *  Чем больше значение, тем ниже приоритет. Для совместимости с \b vxWorks
 *  рекомендуется выбирать значения в диапазоне 0 – 255.
 *  @param options Опции создаваемой задачи. Для обычной задачи следует задавать значение 0.
 *  Если в задаче используются операции с вещественными числами и таких задач несколько,
 *  то следует задавать опцию \ref MX_FP_TASK (или \ref VX_FP_TASK, принятую в \b vxWorks
 *  и сохраненную для совместимости). При этом, при каждом переключении с этой задачи на другую,
 *  ядро @osName будет сохранять контекст сопроцессора в специально отводимой
 *  для этой задачи области. При переключении на эту задачу контекст сопроцессора
 *  будет автоматически восстановлен. Однако, следует иметь ввиду, что время переключения задач
 *  при установке такой опции несколько увеличивается.
 *  @param stackSize Размер стекового кадра для создаваемой задачи.
 *  При выборе размера стека следует учитывать глубину вложений всех процедур,
 *  вызываемых в контексте задачи. Прерывания в @osName также выполняются
 *  с использованием стека прерываемой задачи, поэтому следует рассчитывать
 *  наихудший вариант при возможных вложенных прерываниях. Не рекомендуется выбирать
 *  размер стека менее 1000, типичный размер 10000 байт.
 *  При задании большого размера следует учитывать общий объем оперативной памяти:
 *  так, при создании 100 задач с объемом стека в 20000 байт для каждой,
 *  только под стек будет отведено около 2Мбайт памяти.
 *  - \b Важно! Если указан размер стекового кадра, равный 0,
 *      то под него будет выделен размер такой-же, как у задачи, из которой вызывалась эта функция.
 *  @param entryPt Указатель на точку входа в задачу.
 *  Задачей в @osName может быть любая функция типа <em>int Func(int,…)</em>.
 *  При этом, выход из тела функции (return) приведет к завершению этой задачи
 *  и ее автоматическому удалению из памяти.
 *  @param arg Аргумент, передаваемый функции, запускаемой ядром @osName как задача.
 *  @return Идентификатор создаваемой задачи (указатель на \b TCB -
 *  блок управления задачи).Если функция не смогла создать задачу
 *  (например, система не может выделить требуемый объем памяти),
 *  то возвращаемое значение равно \b 0.
 */
TASK_ID taskSpawn(const char *name, int priority, int options, int stackSize, FUNCPTR entryPt, int arg);

/**
 * @brief Создать блок управления задачей.
 * @details Функция выделяет память и подготавливает структуры для дальнейшего использования.
 * @param task Приведенный к int указатель на функцию, являющуюся точкой входа.
 * @param stacksz Размер стека для задачи.
 * @param param Параметр, передаваемый в точку входа.
 * @return Указатель на блок управления задачей.
 */
taskId *taskCreate(int task,int stacksz,int param);

/**
 *  @brief Поиск задачи по имени.
 *  @details По заданному символическому имени задачи функция отыскивает ее идентификатор.
 *  
 *  @param name Указатель на строку c именем задачи.
 *  @return Идентификатор задачи при успешном выполнении или \b NULL в случае неудачи.
 */
TASK_ID taskNameToId(const char *name);


/**
 *  @brief Получить имя задачи.
 *  @details Возвращает символьное имя указанной задачи.
 *  
 *  @param TID Идентификатор задачи.
 *  @return Указатель на строку символов имени задачи,
 *  заканчивающуюся нулем. Если имя не найдено, функция возвращает \b NULL.
 */
const char *taskName(TASK_ID TID);

/**
 *  @brief Получить приоритет задачи.
 *  @details Возвращает приоритет текущей задачи.
 *  
 *  @return Значение приоритета текущей задачи в диапазоне \b 0-255.
 *  - \b Важно! Меньшие значения соответствуют более высокому приоритету!
 */
int taskPriority(void);


/**
 *  @brief Идентификатор текущей задачи.
 *  @details Функция возвращает идентификатор текущей задачи.
 *  
 *  @return Идентификатор текущей выполняемой задачи.
 */
TASK_ID taskIdSelf(void);

/**
 *  @brief Удалить текущую задачу.
 *  @details Функция удаляет текущую задачу. Эквивалентно вызову taskDelete(NULL).
 *  @return Значение \ref OK при успешном выполнении, или \ref ERROR, если задача защищена от удаления функцией \ref taskSafe().
 */
STATUS deleteWorkTask(void);

/**
 *  @brief Удалить задачу.
 *  @details Функция удаляет задачу из системы вне зависимости от ее состояния.
 *
 *  @param TID Идентификатор удаляемой задачи или \ref NULL для удаления текущей задачи.
 *  @return \b 0 в случае успешного удаления.
 *  @return Отрицательное число в случае неудачи. Неудача может возникнуть в случае,
 *  если идентификатор не является указателем на правильный \b TCB задачи,
 *  или если задача защищена от удаления функцией \ref taskSafe().
 */
int taskDelete(TASK_ID TID);

/**
 *  @brief Приостановить выполнение текущей задачи.
 *  @details Функция приостанавливает выполнение указанной задачи. Эквивалентно вызову taskSuspend(NULL)
 *  @return Значение \ref OK при успешном выполнении, или \ref ERROR, если задача защищена от приостановки функцией \ref taskSafe().
 */
STATUS suspendWorkTask(void);

/**
 *  @brief Приостановить выполнение задачи.
 *  @details Функция приостанавливает выполнение указанной задачи.
 *  
 *  @param TID Идентификатор приостанавливаемой задачи или \ref NULL для приостановки текущей задачи.
 *  Для возобновления выполнения приостановленной задачи воспользуйтесь функцией taskResume().
 *  @return Значение \ref OK при успешном выполнении, или \ref ERROR при неудаче
 *  (неверный идентификатор задачи или защита от приостановки функцией \ref taskSafe()).
 */
STATUS taskSuspend(TASK_ID TID);

/**
 *  @brief Возобновить выполнение задачи.
 *  @details Возобновляет выполнение приостановленной задачи.
 *  
 *  @param TID Идентификатор приостановленной задачи.
 *  @return Значение \ref OK при успешном выполнении,
 *  или \ref ERROR при неудаче (неверный идентификатор задачи или задача
 *  не находится в приостановленном состоянии).
 */
STATUS taskResume(TASK_ID TID);
                    
/**
 * Распечатать в stdout таблицу с информацией о задачах.
 * @return Всегда возвращает \ref OK.
 */
STATUS printTasksInfo(void);

// TODO: Добавить ссылку на описание ядра мультекс

/**
 *  @brief Задать режим планировщика ядра.
 *  @details Функция задает режим планировщика задач ядра @osName.
 *  
 *  @param ticks Квант времени в тиках таймера, выделяемый каждой задаче
 *  в режиме карусельного планирования. При этом через интервал времени
 *  \b ticks планировщик автоматически переключит задачу на другую
 *  активную задачу, приоритет которой выше или равен приоритету текущей задачи.
 *  Для задания режима приоритетного планирования следует указать
 *  значение параметра \b ticks равным нулю. При этом текущая задача
 *  будет выполняться до тех пор, пока не активизируется
 *  более высокоприоритетная задача, или текущая задача не будет
 *  приостановлена по собственной инициативе.
 *  @return Всегда \ref OK.
 */
STATUS kernelTimeSlice(int ticks);



/**
 *  @brief Обработчик прерывания системного таймера.
 *  @details Вызов функции обработчика прерывания от системного таймера.
 *  При подключении пользовательской процедуры обработки прерывания
 *  от системного таймера для сохранения работоспособности ядра @osName
 *  в тело этой функции требуется включить вызов tickAnnounce().
 *  Это необходимо, так как общая синхронизация ядра производится системным таймером. *
 */
void tickAnnounce(void);

/**
 *  @brief Инициализация многозадачного ядра.
 *  @details Функция инициализирует многозадачное ядро @osName,
 *  создает и запускает служебные системные задачи и задачу пользователя.
 * 
 *  @warning Эта функция не завершается до выключения системы
 *  и не возвращает управления вызывающей процедуре!
 *  Пользовательские процедуры не должны производить ее повторный вызов.
 *  
 *  @param archInit Функция инициализации конфигурации аппаратной части.
 *  @param rootRtn Корневая задача – задача пользователя
 *  или командный интерпретатор \b Shell.
 *  @param rootMemSize Размер стека, отводимого для корневой задачи.
 *  @param pMemPoolStart Адрес начала свободной оперативной памяти,
 *  выделяемой под \b HEAP-область.
 *  @param pMemPoolEnd Адрес конца \b HEAP-области.
 */
void kernelInit(void (*archInit)(void), FUNCPTR rootRtn, size_t rootMemSize, size_t pMemPoolStart, size_t pMemPoolEnd);

/**
 *  @brief Проверить идентификатор задачи.
 *  @details Функция проверяет правильность указанного идентификатора.
 *  
 *  @param TID Идентификатор задачи.
 *  @return \ref OK – если идентификатор верен.
 *  @return \ref ERROR – если нет.
 */
STATUS taskIdVerify (TASK_ID TID);

/**
 *  @brief Защита задачи.
 *  @details Функция защищает текущую задачу от случайного удаления.
 *  
 *  @return \ref OK при успешном выполнении.
 */
STATUS taskSafe (void);

/**
 *  @brief Снять защиту задачи.
 *  @details Снимает установленную функцией tasklSafe() защиту задачи от случайного удаления.
 *  
 *  @return \ref OK при успешном выполнении.
 */
STATUS taskUnsafe (void);

/**
 *  @brief Принудительное удаление задачи.
 *  @details Удаляет указанную задачу, даже при установленной защите от удаления.
 *  
 *  @param TID Идентификатор удаляемой задачи.
 *  @return Функция возвращает значение \ref OK в случае успешного завершения,
 *  или \ref ERROR в случае неудачи. Неудача может возникнуть в случае,
 *  если идентификатор не является указателем на правильный \b TCB задачи.
 */
STATUS taskDeleteForce (TASK_ID  TID);

/**
 *  @brief Установить приоритет задачи.
 *  @details Устанавливает заданный приоритет для указанной задачи.
 *  
 *  @param TID Идентификатор задачи, которой требуется сменить приоритет.
 *  @param newPriority Требуемое значение приоритета.
 *  @return Функция возвращает значение \ref OK в случае успешного завершения,
 *  или \ref ERROR в случае неудачи. Неудача может возникнуть в случае,
 *  если идентификатор не является указателем на правильный \b TCB задачи.
 *  - \b Важно! Изменение приоритета задачи может привести
 *    к немедленному переключению на нее.
 */
STATUS taskPrioritySet (TASK_ID  TID, int  newPriority);

/**
 *  @brief Получить приоритет задачи.
 *  @details Функция возвращает приоритет указанной задачи.
 *  
 *  @param TID Идентификатор задачи, у которой запрашивается приоритет.
 *  @param pPriority Возвращаемое значение приоритета.
 *  @return Значение \ref OK в случае успешного завершения,
 *  или \ref ERROR в случае неудачи. Неудача может возникнуть в случае,
 *  если идентификатор не является указателем на правильный \b TCB задачи.
 */
STATUS taskPriorityGet (TASK_ID  TID, int  *pPriority);

/**
 *  @brief Запрет переключения текущей задачи.
 *  @details Функция запрещает переключение контекста
 *  с текущей выполняемой задачи на другую.
 *  
 *  @return Всегда \ref OK.
 */
STATUS taskLock (void);

/**
 *  @brief Отмена запрета переключения.
 *  @details Отменяет действие функции taskLock().
 *  
 *  @return Всегда \ref OK.
 */
STATUS taskUnlock (void);        

/**
 * Проверить корректность идентификатора задачи.
 * @param TID Проверяемый идентификатор задачи.
 * @return TID, если идентификатор корректный, или NULL, если не корректный.
 */
TASK_ID taskTcb (TASK_ID TID);

/**
 *  @brief Оценка загрузки процессора в процентах.
 *  @details Функция оценивает значение счётчика простоя процессора
 *  и сравнивает его со значением полученным на ненагруженном процессоре.
 *  
 *  @return Процент загрузки процессора: 0 -- процессор не нагружен,
 *  100 -- процессор загружен полностью.
 */
int cpuUsage ();


//! @privatesection

// Внутренние флаги задачи
//! @details Задача ожидает разблокировки семафора.
#define TASK_PEND         0x01
//! @details Задача отложена.
#define TASK_DELAY        0x02
//! @details Задача приостановлена.
#define TASK_SUSPEND      0x04
//! @details Задача является таймером-вочдогом.
#define TASK_WDT          0x08
//! @details Задача обрабатывает прерывание.
#define TASK_INT_HANDLING 0x10

extern taskId *WorkTask;  ///< Pointer to current TCB.
extern taskId *RootTask;  ///< Pointer to Root Task.
extern taskId *NextTask;  ///< Pointer to next TCB.
extern taskId *IntrTask;
extern taskId *ForkTask;

extern taskId *FirstTaskA;  ///< Очередь активных задач.
extern taskId *FirstTaskD;  ///< Очередь задержанных задач.


/**
 *  @brief Структура базовой информации о задаче.
 */
typedef struct
{
 const char *name;
 int priority;
 int breakpoint;
 int state;
 int delay;
 int in, out;
} taskInfo;

/**
 * @brief Получить указатель на структуру taskInfo с информацией о задаче.
 * @param TID Идентификатор задачи.
 * @return Указатель на структуру taskInfo или \b NULL в случае неудачи.
 * @note Полученную структуру нужно будет освободить, когда в ней пропадет надобность.
 */
taskInfo *getTaskInfo(TASK_ID TID);




#ifdef __cplusplus
}
#endif

         
#endif
