#include <math.h>

#include <ctype.h>
#include <multex.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inifiles.h>

double fabs (double);
int copyFile_Delay (const char *name1, const char *name2, int mode, int delay);
int copyFiles_Delay (const char *name1, const char *name2, int mode, int delay);
int rmdirf(const char *src);
bool iniFileCheckIfSectionExists (INI_FILE iniFileDesc, const char *sectionName);
uint32_t __div64_32(uint64_t *n, uint32_t base);
//-------------------------------
// Generic compare
#define compare_any(val1, val2) _Generic ((val1), \
    double: compare_double,                       \
    float: compare_float,                         \
    long double: compare_ldouble,                 \
    int: compare_int,                             \
    unsigned int: compare_uint,                   \
    long: compare_long,                           \
    unsigned long: compare_ulong,                 \
    long long int: compare_longlong,              \
    unsigned long long int: compare_ulonglong,    \
    void *: compare_ptr,                          \
    char *: compare_str,                          \
    const char *: compare_str,                    \
    div_t: compare_divt,                          \
    ldiv_t: compare_ldivt,                        \
    lldiv_t: compare_lldivt) (val1, val2)

#define declareCompareInteger(name, type) \
    static bool compare_##name (type a, type b) { return a == b; }

#define compareIEEE754(a, b)                                         \
    do {                                                             \
        __typeof__ (a) eps = fabs (a / 10000.0 + b / 10000.0) / 2.0; \
        return (a - eps <= b && a + eps >= b);                       \
    } while (0)
#define declareCompareFloatingPoint(name, type) \
    static bool compare_##name (type a, type b) { compareIEEE754 (a, b); }

declareCompareInteger (int, int);
declareCompareInteger (uint, unsigned int);
declareCompareInteger (long, long);
declareCompareInteger (ulong, unsigned long);
declareCompareInteger (longlong, long long);
declareCompareInteger (ulonglong, unsigned long long);
declareCompareFloatingPoint (float, float);
declareCompareFloatingPoint (double, double);
declareCompareFloatingPoint (ldouble, long double);
declareCompareInteger (ptr, void *);

bool compare_str (const char *a, const char *b) {
    return strcmp (a, b) == 0;
}

bool compare_divt (div_t a, div_t b) {
    return a.quot == b.quot && a.rem == b.rem;
}
bool compare_ldivt (ldiv_t a, ldiv_t b) {
    return a.quot == b.quot && a.rem == b.rem;
}
bool compare_lldivt (lldiv_t a, lldiv_t b) {
    return a.quot == b.quot && a.rem == b.rem;
}

//-------------------------------
// Generic printing

#define sprint_any(val, buf) _Generic ((val), \
    double: print_double,                     \
    float: print_float,                       \
    long double: print_ldouble,               \
    int: print_int,                           \
    unsigned int: print_uint,                 \
    long: print_long,                         \
    unsigned long: print_ulong,               \
    long long int: print_longlong,            \
    unsigned long long int: print_ulonglong,  \
    void *: print_ptr,                        \
    char *: print_str,                        \
    const char *: print_str,                  \
    div_t: print_divt,                        \
    ldiv_t: print_ldivt,                      \
    lldiv_t: print_lldivt) (buf, val)

#define declarePrint(name, type, format) \
    static void print_##name (char *buf, type var) { sprintf (buf, format, var); }
declarePrint (float, float, "%f");
declarePrint (double, double, "%f");
declarePrint (ldouble, double, "%f");  // TODO Our printf currently do not support long double
declarePrint (int, int, "%i");
declarePrint (uint, unsigned int, "%u");
declarePrint (long, long, "%li");
declarePrint (ulong, unsigned long, "%lu");
declarePrint (longlong, long long, "%lli");
declarePrint (ulonglong, unsigned long long, "%llu");
declarePrint (ptr, void *, "%p");
declarePrint (str, const char *, "%s");

void print_divt (char *buf, div_t divVar) {
    sprintf (buf, "Q:%i::R:%i", divVar.quot, divVar.rem);
}
void print_ldivt (char *buf, ldiv_t divVar) {
    sprintf (buf, "Q:%li::R:%li", divVar.quot, divVar.rem);
}
void print_lldivt (char *buf, lldiv_t divVar) {
    sprintf (buf, "Q:%lli::R:%lli", divVar.quot, divVar.rem);
}

void print_unknown (char *buf, ...) {
    sprintf (buf, "ERROR: Unknown type");
}

//-------------------------------
// Additional macro magic

#if defined(__CDT_PARSER__)
#undef compare_any(val, buf)
#define compare_any(val, buf) false

#undef sprint_any(val, buf)
#define sprint_any(val, buf)
#endif

#define TEST(functionCall, expectedResult...)                     \
    do {                                                          \
        __typeof__ (expectedResult) res = (expectedResult);       \
        __typeof__ (expectedResult) ret = (functionCall);         \
        if (!compare_any (ret, res)) {                            \
            char buf1[50], buf2[50];                              \
            sprint_any (ret, buf1);                               \
            sprint_any (res, buf2);                               \
            reportTestError (libName, #functionCall, buf1, buf2); \
        }                                                         \
    } while (0)

#define PRINT_LIB_NAME()                         \
    do {                                         \
        printf ("\n\nTesting %s...\n", libName); \
    } while (0)

static void reportTestError (const char *libName, const char *functionCallStr, const char *result, const char *expectedResult) {
    printf ("%s: Test failed! Function call '%s' returned '%s', expected value: '%s'\n", libName, functionCallStr, result, expectedResult);
}

//-------------------------------
// Tests

static void testCType (void) {
    const char *libName = "ctype";
    PRINT_LIB_NAME ();

    TEST (!isalnum ('0'), !1);
    TEST (isalnum ('~'), 0);

    TEST (!isalpha ('A'), !1);
    TEST (isalpha ('1'), 0);

    TEST (!isblank (' '), !1);
    TEST (isblank ('1'), 0);

    TEST (!iscntrl (0x01), !1);
    TEST (iscntrl ('1'), 0);

    TEST (!isdigit ('1'), !1);
    TEST (isdigit ('A'), 0);

    TEST (!isgraph ('1'), !1);
    TEST (isgraph (' '), 0);
    TEST (isgraph (0x01), 0);

    TEST (!islower ('t'), !1);
    TEST (islower ('T'), 0);

    TEST (!isprint ('1'), !1);
    TEST (isprint (0x01), 0);

    TEST (!ispunct (','), !1);
    TEST (ispunct ('1'), 0);

    TEST (!isspace (' '), !1);
    TEST (!isspace ('\t'), !1);
    TEST (isspace ('1'), 0);

    TEST (!isupper ('T'), !1);
    TEST (isupper ('t'), 0);

    TEST (!isxdigit ('1'), !1);
    TEST (!isxdigit ('a'), !1);
    TEST (!isxdigit ('B'), !1);
    TEST (isxdigit ('Y'), 0);

    TEST (!isascii ('1'), !1);
    TEST (isascii (0x87), 0);

    TEST (tolower ('T'), 't');
    TEST (tolower ('t'), 't');
    TEST (tolower ('0'), '0');
    TEST (tolower (' '), ' ');

    TEST (toupper ('T'), 'T');
    TEST (toupper ('t'), 'T');
    TEST (toupper ('0'), '0');
    TEST (toupper (' '), ' ');

    TEST (toascii (0x41), 'A');
}

static void testStdLib (void) {
    const char *libName = "stdlib";
    PRINT_LIB_NAME ();

    // String to float
    TEST (atof ("0.0"), 0.0);
    TEST (atof ("10.0"), 10.0);
    TEST (atof ("-90.0"), -90.0);
    TEST (atof ("    3.14"), 3.14);
    TEST (atof ("2e0"), 2.0);
    TEST (atof ("3e2"), 300.0);
    TEST (atof ("-4e1"), -40.0);
    TEST (atof ("3e-2"), 0.03);
    TEST (atof ("-4e-1"), -0.4);

    TEST (strtod ("0.0", NULL), 0.0);
    TEST (strtod ("10.0", NULL), 10.0);
    TEST (strtod ("-90.0", NULL), -90.0);
    TEST (strtod ("    3.14", NULL), 3.14);
    TEST (strtod ("2e0", NULL), 2.0);
    TEST (strtod ("3e2", NULL), 300.0);
    TEST (strtod ("-4e1", NULL), -40.0);
    TEST (strtod ("3e-2", NULL), 0.03);
    TEST (strtod ("-4e-1", NULL), -0.4);

    TEST (strtof ("0.0", NULL), 0.0f);
    TEST (strtof ("10.0", NULL), 10.0f);
    TEST (strtof ("-90.0", NULL), -90.0f);
    TEST (strtof ("    3.14", NULL), 3.14f);
    TEST (strtof ("2e0", NULL), 2.0f);
    TEST (strtof ("3e2", NULL), 300.0f);
    TEST (strtof ("-4e1", NULL), -40.0f);
    TEST (strtof ("3e-2", NULL), 0.03f);
    TEST (strtof ("-4e-1", NULL), -0.4f);

    TEST (strtolf ("0.0", NULL), 0.0l);
    TEST (strtolf ("10.0", NULL), 10.0l);
    TEST (strtolf ("-90.0", NULL), -90.0l);
    TEST (strtolf ("    3.14", NULL), 3.14l);
    TEST (strtolf ("2e0", NULL), 2.0l);
    TEST (strtolf ("3e2", NULL), 300.0l);
    TEST (strtolf ("-4e1", NULL), -40.0l);
    TEST (strtolf ("3e-2", NULL), 0.03l);
    TEST (strtolf ("-4e-1", NULL), -0.4l);

    // string to integer
    TEST (atoi ("100"), 100);
    TEST (atoi ("-200"), -200);
    TEST (atoi ("0 10 0"), 0);
    TEST (atoi ("100"), 100);
    TEST (atoi ("-200"), -200);
    TEST (atoi ("  +9999 d"), 9999);

    TEST (atol ("100"), 100l);
    TEST (atol ("-200"), -200l);
    TEST (atol ("0 10 0"), 0l);
    TEST (atol ("100"), 100l);
    TEST (atol ("-200"), -200l);
    TEST (atol ("  +9999 d"), 9999l);

    TEST (atoll ("100"), 100ll);
    TEST (atoll ("-200"), -200ll);
    TEST (atoll ("0 10 0"), 0ll);
    TEST (atoll ("100"), 100ll);
    TEST (atoll ("-200"), -200ll);
    TEST (atoll ("  +9999 d"), 9999ll);

    TEST (strtol ("100", NULL, 10), 100);
    TEST (strtol ("-200", NULL, 10), -200);
    TEST (strtol ("0 10 0", NULL, 10), 0);
    TEST (strtol ("100", NULL, 0), 100);
    TEST (strtol ("-200", NULL, 0), -200);
    TEST (strtol ("  +9999 d", NULL, 0), 9999);
    TEST (strtol ("0xF", NULL, 0), 15);
    TEST (strtol ("020", NULL, 8), 16);

    TEST (strtoll ("100", NULL, 10), 100LL);
    TEST (strtoll ("-200", NULL, 10), -200LL);
    TEST (strtoll ("0 10 0", NULL, 10), 0LL);
    TEST (strtoll ("100", NULL, 0), 100LL);
    TEST (strtoll ("-200", NULL, 0), -200LL);
    TEST (strtoll ("  +9999 d", NULL, 0), 9999LL);
    TEST (strtoll ("0xF", NULL, 0), 15LL);
    TEST (strtoll ("020", NULL, 8), 16LL);

    TEST (strtoul ("100", NULL, 10), 100U);
    TEST (strtoul ("200", NULL, 10), 200U);
    TEST (strtoul ("0 10 0", NULL, 10), 0U);
    TEST (strtoul ("100", NULL, 0), 100U);
    TEST (strtoul ("200", NULL, 0), 200U);
    TEST (strtoul ("  9999 d", NULL, 0), 9999U);
    TEST (strtoul ("0xF", NULL, 0), 15U);
    TEST (strtoul ("020", NULL, 8), 16U);

    TEST (strtoull ("100", NULL, 10), 100ULL);
    TEST (strtoull ("200", NULL, 10), 200ULL);
    TEST (strtoull ("0 10 0", NULL, 10), 0ULL);
    TEST (strtoull ("100", NULL, 0), 100ULL);
    TEST (strtoull ("200", NULL, 0), 200ULL);
    TEST (strtoull ("  9999 d", NULL, 0), 9999ULL);
    TEST (strtoull ("0xF", NULL, 0), 15ULL);
    TEST (strtoull ("020", NULL, 8), 16ULL);

    // rand and srand.
    {
        const int seed = 0x12345678;
        int val;
        srand (seed);
        val = rand ();
        for (size_t i = 0; i < 10; i++) {
            srand (seed);
            TEST (rand (), val);
        }
    }

    // abs and div
    TEST (abs (100), 100);
    TEST (abs (-100), 100);
    TEST (abs (0), 0);
    TEST (abs (INT_MAX), INT_MAX);
    TEST (abs (-INT_MAX), INT_MAX);

    TEST (labs (100L), 100L);
    TEST (labs (-100L), 100L);
    TEST (labs (0L), 0L);
    TEST (labs (LONG_MAX), LONG_MAX);
    TEST (labs (-LONG_MAX), LONG_MAX);

    TEST (llabs (100LL), 100LL);
    TEST (llabs (-100LL), 100LL);
    TEST (llabs (0LL), 0LL);
    TEST (llabs (LLONG_MAX), LLONG_MAX);
    TEST (llabs (-LLONG_MAX), LLONG_MAX);

    TEST (div (10, 5), (div_t){.quot = 2, .rem = 0});
    TEST (div (11, 5), (div_t){.quot = 2, .rem = 1});
    TEST (div (0, 5), (div_t){.quot = 0, .rem = 0});
    TEST (div (-10, 5), (div_t){.quot = -2, .rem = 0});
    TEST (div (-11, 5), (div_t){.quot = -2, .rem = -1});

    TEST (ldiv (10, 5), (ldiv_t){.quot = 2, .rem = 0});
    TEST (ldiv (11, 5), (ldiv_t){.quot = 2, .rem = 1});
    TEST (ldiv (0, 5), (ldiv_t){.quot = 0, .rem = 0});
    TEST (ldiv (-10, 5), (ldiv_t){.quot = -2, .rem = 0});
    TEST (ldiv (-11, 5), (ldiv_t){.quot = -2, .rem = -1});

    TEST (lldiv (10, 5), (lldiv_t){.quot = 2, .rem = 0});
    TEST (lldiv (11, 5), (lldiv_t){.quot = 2, .rem = 1});
    TEST (lldiv (0, 5), (lldiv_t){.quot = 0, .rem = 0});
    TEST (lldiv (-10, 5), (lldiv_t){.quot = -2, .rem = 0});
    TEST (lldiv (-11, 5), (lldiv_t){.quot = -2, .rem = -1});

    // bsearch and qsort
    {
        int iCompare (const void *key, const void *element) { return *(int *)key - *(int *)element; }
        int dCompare (const void *key, const void *element) {
            double dif = *(double *)key - *(double *)element;
            double eps = fabs (*(double *)element / 10000.0) + fabs (*(double *)key / 10000.0);
            if (fabs (dif) < eps) {
                return 0;
            } else if (dif < 0) {
                return -1;
            } else {
                return 1;
            }
        }
        bool arrayWasSorted (const void *base, size_t nmemb, size_t size, int (*compar) (const void *, const void *)) {
            const char *base0 = base;
            for (size_t i = 0; i < nmemb - 1; i++) {
                if (compar (&base0[size * i], &base0[size * (i + 1)]) > 0) {
                    return false;
                }
            }
            return true;
        }

        int arrayI[51];
        double arrayD[ARRAY_SIZE (arrayI)];
        for (size_t i = 0; i < ARRAY_SIZE (arrayI); i++) {
            arrayI[i] = i - ARRAY_SIZE (arrayI) / 2;
            arrayD[i] = ((double)i - ARRAY_SIZE (arrayI) / 2) * 10.0;
        }

        // Try to find 4th and 46th elements
        TEST (bsearch (&arrayI[4], arrayI, ARRAY_SIZE (arrayI), sizeof (int), iCompare), (void *)&arrayI[4]);
        TEST (bsearch (&arrayI[46], arrayI, ARRAY_SIZE (arrayI), sizeof (int), iCompare), (void *)&arrayI[46]);
        TEST (bsearch (&arrayD[4], arrayD, ARRAY_SIZE (arrayD), sizeof (double), dCompare), (void *)&arrayD[4]);
        TEST (bsearch (&arrayD[46], arrayD, ARRAY_SIZE (arrayD), sizeof (double), dCompare), (void *)&arrayD[46]);

        int bufI = 999999;
        double bufD = 100000000.0;
        TEST (bsearch (&bufI, arrayI, ARRAY_SIZE (arrayI), sizeof (int), iCompare), NULL);
        TEST (bsearch (&bufD, arrayD, ARRAY_SIZE (arrayD), sizeof (double), dCompare), NULL);

        for (size_t i = 0; i < ARRAY_SIZE (arrayI); i++) {
            arrayI[i] = rand () % 256 - 128;
            arrayD[i] = (rand () % 2560 - 1280) / 10.0;
        }
        qsort (arrayI, ARRAY_SIZE (arrayI), sizeof (int), iCompare);
        qsort (arrayD, ARRAY_SIZE (arrayD), sizeof (double), dCompare);
        for (size_t i = 0; i < ARRAY_SIZE (arrayI) - 1; i++) {
            if (arrayI[i] > arrayI[i + 1]) {
                TEST (arrayWasSorted (arrayI, ARRAY_SIZE (arrayI), sizeof (int), iCompare), true);
                TEST (arrayWasSorted (arrayD, ARRAY_SIZE (arrayD), sizeof (double), dCompare), true);
            }
        }
    }

    {
        char buffer[64];

        itoa (123, buffer);
        TEST (strcmp (buffer, "123"), 0);
        lltoa (123ll, buffer);
        TEST (strcmp (buffer, "123"), 0);
        uitoa (123U, buffer);
        TEST (strcmp (buffer, "123"), 0);
        ulltoa (123ULL, buffer);
        TEST (strcmp (buffer, "123"), 0);

        gcvt (10.12, 5, buffer);
        TEST (strcmp (buffer, "10.12"), 0);
    }

    {
        TEST (!!isdigitex ('1', 10), !0);
        TEST (isdigitex ('a', 10), 0);
        TEST (isdigitex ('9', 8), 0);
        TEST (!!isdigitex ('a', 16), !0);
        TEST (isdigitex ('g', 16), 0);
        TEST (isdigitex ('!', 10), 0);
    }

    {
        char src[] = "Hello! It's me \"Mario\"";
        char buffer[64];

        TEST (getWord (1, src, buffer), 1);
        TEST (strcmp (buffer, "Hello!"), 0);

        TEST (getWord (2, src, buffer), 1);
        TEST (strcmp (buffer, "It's"), 0);

        TEST (getWord (3, src, buffer), 1);
        TEST (strcmp (buffer, "me"), 0);

        TEST (getWord (4, src, buffer), 2);
        TEST (strcmp (buffer, "Mario"), 0);
    }

    {
        TEST (bcd2d (0b10000), 10);
        TEST (d2bcd (10), 0b10000);
    }
}

#include "fnames.h"
static void testFnames (void) {
    const char *libName = "fnames";
    PRINT_LIB_NAME ();

    {
        const char justFileName[] = "name.txt";
        const char partPath1[] = "folder/name.txt";
        const char partPath2[] = "/folder/name.txt";
        const char fullPath[] = "C:/folder/name.txt";
        TEST (getFileName (justFileName), &justFileName[0]);
        TEST (getFileName (partPath1), &partPath1[7]);
        TEST (getFileName (partPath2), &partPath2[8]);
        TEST (getFileName (fullPath), &fullPath[10]);
    }

    {
        char justFileName[] = "name.txt";
        char partPath1[] = "folder/name.txt";
        char partPath2[] = "/folder/name.txt";
        char fullPath[] = "C:/folder/name.txt";
        TEST (getFileNameNonConst (justFileName), &justFileName[0]);
        TEST (getFileNameNonConst (partPath1), &partPath1[7]);
        TEST (getFileNameNonConst (partPath2), &partPath2[8]);
        TEST (getFileNameNonConst (fullPath), &fullPath[10]);
    }

    {
        char path1[] = "C:/folder/test.txt";
        char path2[] = "C:/test.txt";
        char drive[] = "C:";
        // char folder[] = "C:/folder/";
        char buf[128];

        extractFileDrive (path1, buf);
        TEST (strcmp (buf, drive), 0);
        extractFileDrive (path2, buf);
        TEST (strcmp (buf, drive), 0);

        extractFilePath (path1, buf);
        TEST (strcmp (buf, "/FOLDER/"), 0);

        TEST (strcmp (getWorkDirectory (drive), "/"), 0);

        expandFileName (path1, buf);
        TEST (strcmp (buf, "C:/FOLDER/TEST.TXT"), 0);
        expandFileName ("test.txt", buf);
        TEST (strcmp (buf, "C:/TEST.TXT"), 0);

        extractDevice (path1, buf);
        TEST (strcmp (buf, drive), 0);
    }

    {
        char path1[] = "dir/file.txt";
        char path2[] = "/home/dir/file.txt";
        char root[] = "C:/root/";
        char test1[] = "C:/root/dir/";
        char test2[] = "/home/dir/";
        char buf[128];

        extractFileDevPath (path1, root, buf);
        TEST (strcmp (buf, test1), 0);
        extractFileDevPath (path2, "none", buf);
        TEST (strcmp (buf, test2), 0);

        extractNextDir (path1, buf);
        TEST (strcmp (buf, "dir"), 0);

        fileNamePreprocess (test1, buf, NULL);
        TEST (strcmp (buf, "C:"), 0);
    }

    {
        char file[] = "file.txt";
        char mask[] = "*i*.*t";
        char buf[128];

        TEST (compareNames (mask, file), OK);

        buildFileName (file, "dir/?*.*", buf);
        TEST (strcmp (buf, "C:/DIR/file.txt"), 0);
    }

    // TODO other tests

    // TODO extractFileDevPath  - still don't understand what's the purpose of this.
    // TODO expandFileName - relate on filesystem, required more complicated test?
}

#include "stdio.h"
static void testStdio (void) {
    const char *libName = "stdio";
    PRINT_LIB_NAME ();

    {
        char buffer[64];
        snprintf (buffer, ARRAY_SIZE (buffer), "%d", 0);
        TEST (buffer, (char *)"0");
        snprintf (buffer, ARRAY_SIZE (buffer), "%d", -7);
        TEST (buffer, (char *)"-7");
        snprintf (buffer, ARRAY_SIZE (buffer), "%d", 15056564);
        TEST (buffer, (char *)"15056564");
        snprintf (buffer, ARRAY_SIZE (buffer), "%d", -20456456);
        TEST (buffer, (char *)"-20456456");

        snprintf (buffer, ARRAY_SIZE (buffer), "%5d", 0);
        TEST (buffer, (char *)"    0");
        snprintf (buffer, ARRAY_SIZE (buffer), "%5d", -7);
        TEST (buffer, (char *)"   -7");
        snprintf (buffer, ARRAY_SIZE (buffer), "%5d", 15056564);
        TEST (buffer, (char *)"15056564");
        snprintf (buffer, ARRAY_SIZE (buffer), "%5d", -20456456);
        TEST (buffer, (char *)"-20456456");

        snprintf (buffer, ARRAY_SIZE (buffer), "%-5d", 0);
        TEST (buffer, (char *)"0    ");
        snprintf (buffer, ARRAY_SIZE (buffer), "%-5d", -7);
        TEST (buffer, (char *)"-7   ");
        snprintf (buffer, ARRAY_SIZE (buffer), "%-5d", 15056564);
        TEST (buffer, (char *)"15056564");
        snprintf (buffer, ARRAY_SIZE (buffer), "%-5d", -20456456);
        TEST (buffer, (char *)"-20456456");

        snprintf (buffer, ARRAY_SIZE (buffer), "%05d", 0);
        TEST (buffer, (char *)"00000");
        snprintf (buffer, ARRAY_SIZE (buffer), "%05d", -7);
        TEST (buffer, (char *)"-0007");
        snprintf (buffer, ARRAY_SIZE (buffer), "%05d", 15056564);
        TEST (buffer, (char *)"15056564");
        snprintf (buffer, ARRAY_SIZE (buffer), "%05d", -20456456);
        TEST (buffer, (char *)"-20456456");

        snprintf (buffer, ARRAY_SIZE (buffer), "%+5d", 0);
        TEST (buffer, (char *)"   +0");
        snprintf (buffer, ARRAY_SIZE (buffer), "%+5d", -7);
        TEST (buffer, (char *)"   -7");
        snprintf (buffer, ARRAY_SIZE (buffer), "%+5d", 15056564);
        TEST (buffer, (char *)"+15056564");
        snprintf (buffer, ARRAY_SIZE (buffer), "%+5d", -20456456);
        TEST (buffer, (char *)"-20456456");

        snprintf (buffer, ARRAY_SIZE (buffer), "% -5d", 0);
        TEST (buffer, (char *)" 0   ");
        snprintf (buffer, ARRAY_SIZE (buffer), "% -5d", -7);
        TEST (buffer, (char *)"-7   ");
        snprintf (buffer, ARRAY_SIZE (buffer), "% -5d", 15056564);
        TEST (buffer, (char *)" 15056564");
        snprintf (buffer, ARRAY_SIZE (buffer), "% -5d", -20456456);
        TEST (buffer, (char *)"-20456456");

        snprintf (buffer, ARRAY_SIZE (buffer), "%+05d", 0);
        TEST (buffer, (char *)"+0000");
        snprintf (buffer, ARRAY_SIZE (buffer), "%+05d", -7);
        TEST (buffer, (char *)"-0007");
        snprintf (buffer, ARRAY_SIZE (buffer), "%+05d", 15056564);
        TEST (buffer, (char *)"+15056564");
        snprintf (buffer, ARRAY_SIZE (buffer), "%+05d", -20456456);
        TEST (buffer, (char *)"-20456456");

        snprintf (buffer, ARRAY_SIZE (buffer), "% 05d", 0);
        TEST (buffer, (char *)" 0000");
        snprintf (buffer, ARRAY_SIZE (buffer), "% 05d", -7);
        TEST (buffer, (char *)"-0007");
        snprintf (buffer, ARRAY_SIZE (buffer), "% 05d", 15056564);
        TEST (buffer, (char *)" 15056564");
        snprintf (buffer, ARRAY_SIZE (buffer), "% 05d", -20456456);
        TEST (buffer, (char *)"-20456456");

        snprintf (buffer, ARRAY_SIZE (buffer), "%s", "");
        TEST (buffer, (char *)"");
        snprintf (buffer, ARRAY_SIZE (buffer), "%s", "a");
        TEST (buffer, (char *)"a");
        snprintf (buffer, ARRAY_SIZE (buffer), "%s", "ab");
        TEST (buffer, (char *)"ab");
        snprintf (buffer, ARRAY_SIZE (buffer), "%s", "abcdefg");
        TEST (buffer, (char *)"abcdefg");

        snprintf (buffer, ARRAY_SIZE (buffer), "%5s", "");
        TEST (buffer, (char *)"     ");
        snprintf (buffer, ARRAY_SIZE (buffer), "%5s", "a");
        TEST (buffer, (char *)"    a");
        snprintf (buffer, ARRAY_SIZE (buffer), "%5s", "ab");
        TEST (buffer, (char *)"   ab");
        snprintf (buffer, ARRAY_SIZE (buffer), "%5s", "abcdefg");
        TEST (buffer, (char *)"abcdefg");

        snprintf (buffer, ARRAY_SIZE (buffer), "%-5s", "");
        TEST (buffer, (char *)"     ");
        snprintf (buffer, ARRAY_SIZE (buffer), "%-5s", "a");
        TEST (buffer, (char *)"a    ");
        snprintf (buffer, ARRAY_SIZE (buffer), "%-5s", "ab");
        TEST (buffer, (char *)"ab   ");
        snprintf (buffer, ARRAY_SIZE (buffer), "%-5s", "abcdefg");
        TEST (buffer, (char *)"abcdefg");

        snprintf (buffer, ARRAY_SIZE (buffer), "%f", 2.718281828);
        TEST (buffer, (char *)"2.718282");
        snprintf (buffer, ARRAY_SIZE (buffer), "%.1f", 2.718281828);
        TEST (buffer, (char *)"2.7");
        snprintf (buffer, ARRAY_SIZE (buffer), "%.2f", 2.718281828);
        TEST (buffer, (char *)"2.72");
        snprintf (buffer, ARRAY_SIZE (buffer), "%.6f", 2.718281828);
        TEST (buffer, (char *)"2.718282");
        snprintf (buffer, ARRAY_SIZE (buffer), "%.7f", 2.718281828);
        TEST (buffer, (char *)"2.7182818");

        snprintf (buffer, ARRAY_SIZE (buffer), "%5.1f", 2.718281828);
        TEST (buffer, (char *)"  2.7");
        snprintf (buffer, ARRAY_SIZE (buffer), "%-5.1f", 2.718281828);
        TEST (buffer, (char *)"2.7  ");
        snprintf (buffer, ARRAY_SIZE (buffer), "%+5.1f", 2.718281828);
        TEST (buffer, (char *)" +2.7");
        snprintf (buffer, ARRAY_SIZE (buffer), "%+-5.1f", 2.718281828);
        TEST (buffer, (char *)"+2.7 ");
        snprintf (buffer, ARRAY_SIZE (buffer), "%05.1f", 2.718281828);
        TEST (buffer, (char *)"002.7");
        snprintf (buffer, ARRAY_SIZE (buffer), "%+05.1f", 2.718281828);
        TEST (buffer, (char *)"+02.7");
        snprintf (buffer, ARRAY_SIZE (buffer), "% 05.1f", 2.718281828);
        TEST (buffer, (char *)" 02.7");
        snprintf (buffer, ARRAY_SIZE (buffer), "%- 5.1f", 2.718281828);
        TEST (buffer, (char *)" 2.7 ");

        snprintf (buffer, ARRAY_SIZE (buffer), "%6x", 0x1234);
        TEST (buffer, (char *)"  1234");
        snprintf (buffer, ARRAY_SIZE (buffer), "%-6x", 0x1234);
        TEST (buffer, (char *)"1234  ");
        snprintf (buffer, ARRAY_SIZE (buffer), "%06x", 0x1234);
        TEST (buffer, (char *)"001234");

        snprintf (buffer, ARRAY_SIZE (buffer), "%e", 2.718281828);
        TEST (buffer, (char *)"2.718282e+00");
        snprintf (buffer, ARRAY_SIZE (buffer), "%.1e", 2.718281828);
        TEST (buffer, (char *)"2.7e+00");
        snprintf (buffer, ARRAY_SIZE (buffer), "%.2e", 2.718281828);
        TEST (buffer, (char *)"2.72e+00");
        snprintf (buffer, ARRAY_SIZE (buffer), "%.6e", 2.718281828);
        TEST (buffer, (char *)"2.718282e+00");
        snprintf (buffer, ARRAY_SIZE (buffer), "%.7e", 2.718281828);
        TEST (buffer, (char *)"2.7182818e+00");

        snprintf (buffer, ARRAY_SIZE (buffer), "%15e", 2.718281828);
        TEST (buffer, (char *)"   2.718282e+00");
        snprintf (buffer, ARRAY_SIZE (buffer), "%15.1e", 2.718281828);
        TEST (buffer, (char *)"        2.7e+00");
        snprintf (buffer, ARRAY_SIZE (buffer), "%15.2e", 2.718281828);
        TEST (buffer, (char *)"       2.72e+00");
        snprintf (buffer, ARRAY_SIZE (buffer), "%15.6e", 2.718281828);
        TEST (buffer, (char *)"   2.718282e+00");
        snprintf (buffer, ARRAY_SIZE (buffer), "%15.7e", 2.718281828);
        TEST (buffer, (char *)"  2.7182818e+00");

        snprintf (buffer, ARRAY_SIZE (buffer), "% 015e", 2.718281828);
        TEST (buffer, (char *)" 002.718282e+00");
        snprintf (buffer, ARRAY_SIZE (buffer), "% 015.1e", 2.718281828);
        TEST (buffer, (char *)" 00000002.7e+00");
        snprintf (buffer, ARRAY_SIZE (buffer), "% 015.2e", 2.718281828);
        TEST (buffer, (char *)" 0000002.72e+00");
        snprintf (buffer, ARRAY_SIZE (buffer), "% 015.6e", 2.718281828);
        TEST (buffer, (char *)" 002.718282e+00");
        snprintf (buffer, ARRAY_SIZE (buffer), "% 015.7e", 2.718281828);
        TEST (buffer, (char *)" 02.7182818e+00");
    }

    {
        char format[] = "%c %s %d %x %6f";
        char c = '\0';
        char buffer[64];
        int i = 0;
        int x = 0;
        double f = 0;

        sscanf ("M   itsMeMario2!   123   123 12.345678", format, &c, &buffer, &i, &x, &f);
        TEST (c, 'M');
        TEST (strcmp (buffer, "itsMeMario2!"), 0);
        TEST (i, 123);
        TEST (x, 0x123);
        TEST (f, 12.345);
    }
}

#include "iolib.h"
static void testIolib (void) {
    const char *libName = "iolib";
    PRINT_LIB_NAME ();

    char file1[] = "C:/F1.TXT";
    char file2[] = "C:/F2.TXT";
    char dir1[] = "C:/SOUND";
    char dir2[] = "C:/TEMP";

    TEST (fileExists (file1), true);
    TEST (fileSize (file1), 1001);

    TEST (fileExists (file2), false);
    TEST (fileSize (file2), -1);

    TEST (dirExists (dir1), true);
    TEST (dirExists (dir2), false);
}

static void testStdfio (void) {
    const char *libName = "stdfio";
    PRINT_LIB_NAME ();

    char file1[] = "C:/F1.TXT";
    char file2[] = "C:/F2.TXT";

    TEST (getFileSize (file1), 1001);
    TEST (getFileSize (file2), -1);
}

#include "datetime.h"
static void testDateTime (void) {
    const char *libName = "dateTime";
    PRINT_LIB_NAME ();

    DATE_TIME dt1 = {.Year = 2000, .Month = 1, .Day = 1, .Hour = 1, .Minute = 1, .Second = 0};
    DATE_TIME dt2 = {.Year = 2000, .Month = 1, .Day = 1, .Hour = 1, .Minute = 2, .Second = 30};
    DATE_TIME dt3 = {.Year = 2023, .Month = 8, .Day = 30, .Hour = 10, .Minute = 33, .Second = 41};

    TEST (diffdate (&dt1, &dt1), 0);
    TEST (diffdate (&dt2, &dt1), 90);
    TEST (diffdate (&dt1, &dt2), -90);

    TEST (convertDateTimeTime (&dt3), 1693391621);
}

#include "time.h"
static void testTime (void) {
    const char *libName = "time";
    PRINT_LIB_NAME ();

    struct tm dt1 = {.tm_isdst = 0, .tm_yday = 1, .tm_wday = 6, .tm_year = 100, .tm_mon = 0, .tm_mday = 1, .tm_hour = 1, .tm_min = 1, .tm_sec = 0};
    struct tm dt2 = {.tm_isdst = 0, .tm_yday = 1, .tm_wday = 6, .tm_year = 100, .tm_mon = 0, .tm_mday = 1, .tm_hour = 1, .tm_min = 2, .tm_sec = 30};
    struct tm dt3;

    time_t t1 = mktime (&dt1);
    time_t t2 = mktime (&dt2);
    time_t t3;

    TEST (difftime (t1, t1), 0);
    TEST (difftime (t2, t1), 90);
    TEST (difftime (t1, t2), -90);

    char *tstr1 = ctime (&t1);
    tstr1[21] = 0;

    TEST (strcmp (tstr1, "Sat Jan  1 1:1:0 2000"), 0);
    dt3 = *gmtime (&t2);
    t3 = mktime (&dt3);

    char *tstr2 = ctime (&t3);
    tstr2[22] = 0;

    TEST (difftime (t3, t2), 0);
    TEST (strcmp (tstr2, "Sat Jan  1 1:2:30 2000"), 0);
}

#include "vector.h"
static void testVector (void) {
    const char *libName = "vector";
    PRINT_LIB_NAME ();

    sVector *vec = vector_Create (0, 2);
    TEST (vec, NULL);
    vec = vector_Create (sizeof (int), 2);
    TEST (!!vec, !NULL);

    int i[5] = {1, 2, 3, 4, 5};
    vector_Add (vec, &i[0]);
    TEST (vec->MaxElementsAmount, 2);
    TEST (*(int *)vector_Get (vec, 0), 1);
    vector_Add (vec, &i[1]);
    TEST (*(int *)vector_Get (vec, 1), 2);
    vector_Add (vec, &i[2]);
    TEST (vec->MaxElementsAmount, 5);
    TEST (*(int *)vector_Get (vec, 2), 3);
    vector_Add (vec, &i[3]);
    TEST (*(int *)vector_Get (vec, 3), 4);
    vector_Add (vec, &i[4]);
    TEST (*(int *)vector_Get (vec, 4), 5);

    vector_AddEmpty (vec);
    TEST (vec->MaxElementsAmount, 11);
    TEST (*(int *)vector_Get (vec, 5), 0);

    vector_RemoveLast (vec);
    TEST (vector_Get (vec, 5), NULL);

    vector_Remove (vec, 2);
    TEST (vector_Get (vec, 4), NULL);
    TEST (*(int *)vector_Get (vec, 2), 4);

    TEST (*(int *)vector_GetLast (vec), 5);
    TEST (*(int *)vector_GetFirst (vec), 1);

    sVector *cpy = vector_Duplicate (vec);

    TEST (cpy->MaxElementsAmount, 11);
    TEST (*(int *)vector_Get (cpy, 0), 1);
    TEST (*(int *)vector_Get (cpy, 1), 2);
    TEST (*(int *)vector_Get (cpy, 2), 4);
    TEST (*(int *)vector_Get (cpy, 3), 5);
}

// #include "utils.h"
// static void testUtils(void)
// {
//     const char* libName = "utils";
//     PRINT_LIB_NAME();

//     TEST(log_2_n_round_down(4), 2);
//     TEST(log_2_n_round_down(5), 2);
//     TEST(log_2_n_round_down(6), 2);
//     TEST(log_2_n_round_down(7), 2);
//     TEST(log_2_n_round_down(8), 3);

//     TEST(log_2_n_round_up(4), 2);
//     TEST(log_2_n_round_up(5), 3);
//     TEST(log_2_n_round_up(6), 3);
//     TEST(log_2_n_round_up(7), 3);
//     TEST(log_2_n_round_up(8), 3);
// }

static void testFutils () {
    const char *libName = "futils";
    PRINT_LIB_NAME ();

    copyFile ("C:/F1.TXT", "C:/F2.TXT", 1);
    TEST (fileExists ("C:/F2.TXT"), true);
    TEST (fileSize ("C:/F2.TXT"), 1001);

    del ("F2.TXT");
    TEST (fileExists ("C:/F2.TXT"), false);

    copy ("C:/F1.TXT", "C:/F2.TXT", 1);
    TEST (fileExists ("C:/F2.TXT"), true);
    TEST (fileSize ("C:/F2.TXT"), 1001);

    del ("F2.TXT");
    TEST (fileExists ("C:/F2.TXT"), false);

    move ("C:/F1.TXT", "C:/SURF/F1.TXT", 1);
    TEST (fileExists ("C:/SURF/F1.TXT"), true);
    TEST (fileSize ("C:/SURF/F1.TXT"), 1001);

    move ("C:/SURF/F1.TXT", "C:/F1.TXT", 1);
    TEST (fileExists ("C:/F1.TXT"), true);
    TEST (fileSize ("C:/F1.TXT"), 1001);

    {
        char buf[256];
        getWorkDir (buf);
        TEST (strcmp (buf, "C:/"), 0);
    }

    dircopy ("C:/TEMP1/", "C:/TEMP2/");
    TEST (fileExists ("C:/TEMP2/F1.TXT"), true);
    TEST (fileSize ("C:/TEMP2/F1.TXT"), 1001);
    TEST (fileExists ("C:/TEMP2/F2.TXT"), true);
    TEST (fileSize ("C:/TEMP2/F2.TXT"), 2002);

    del ("C:/TEMP2/F1.TXT");
    TEST (fileExists ("C:/TEMP2/F1.TXT"), false);
    del ("C:/TEMP2/F2.TXT");
    TEST (fileExists ("C:/TEMP2/F2.TXT"), false);

    dirCopy_Delay ("C:/TEMP1/", "C:/TEMP2/", -1);
    TEST (fileExists ("C:/TEMP2/F1.TXT"), true);
    TEST (fileSize ("C:/TEMP2/F1.TXT"), 1001);
    TEST (fileExists ("C:/TEMP2/F2.TXT"), true);
    TEST (fileSize ("C:/TEMP2/F2.TXT"), 2002);

    del ("C:/TEMP2/F1.TXT");
    TEST (fileExists ("C:/TEMP2/F1.TXT"), false);
    del ("C:/TEMP2/F2.TXT");
    TEST (fileExists ("C:/TEMP2/F2.TXT"), false);

    copyFile_Delay ("C:/F1.TXT", "C:/F2.TXT", 1, -1);
    TEST (fileExists ("C:/F2.TXT"), true);
    TEST (fileSize ("C:/F2.TXT"), 1001);

    del ("F2.TXT");
    TEST (fileExists ("C:/F2.TXT"), false);

    copyFiles_Delay ("C:/TEMP1/", "C:/TEMP2/", 1, -1);
    TEST (fileExists ("C:/TEMP2/F1.TXT"), true);
    TEST (fileSize ("C:/TEMP2/F1.TXT"), 1001);
    TEST (fileExists ("C:/TEMP2/F2.TXT"), true);
    TEST (fileSize ("C:/TEMP2/F2.TXT"), 2002);

    del ("C:/TEMP2/F1.TXT");
    TEST (fileExists ("C:/TEMP2/F1.TXT"), false);
    del ("C:/TEMP2/F2.TXT");
    TEST (fileExists ("C:/TEMP2/F2.TXT"), false);

    copyFiles_Delay ("C:/TEMP1/", "C:/TEMP2/", 1, -1);
    rmdirf ("C:/TEMP2/");
    TEST (dirExists ("C:/TEMP2/"), false);
    mkdir ("C:/TEMP2/");

    copyFiles_Delay ("C:/TEMP1/", "C:/TEMP2/", 1, -1);
    removeContentFromDir ("C:/TEMP2/");
    TEST (dirExists ("C:/TEMP2/"), true);
    TEST (dirIsEmpty ("C:/TEMP2/"), OK);

    {
        int r = 10;
        char **arr = (char **)malloc (r * sizeof (char *));

        /// @todo Проверять выделение памяти на NULL
        for (int i = 0; i < 10; i++) 
            arr[i] = (char *)malloc (256 * sizeof (char));

        /// @todo В этом месте память выделится повторно - исправлять!
        arr = findFilesInDirAndSubdirs ("*.TXT", "C:/", 10, &r);

        TEST (strcmp ("TEMP1/F1.TXT", arr[0]), 0);
        TEST (strcmp ("TEMP1/F2.TXT", arr[1]), 0);
        TEST (strcmp ("F1.TXT", arr[2]), 0);

        for (int i = 0; i < 10; i++)
            if (arr[i] != NULL)
                free (arr[i]);
        free (arr);
    }
}

#include "inifiles.h"
static void testInifiles (void) {
    const char *libName = "inifiles";
    PRINT_LIB_NAME ();

    INI_FILE ini;
    ini = iniFileOpen ("TEST.INI");

    TEST (iniFileCheckIfSectionExists (ini, "section1"), true);
    TEST (iniFileCheckIfItemExists (ini, "section1", "key1"), TRUE);
    TEST (iniFileCheckIfItemExists (ini, "section1", "key2"), TRUE);
    TEST (iniFileCheckIfSectionExists (ini, "section2"), true);
    TEST (iniFileCheckIfItemExists (ini, "section2", "key3"), TRUE);
    TEST (iniFileCheckIfItemExists (ini, "section2", "key4"), TRUE);

    TEST (strcmp (iniFileSectionName (ini, 1), "section2"), 0);

    iniFileRenameSection (ini, "section1", "section0");
    TEST (iniFileCheckIfSectionExists (ini, "section1"), false);
    iniFileRenameSection (ini, "section2", "section1");
    TEST (iniFileCheckIfSectionExists (ini, "section1"), true);

    iniFileRenameSection (ini, "section1", "section2");
    iniFileRenameSection (ini, "section0", "section1");

    iniFileRenameItem (ini, "section1", "key1", "key0");
    TEST (iniFileCheckIfItemExists (ini, "section1", "key1"), FALSE);
    iniFileRenameItem (ini, "section1", "key0", "key1");

    TEST (strcmp (iniFileItemName (ini, "section2", 1), "key4"), 0);

    iniFileDeleteItem (ini, "section2", "key4");
    TEST (iniFileCheckIfItemExists (ini, "section2", "key4"), FALSE);

    iniFileDeleteSection (ini, "section2");
    TEST (iniFileCheckIfSectionExists (ini, "section2"), false);

    iniFileFree (ini);

    ini = iniFileOpen ("TEST.INI");
    iniFileChangeName (ini, "TEST1.INI");
    TEST (strcmp (ini->Name, "TEST1.INI"), 0);

    TEST (iniFileReadInteger (ini, "section1", "key1", 0), 1);
    TEST (strcmp (iniFileReadString (ini, "section1", "key2", "ERROR"), "string"), 0);
    TEST (strcmp (iniFileReadString (ini, "section2", "key3", "ERROR"), "idk"), 0);
    TEST (strcmp (iniFileReadString (ini, "section2", "key4", "ERROR"), "what to write here"), 0);
    TEST (iniFileReadHex (ini, "section3", "hex", -1), 0xA1);

    iniFileWriteInteger (ini, "section1", "key1", 12);
    TEST (iniFileReadInteger (ini, "section1", "key1", 0), 12);
    iniFileWriteInteger (ini, "section1", "key1", 1);

    iniFileWriteString (ini, "section2", "key3", "i don't know");
    TEST (strcmp (iniFileReadString (ini, "section2", "key3", "ERROR"), "i don't know"), 0);
    iniFileWriteString (ini, "section2", "key3", "idk");

    iniFileWriteHex (ini, "section3", "hex", 0xA2);
    TEST (iniFileReadHex (ini, "section3", "hex", -1), 0xA2);
    iniFileWriteHex (ini, "section3", "hex", 0xA1);

    {
        iniRect *rect = iniFileReadTextRect (ini, "section3", "textRect", rect);
        /// @todo Продумать тест (rect - значение, возвращаемое по умолчанию, это можно использовать).
        if (rect != NULL) {
            TEST (rect->x, 1);
            TEST (rect->y, 2);
            TEST (rect->w, 3);
            TEST (rect->h, 4);
            rect->x = 0;
            iniFileWriteTextRect (ini, "section3", "textRect", rect);
            TEST (rect->x, 0);
            rect->x = 1;
            iniFileWriteTextRect (ini, "section3", "textRect", rect);
            free (rect);
        }
        /// @todo Вывод, если NULL - продумать.
    }

    {
        iniCoords *coor;
        /// @todo Продумать, аналогично rect
        // coor->x = 0;
        // coor->y = 0;

        coor = iniFileReadCoords (ini, "section3", "coords", coor);
        TEST (coor->x, 5);
        TEST (coor->y, 6);
        coor->x = 0;
        iniFileWriteCoords (ini, "section3", "textRect", coor);
        TEST (coor->x, 0);
        coor->x = 5;
        iniFileWriteCoords (ini, "section3", "textRect", coor);
    }

    {
        iniIntArray *arr;
        /// @todo Продумать, аналогично rect

        arr = iniFileReadIntArray (ini, "section3", "int", arr);
        TEST (arr->Array[0], 7);
        TEST (arr->Array[1], 8);
        TEST (arr->Array[2], 9);
        TEST (arr->Array[3], 10);
        arr->Array[0] = 0;
        iniFileWriteIntArray (ini, "section3", "int", arr);
        TEST (arr->Array[0], 0);
        arr->Array[0] = 7;
        iniFileWriteIntArray (ini, "section3", "int", arr);
    }

    {
        iniBinaryArray *arr;
        /// @todo Продумать, аналогично rect

        arr = iniFileReadBinaryArray (ini, "section3", "bin", arr);
        TEST (arr->Array[0], 0xAB);
        TEST (arr->Array[1], 0x78);
        TEST (arr->Array[2], 0x22);
        TEST (arr->Array[3], 0x31);
        TEST (arr->Array[4], 0x09);
        arr->Array[0] = 0;
        iniFileWriteBinaryArray (ini, "section3", "bin", arr);
        TEST (arr->Array[0], 0);
        arr->Array[0] = 0xAB;
        iniFileWriteBinaryArray (ini, "section3", "bin", arr);
    }
}

#include "string.h"
#include "uchar.h"
static void testString (void) {
    const char *libName = "string";
    PRINT_LIB_NAME ();

    {
        char t1[] = "this is test string";
        char t2[] = "and this is too!";

        TEST (strcmp (t1, strcpy (t1, t2)), 0);
    }

    {
        char t1[] = "this is test string";
        char t2[] = "and this is too!";

        TEST (strcmp ("and thistest string", strncpy (t1, t2, 8)), 0);
    }

    {
        char t1[] = "this is test string";
        char t2[] = "and this is too!";

        TEST (strcmp ("this is test stringand this is too!", strcat (t1, t2)), 0);
    }

    {
        char t1[] = "this is test string";
        char t2[] = "and this is too!";

        TEST (strcmp ("this is test stringand this", strncat (t1, t2, 8)), 0);
    }

    {
        char t1[] = "this is test string";
        char t2[] = "this is too!";

        TEST (strncmp (t1, t2, 7), 0);
    }

    {
        char t1[] = "this is test string";

        TEST (strchr (t1, (int)'s'), &t1[3]);
    }

    {
        char t1[] = "this is test string";

        TEST (strcspn (t1, "wyar"), (size_t)15);
    }

    {
        char t1[] = "this is test string";

        TEST (strpbrk (t1, "wyar"), &t1[15]);
    }

    {
        char t1[] = "this is test string";

        TEST (strrchr (t1, (int)'s'), &t1[13]);
    }

    {
        char t1[] = "this is test string";

        TEST (strspn (t1, "this "), (size_t)9);
    }

    {
        char t1[] = "this is test string";

        TEST (strstr (t1, "is is"), &t1[2]);
    }

    {
        char t1[] = "this is test string";
        char *p;

        p = strtok (t1, " ");
        TEST (strcmp ("this", p), 0);
        p = strtok (0, " ");
        TEST (strcmp ("is", p), 0);
        p = strtok (0, " ");
        TEST (strcmp ("test", p), 0);
        p = strtok (0, " ");
        TEST (strcmp ("string", p), 0);
    }

    {
        TEST (strlen (""), 0);
        TEST (strlen ("1"), 1);
        TEST (strlen ("test"), 4);
    }

    {
        char *t1 = strdup ("test");
        TEST (strcmp (t1, "test"), 0);
    }

    {
        char T[] = "TEST";
        char t[] = "test";
        TEST (stricmp (t, T), 0);
    }

    {
        char t1[] = "this is test string";

        TEST (stristr (t1, "ST ST"), &t1[10]);
    }

    {
        char t1[] = "this is test string";

        TEST (strnlen (t1, (size_t)10), (size_t)10);
        TEST (strnlen (t1, (size_t)100), (size_t)19);
    }

    {
        char T[] = "TEST";
        char t[] = "test";
        TEST (strcmp (t, strlwr (T)), 0);
        strup (T);
        TEST (strcmp (strup (t), T), 0);
    }

    {
        TEST (upcase ('t'), 'T');
        TEST (lowcase ('T'), 't');
    }

    {
        char *t1 = malloc (sizeof (char) * 20);
        strcpy (t1, "this is test string");
        int len1 = strlen (t1);
        int len2 = merge (&t1, "!");
        TEST (len1 + 1, len2);
        TEST (strcmp ("this is test string!", t1), 0);
    }

    {
        char *t1 = "this is test string";
        strsep (&t1, " ");
        TEST (strcmp ("is test string", t1), 0);
    }

    {
        char t1[] = "this is test string";
        char *end = stpcpy (t1, "that is");
        TEST (strcmp ("that is", t1), 0);
        TEST ("", end - 1);
    }

    {
        char t1[] = "this is test string";
        char t2[] = "and this is too!";
        strlcpy (t1, t2, (size_t)8);
        TEST (strcmp ("and thi", t1), 0);
    }

    {
        char t1[25] = "this is test string";
        char t2[] = "and this is too!";
        strlcat (t1, t2, (size_t)25);
        TEST (strcmp ("this is test stringand t", t1), 0);
    }

    // тесты для char16_t

    // {
    //     char16_t *t1 = "this is test string";
    //     char16_t *t2 = "and this is too!";
    //     str16lcpy(t1, t2, (size_t)8);
    //     TEST(str16cmp("and thi", t1), 0);
    // }

    // {
    //     char16_t *t1 = "this is test string\0\0\0\0\0";
    //     char16_t *t2 = "and this is too!";
    //     str16lcat(t1, t2, (size_t)25);
    //     TEST(str16cmp("this is test stringand t", t1), 0);
    //     TEST(str16len(t2), (size_t)16);
    // }

    // {
    //     const char16_t *t1 = "this is test string";
    //     char16_t *t2 = str16dup(t1);
    //     TEST(str16cmp(t1, t2), 0);
    //     char16_t src = 'i';
    //     TEST(&t2[2], str16chr(t2, src));
    // }

    // тесты для hex-int/int-hex преобразований

    {
        char *_HEX_Base = "0123456789ABCDEF";
        UCHAR uch = UCHAR_MAX;
        USHORT ush = USHRT_MAX;
        int i = INT_MAX;
        long long ll = LLONG_MAX;
        char t[20];
        charToHex (uch, t);
        TEST (strcmp (t, "FF"), 0);
        shortToHex (ush, t);
        TEST (strcmp (t, "FFFF"), 0);
        intToHex (i, t);
        TEST (strcmp (t, "7FFFFFFF"), 0);
        longlongToHex (ll, t);
        TEST (strcmp (t, "7FFFFFFFFFFFFFFF"), 0);
        intToHexUniversal (i, t, 8, '0', _HEX_Base);
        TEST (strcmp (t, "7FFFFFFF"), 0);

        TEST (hexToInt ("FF"), UCHAR_MAX);
        TEST (hexToInt ("FFFF"), USHRT_MAX);
        TEST (hexToInt ("7FFFFFFF"), INT_MAX);
        TEST (hexToInt ("FFFFFFFF"), UINT_MAX);
    }
}

#include "inttypes.h"
static void testDiv64 (void) {
    const char *libName = "div64";
    PRINT_LIB_NAME ();

    uint64_t a = 1087;
    uint32_t b = 17;

    TEST (__div64_32 (&a, b), (uint32_t)16);
    TEST ((uint32_t)a, (uint32_t)63);
}

/// @todo Добавить параметр - вывод с подсветкой текста (ошибки красным, ок-зелёным и т.п.) и без подсветки.
void libKernel_TestLibraryFunctions (void) {
    /// @todo Проверить на утечки (memAvail до и после тестов).
    testStdio ();
    testStdLib ();
    testCType ();
    testFnames ();
    testDateTime ();
    testTime ();
    /// @todo Не прошёл
    testIolib ();
    /// @todo Не прошёл
    testStdfio ();
    testVector ();
    // testUtils();
    /// @todo Не прошёл
    testFutils ();
    testInifiles ();
    testString ();
    testDiv64 ();

    /// @todo Окончание с ошибкой - разбираться.
}