/**
 *  @addtogroup groupStdTypes
 *  @{
 *   @file time.h
 *   @brief Стандартные манипуляции с датой и временем.
 *   @details См. стандарт C11 7.27.
 *   @see <a href="https://www.iso.org/standard/57853.html">C11 standard 7.27</a>.
 *  @}
 */

/*
 * @author Sergey Sokolov
 * @author Dubovich Timofey
*/

#ifndef __TIME_H_
#define __TIME_H_

#include <stddef.h> // size_t, NULL

#define CLOCKS_PER_SEC (1000000)    //! @details Кол-во тиков в секунде.

/**
 * @name Временные базисы.
 * {
 */
#define TIME_UTC (1)    //! @details Временной базис: Всемирное координированное время.
///}@



typedef size_t time_t;  //! Календарное время. Секунды с 1 января 1970 года времени UTC.

typedef size_t clock_t; //! Тики процессора.

/**
 * Временной интервал.
 */
struct timespec
{
    time_t tv_sec;  //!< Количество целых секунд.
    long tv_nsec;   //!< Наносекунды, [0, 999999999].
};

/**
 * Время в календарном представлении.
 */
struct tm
{
    int tm_sec;     //!< @details Количество секунд, [0, 60].
    int tm_min;     //!< @details Количество минут, [0, 59].
    int tm_hour;    //!< @details Количество часов, [0, 23].
    int tm_mday;    //!< @details День в месяце, [1, 31].
    int tm_mon;     //!< @details Количество месяцев с Января, [0, 11].
    int tm_year;    //!< @details Количество лет с 1900.
    int tm_wday;    //!< @details Количество дней с Воскресенья, [0, 6].
    int tm_yday;    //!< @details Количество дней с 1го января, [0, 365].
    int tm_isdst;   //!< @details Флаг летнего времени.
};


/**
 * Получить суммарное процессорное время, использованное программой.
 * @return Суммарное процессорное время, использованное программой.
 */
clock_t clock(void);

/**
 * Получить кол-во секунд, прошедших со времени time0 и до времени time1.
 * @param time1 Конечная временная точка.
 * @param time0 Начальная временная точка.
 * @return Количество секунд разницы.
 */
double difftime(time_t time1, time_t time0);

/**
 * Преобразовать время в календарном представлении в абсолютное время.
 * @param timeptr Время в календарном представлении.
 * @return Календарное время для представленного времени или -1 при ошибке.
 */
time_t mktime(struct tm *timeptr);

/**
 * Вернуть время в секундах, прошедшее с начала эпохи.
 * @param timer Если не-NULL, то также сохранить результат в буфере.
 * @return Кол-во секунд с начала эпохи.
 */
time_t time(time_t *timer);

/**
 * Установить значение интервала времени в соответствии с временным базисом.
 * @param ts Указатель на заполняемый интервал.
 * @param base Временной базис.
 * @return Значение base при успехе, 0 при ошибке.
 */
int timespec_get(struct timespec *ts, int base);


/**
 * Преобразовать время в календарном представлении в строку.
 * @param timeptr Указатель время в календарном представлении.
 * @return Указатель на статически выделенную строку с результатом или \ref NULL при ошибке.
 */
char *asctime(const struct tm *timeptr);

/**
 * Преобразовать время в календарном представлении в строку и записать ее в предоставленный буфер.
 * @param timeptr Указатель время в календарном представлении.
 * @param buf Буфер длинной минимум 26 байт под итоговую строку.
 * @return Указатель на итоговую строку с результатом или \ref NULL при ошибке.
 */
char *asctime_r(const struct tm *restrict timeptr, char buf[static 26]);

/**
 * Преобразовать календарное время в строку.
 * @param timer Указатель на календарное время.
 * @return Указатель на статически выделенную строку с результатом или \ref NULL при ошибке.
 */
char *ctime(const time_t *timer);

/**
 * Преобразовать календарное время в локальное представление времени.
 * @param timer Указатель на календарное время.
 * @return Указатель на статически выделенную структуру или NULL при ошибке.
 */
struct tm *gmtime(const time_t *timer);

/**
 * Преобразовать календарное время в календарное представление времени и записать его в предоставленный буфер.
 * @param timer Указатель на календарное время.
 * @param result Буфер для записи.
 * @return Указатель на буфер для записи или NULL при ошибке.
 */
struct tm *gmtime_r(const time_t *restrict timer, struct tm *restrict result);

/**
 * Преобразовать календарное время в локальное время в календарном представлении.
 * @param timer Указатель на календарное время.
 * @return Указатель на статически выделенную структуру или NULL при ошибке.
 */
struct tm *localtime(const time_t *timer);

/**
 * Преобразовать календарное время в локальное время в календарном представлении и записать его в предоставленный буфер.
 * @param timer Указатель на календарное время.
 * @param result Буфер для записи.
 * @return Указатель на буфер для записи или NULL при ошибке.
 */
struct tm *localtime_r(const time_t *restrict timer,  struct tm *restrict result);

/**
 * Преобразовать календарное время в строку в соответствии с форматом.
 * @param s Буфер для записи.
 * @param maxsize Размер буфера для записи.
 * @param format Формат даты и времени.
 * @param timeptr Дата-время.
 * @return Кол-во символов в массиве s, не считая нуль-терминатор, при возврате 0 содержимое массива не определено.
 * @warning Фактически всегда распечатывает результат в формате asctime.
 */
size_t strftime(char *restrict s, size_t maxsize, const char *restrict format, const struct tm *restrict timeptr);


#endif /* __TIME_H_ */
