/**
    @addtogroup groupDrivers
    @{
    @file uart.h
    @brief Драйвер UART. Потоковая передача данных.
    @details Настройка аппаратного модуля \b UART.

    <b>Подключение</b>: @verbatim #include <uart.h> @endverbatim
    
    @see Общее описание работы с последовательным портом \b UART в разделе \ref sectionUart.
    @}

    @page pageDrivers

    @section sectionUart UART
    @see Описание методов работы с последовательным портом \b UART в файле uart.h.

    В состав @osName включен драйвер, обеспечивающий работу с последовательными
    портами \b UART в режиме потоковой записи / чтения. После настройки порт
    следует открыть с помощью функции open(), которая вернет дескриптор выбранного
    устройства. Используя этот дескриптор, записывать и читать данные
    можно стандартными средствами ОС -- функциями read() и write().


    Ниже приведён пример использования драйвера для приёма и пересылки обратно
    принятых данных. В начале инициализируется выбранный для передачи
    аппаратный модуль \b UART2. При инициализации он получает стандартные
    настройки, некоторые из которых бывает необходимо изменить. В примере
    изменяется скорость передачи данных на 38400 бит в секунду. Так же имеет
    смысл изменить таймаут на приём данных, чтобы задача периодически
    получала управление при отсутствии данных. Таймаут на добавление данных в
    очередь отправки можно сделать нулевым (\ref NO_WAIT), так как
    переполнений выходного буфера не предвидится. Размер буферов приёма и
    отправки можно было бы оставить без изменений, так как при создании
    каждому из них выделяется по 4096 байт, но на деле достаточно выделить
    под буферы память просто большую, чем ожидаемый размер пакетов. В примере
    буферы получают размер вдвое больший, чем нужно для приёма и передачи
    данных. 
    
    После инициализации аппаратного модуля происходит открытие устройства с
    помощью open(). В качестве имени устройства следует использовать функцию
    получения имени uartName(). После успешного открытия порта данные из
    порта можно читать порциями с помощью read() с указанием полученного
    дескриптора. Если данных достаточно они будут считаны сразу. Если данных
    не достаточно функция будет ждать накопления данных до истечения
    указанного при инициализации таймаута. По истечении заданного времени
    функция вернёт количество прочитанных байт. Если же указан таймаут \ref
    WAIT_FOREVER -- функция может не возвращать управление задаче разбора
    пакетов до накопления нужных данных. Отправку данных следует производить
    с помощью функции write() с указанием полученного при открытии дескриптора.

    @code
    int maxLen = 32;
    char data[maxLen];
    uartInit (UART_2, UART_GPIO_DEFAULT);
    uartSetBaudRate (UART_2, UART_BAUD_38400);
    uartSetReadTimeout (UART_2, 100);
    uartSetWriteTimeout (UART_2, NO_WAIT);
    uartSetWriteBufferSize (UART_2, maxLen*2);
    uartSetReadBufferSize (UART_2, maxLen*2);

    int uartFd = open (uartName (UART_2), O_RDWR);
    while (true) {
        int len = read (uartFd, data, maxLen);
        if (len)
            write (uartFd, data, len);
        else
            printf ("no data\n");
    }
    @endcode
    
 */

/*
    @author Матяш А.В. (a.matiash@set-code.ru)
    @date 07.07.2023
*/

#ifndef UART_H_
#define UART_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "siodrv.h"


/**
 *  @anchor ancUartDefines
 *  @name Макросы выбора каналов UART
 *  @{
 */

#define UART_0   0 ///< @details Индекс для \b UART0.
#define UART_1   1 ///< @details Индекс для \b UART1.
#define UART_2   2 ///< @details Индекс для \b UART2.
#define UART_3   3 ///< @details Индекс для \b UART3.
#define UART_4   4 ///< @details Индекс для \b UART4.
#define UART_5   5 ///< @details Индекс для \b UART5.
#define UART_6   6 ///< @details Индекс для \b UART6.
#define UART_7   7 ///< @details Индекс для \b UART7.
/// @}


/**
 *  @anchor ancUartGpioDefines
 *  @name Макросы выбора набора выходных линий UART
 *  @{
 */

#define UART_GPIO_DEFAULT    0 ///< @details Выбор основного набора линий модуля UART (обычно расположен в младших портах \b GPIO).
#define UART_GPIO_ADDITIONAL 1 ///< @details Выбор дополнительного набора линий модуля UART (обычно расположен в старших портах \b GPIO).

/// @}

/**
 *  @anchor ancUartBaudDefines
 *  @name Макросы выбора частоты передаваемого сигнала
 *  @{
 */

#define UART_BAUD_2400      2400    ///< @details Частота передачи данных   2400 бит/с.
#define UART_BAUD_4800      4800    ///< @details Частота передачи данных   4800 бит/с.
#define UART_BAUD_7200      7200    ///< @details Частота передачи данных   7200 бит/с.
#define UART_BAUD_9600      9600    ///< @details Частота передачи данных   9600 бит/с.
#define UART_BAUD_19200    19200    ///< @details Частота передачи данных  19200 бит/с.
#define UART_BAUD_38400    38400    ///< @details Частота передачи данных  38400 бит/с.
#define UART_BAUD_57600    57600    ///< @details Частота передачи данных  57600 бит/с.
#define UART_BAUD_115200  115200    ///< @details Частота передачи данных 115200 бит/с.
#define UART_BAUD_230400  230400    ///< @details Частота передачи данных 230400 бит/с.
#define UART_BAUD_460800  460800    ///< @details Частота передачи данных 460800 бит/с.
#define UART_BAUD_921600  921600    ///< @details Частота передачи данных 921600 бит/с.

/// @}

/**
 *  @brief Выбор режима контроля чётности.
 */
typedef enum {
    uartNoParity = 0,   ///< @details Бит контроля чётности отсутствует.
    uartOddParity,      ///< @details Бит контроля нечётности -- выставляется так, чтобы дополнить количество единиц в байте до нечётного числа.
    uartEvenParity,     ///< @details Бит контроля чётности -- выставляется так, чтобы дополнить количество единиц в байте до чётного числа.
    uartMarkParity,     ///< @details Бит контроля всегда выставляется в единицу.
    uartSpaceParity     ///< @details Бит контроля всегда выставляется в ноль.
} eUartParity;


/**
 *  @name Запуск аппаратного модуля
 *  @{
 */

/**
 *  @brief Настройка аппаратного модуля \b UART.
 *  @details Функция настраивает регистры выбранного аппаратного модуля
 *  и настраивает драйвер потокового ввода / вывода. По умолчанию
 *  драйвер \b UART имеет следующие настройки передачи данных: 
 *  - скорость 115200 бит/с;
 *  - 8 бит данных;
 *  - 1 старт бит;
 *  - 1 стоп бит;
 *  - без проверки чётности;
 *  - размер входного буфера -- 4096 байт;
 *  - размер выходного буфера -- 4096 байт;
 *  - таймаут ожидания приёма данных -- ждать до появления данных в приёмном буфере;
 *  - таймаут ожидания отправки данных -- ждать до успешного помещения данных в приёмный буфер;
 *  - режим передачи данных -- бинарный;
 *  - режим ожидания окончания отправки по линии -- отключен (см. uartSetWaitTxComplete() ).
 * 
 *  Для изменения настрое по умолчанию можно воспользоваться 
 *  \ref ancUartParams "функциями"
 *  настройки параметров передачи.
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @param gpion Номер набора портов ввода/вывода. Рекомендуется использовать
 *  значение из группы \ref ancUartGpioDefines "макросов".
 *  @return \ref OK если настройка прошла успешно, иначе \ref ERROR.
 */
STATUS uartInit (unsigned int n, unsigned int gpion);

/**
 *  @brief Имя устройства в текстовом виде.
 *  @details Имя устройство используется для открытия с помощью
 *  стандартной функции open().
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @return Имя устройства.
 */
const char *uartName (unsigned int n);

/// @}

/**
 *  @anchor ancUartParams
 *  @name Функции настройки параметров передачи данных
 *  @brief Функции настройки параметров можно использовать
 *  как при инициализации аппаратного модуля uartInit(),
 *  так и в паузах между отправляемыми пакетами данных.
 *  @{
 */

/**
 *  @brief Задать частоту передачи данных.
 *  @details Функция выставляет частоту передачи данных
 *  аппаратного модуля \b UART.
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @param baud Частота передачи в битах в секунду.
 *  Рекомендуется выбирать значение из соответствующей группы 
 *  \ref ancUartBaudDefines "макросов".
 *  @return \ref OK если настройка прошла успешно, иначе \ref ERROR.
 */
STATUS uartSetBaudRate (unsigned int n,unsigned int baud);

/**
 *  @brief Задать количество передаваемых бит.
 *  @details Функция задаёт количество значащих бит 
 *  между старт и стоп битами.
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @param bits Количество значащих бит от 5 до 8 включительно.
 *  @return \ref OK если настройка прошла успешно, иначе \ref ERROR.
 */
STATUS uartSetBitsNumber (unsigned int n, unsigned int bits);

/**
 *  @brief Задать количество стоп-бит.
 *  @details Функция задаёт количество стоп-бит при передаче данных.
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @param bits Количество стоп бит от 1 до 2 включительно.
 *  При посылках из 5-ти значащих бит, вместо 2 стоп битов будет генерироваться
 *  стоп бит длительностью 1,5.
 *  @return \ref OK если настройка прошла успешно, иначе \ref ERROR.
 */
STATUS uartSetStopBitsNumber (unsigned int n, unsigned int bits);

/**
 *  @brief Управлением битом контроля чётности.
 *  @details Функция позволяет задать поведение бита контроля чётности,
 *  либо вообще не передавать данный бит.
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @param parity Поведение бита контроля чётности.
 *  @return \ref OK если настройка прошла успешно, иначе \ref ERROR.
 */
STATUS uartSetParity (unsigned int n, eUartParity parity);



/**
 *  @brief Настройка размера приёмного буфера.
 *  @details Функция настраивает размер входного буфера.
 *  По умолчанию размер буфера \b 4096 байт.
 *  Рекомендуется устанавливать размер входного буфера
 *  сопоставимым с максимальным ожидаемым размером 
 *  единовременно принимаемых данных.
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @param size Размер приёмного буфера в байтах.
 *  @return \ref OK если настройка прошла успешно, иначе \ref ERROR.
 */
STATUS uartSetReadBufferSize (unsigned int n, unsigned int size);

/**
 *  @brief Настройка размера буфера отправки данных.
 *  @details Функция настраивает размер выходного буфера.
 *  По умолчанию размер буфера \b 4096 байт.
 *  Рекомендуется устанавливать размер выходного буфера
 *  сопоставимым с максимальным размером единовременно отправляемых данных.
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @param size Размер буфера отправки данных в байтах.
 *  @return \ref OK если настройка прошла успешно, иначе \ref ERROR.
 */
STATUS uartSetWriteBufferSize (unsigned int n, unsigned int size);

/**
 *  @brief Настройка таймаута приёма данных.
 *  @details Функция настраивает таймаут ожидания приёма данных.
 *  Таймаут задается в тиках системного таймера. Один тик по умолчанию равен
 *  одной милисекунде.
 *  Допустимы значения \ref NO_WAIT и \ref WAIT_FOREVER.
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @param timeout Таймаут на чтение данных в тиках системного таймера.
 *  @return \ref OK если настройка прошла успешно, иначе \ref ERROR.
 */
STATUS uartSetReadTimeout (unsigned int n, int timeout);


/**
 *  @brief Настройка таймаута передачи данных.
 *  @details Функция настраивает таймаут ожидания освобождения буфера для записи новых данных.
 *  Таймаут задается в тиках системного таймера. Один тик по умолчанию равен
 *  одной милисекунде.
 *  Допустимы значения \ref NO_WAIT и \ref WAIT_FOREVER.
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @param timeout Таймаут на запись данных в тиках системного таймера.
 *  @return \ref OK если настройка прошла успешно, иначе \ref ERROR.
 */
STATUS uartSetWriteTimeout (unsigned int n, int timeout);


/**
 *  @brief Включение текстового режима (режим консоли).
 *  @details Включение текстового режима (режим Консоли).
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @param enable Переключение в режим текстовой консоли. По умолчанию
 *  порты работают в бинарном режиме.
 *  @return \ref OK если настройка прошла успешно, иначе \ref ERROR.
 */
STATUS uartSetTextMode (unsigned int n, bool enable);


/// @}

/**
 *  @name Контроль работы UART
 *  @{
 */

/**
 *  @brief Очистка входного и выходного буферов.
 *  @details Функция очищает входной и выходной буферы 
 *  заданного аппаратного модуля \b UART.
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @return \ref OK если очистка прошла успешно, иначе \ref ERROR.
 */
STATUS uartFlush (unsigned int n);

/**
 *  @brief Количество ошибок получения данных.
 *  @details Функция возвращает количество ошибок получения данных
 *  связанных с переполнением принимающего буфера. Такие ошибки возникают
 *  при приёме большого потока данных, когда задача разбора данных не успевает
 *  читать все приходящие данные. В этом случае рекомендуется увеличить размер
 *  приёмного буфера, либо повысить приоритет такой задачи.
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @return Количество байт отброшенных в следствие переполнения приёмного буфера. 
 */
unsigned int uartReadBufferOverflowCounter (unsigned int n);

/**
 *  @brief Количество байт в приёмном буфере данных.
 *  @details Функция возвращает количество непрочитанных байт
 *  из приёмного буфера данных.
 *  Возвращаемое значение должно быть строго не отрицательным. Отрицательное число
 *  в результате говорит о некорректной работе буфера.
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @return Количество байт в буфере.
 */
int uartReadBufferCounter (unsigned int n);

/**
 *  @brief Количество байт в передающем буфере данных.
 *  @details Функция возвращает количество не отправленных байт
 *  в передающем буфере данных.
 *  Возвращаемое значение должно быть строго не отрицательным. Отрицательное число
 *  в результате говорит о некорректной работе буфера.
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @return Количество байт в буфере.
 */
int uartWriteBufferCounter (unsigned int n);

/**
 *  @brief Задать режим отправки с ожиданием реальной передачи данных по линии.
 *  @details Функция управляет режимом ожидания реальной отправки данных
 *  по линии передачи. По умолчанию функция write() ожидает записи данных
 *  в приёмный буфер и возвращает результат ещё до того как данные были
 *  переданы по линии. С помощью данной функции можно установить режим
 *  ожидания окончания отправки данных по линии передачи, в этом случае
 *  функция write() вернёт управление только после отправки последнего
 *  записываемого байта.
 *  
 *  @param n Номер модуля \b UART рекомендуется брать из соответствующей
 *  группы \ref ancUartDefines "макросов".
 *  @param enable \ref true -- режим ожидания отправки всех байт по линии передачи,
 *  \ref false -- режим ожидания записи данных в приёмный буфер.
 *  @return \ref OK в случае успешной настройки, иначе \ref ERROR.
 */
STATUS uartSetWaitTxComplete (unsigned int n, bool enable);

/// @}


/// @privatesection


/**
 *  @brief Инициализация модуля.
 *  @details Считывание информации об используемом
 *  процессоре. Функция используется системой --- пользователю
 *  вызывать не нужно.
 *  
 *  @return STATUS 
 */
STATUS uartArchInit ();


/**
 *  @brief Инициализация структуры com_port.
 *  @details Инициализация структуры com_port.
 *  @deprecated Функция будет удалена.
 *  
 */
void __attribute__((deprecated)) initUART ();

/**
 *  @brief Базовый адрес порта debug-uart.
 *  @details Функция возвращает базовый адрес регистров 
 *  аппаратного модуля \b UART0, обычно используемого
 *  для подключения \b debug -- консоли. Данная функция
 *  используется ядром ОС.
 *  
 *  @return Базовый адрес регистров \b UART0.
 */
NS16550_t debugUartBaseAddress ();

/**
 *  @brief Базовый адрес порта \b debug-uart.
 *  @details Макрос возвращает базовый адрес \b UART0 
 *  и добавлен для совместимости с 
 *  предыдущей версией драйвера.
 */
#define com_port (debugUartBaseAddress())

#ifdef __cplusplus
}
#endif
#endif  // ifndef UART_H_
