/**
 * @addtogroup groupCore
 * @{
 * @file unicode.h
 * @brief Преобразование строк и символов между различными кодировками (UTF-16, UTF-8, Cp1251, Cp866, Ascii).
 * @}
 */

#ifndef UNICODE_H_
#define UNICODE_H_

#ifdef __cplusplus
extern "C" {
#endif
/* Public declarations */

#include "stddef.h"
#include "uchar.h"

/**
 * Преобразовать нуль-терминированную строку в кодировке UTF-16 в нуль-терминированную строку в кодировке CP-1251.
 * @param uni Исходная строка в кодировке UTF-16.
 * @param res Буфер под строку в кодировке CP-1251.
 * @return Количество символов в итоговой строке.
 */
unsigned int uni2char(unsigned short *uni, unsigned char *res);

/**
 * Преобразовать символ из кодировки CP-1251 в кодировку CP-866.
 * @param c Символ в кодировке CP-1251.
 * @return Соответствующий ему символ в кодировке CP-866.
 */
unsigned char win2dos(unsigned char c);

/**
 * Преобразовать символ из кодировки CP-866 в кодировку CP-1251.
 * @param c Символ в кодировке CP-866.
 * @return Соответствующий ему символ в кодировке CP-1251.
 */
unsigned char dos2win(unsigned char c);

/**
 * Преобразовать строку в кодировке UTF-16 в строку в кодировке ASCII.
 * @param pUtf16String Исходная строка в кодировке UTF-16.
 * @param pOutputBuffer Буфер для вывода.
 * @param outputBufferLength Размер буфера для вывода.
 * @return Кол-во записанных в буфер для вывода символов.
 *
 * @details При недостаточном размере выводного буфера обработка исходной строки будет прекращена. Итоговая строка всегда будет нуль-терминирована.
 *
 * @note Символы, не подходящие под целевую кодировку, будут преобразованы в пробелы.
 */
size_t convert_Utf16ToAscii(const char16_t* pUtf16String, char* pOutputBuffer, size_t outputBufferLength);

/**
 * Преобразовать строку в кодировке UTF-16 в строку в кодировке CP-1251.
 * @param pUtf16String Исходная строка в кодировке UTF-16.
 * @param pOutputBuffer Буфер для вывода.
 * @param outputBufferLength Размер буфера для вывода.
 * @return Кол-во записанных в буфер для вывода символов.
 *
 * @details При недостаточном размере выводного буфера обработка исходной строки будет прекращена. Итоговая строка всегда будет нуль-терминирована.
 *
 * @note Символы, не подходящие под целевую кодировку, будут преобразованы в пробелы.
 */
size_t convert_Utf16ToCp1251(const char16_t* pUtf16String, char* pOutputBuffer, size_t outputBufferLength);

/**
 * Преобразовать строку в кодировке UTF-16 в строку в кодировке UTF-8.
 * @param pUtf16String Исходная строка в кодировке UTF-16.
 * @param pOutputBuffer Буфер для вывода.
 * @param outputBufferLength Размер буфера для вывода.
 * @return Кол-во записанных в буфер для вывода байтов.
 *
 * @details При недостаточном размере выводного буфера обработка исходной строки будет прекращена. Итоговая строка всегда будет нуль-терминирована.
 *
 * @note Поддерживаются только 1-но символьные UTF-16 символы.
 * При обнаружении 2х-символьного символа обработка строки будет прекращена
 * (итоговая строка все равно будет нуль-терминирована).
 */
size_t convert_Utf16ToUtf8(const char16_t* pUtf16String, char* pOutputBuffer, size_t outputBufferLength);

/**
 * Преобразовать строку в кодировке ASCII в строку в кодировке UTF-16.
 * @param pAsciiString Исходная строка в кодировке ASCII.
 * @param pOutputBuffer Буфер для вывода.
 * @param outputBufferLength Размер буфера для вывода (в символах, включая нулевой, не в байтах. Так, буфер в 20 байт будет иметь размер в 10 UTF-16 символов).
 * @return Кол-во записанных в буфер для вывода символов (не байтов).
 *
 * @details При недостаточном размере выводного буфера обработка исходной строки будет прекращена. Итоговая строка всегда будет нуль-терминирована.
 */
size_t convert_AsciiToUtf16(const char *pAsciiString, char16_t *pOutputBuffer, size_t outputBufferLength);

/**
 * Преобразовать строку в кодировке CP-1251 в строку в кодировке UTF-16.
 * @param pCp1251String Исходная строка в кодировке CP-1251.
 * @param pOutputBuffer Буфер для вывода.
 * @param outputBufferLength Размер буфера для вывода (в символах, включая нулевой, не в байтах. Так, буфер в 20 байт будет иметь размер в 10 UTF-16 символов).
 * @return Кол-во записанных в буфер для вывода символов (не байтов).
 *
 * @details При недостаточном размере выводного буфера обработка исходной строки будет прекращена. Итоговая строка всегда будет нуль-терминирована.
 */
size_t convert_Cp1251ToUtf16(const char *pCp1251String, char16_t *pOutputBuffer, size_t outputBufferLength);

/**
 * Преобразовать строку в кодировке UTF-8 в строку в кодировке UTF-16.
 * @param pUtf8String Исходная строка в кодировке UTF-8.
 * @param pOutputBuffer Буфер для вывода.
 * @param outputBufferLength Размер буфера для вывода (в символах, включая нулевой, не в байтах. Так, буфер в 20 байт будет иметь размер в 10 UTF-16 символов).
 * @note 4х-байтные UTF-8 символы не поддерживаются.
 * При обнаружении 4х-байтного символа обработка строки будет прекращена
 * (итоговая строка все равно будет нуль-терминирована).
 * @return Кол-во записанных в буфер для вывода символов (не байтов).
 *
 * @details При недостаточном размере выводного буфера обработка исходной строки будет прекращена. Итоговая строка всегда будет нуль-терминирована.
 */
size_t convert_Utf8ToUtf16(const char* pUtf8String, char16_t *pOutputBuffer, size_t outputBufferLength);

/**
 * Преобразовать строку в кодировке UTF-8 в строку в кодировке CP-1251.
 * @param pUtf8String Исходная строка в кодировке UTF-8.
 * @param pOutputBuffer Буфер для вывода.
 * @param outputBufferLength Размер буфера для вывода.
 * @return Кол-во записанных в буфер для вывода байтов.
 *
 * @details При недостаточном размере выводного буфера обработка исходной строки будет прекращена. Итоговая строка всегда будет нуль-терминирована.
 *
 * @note 4х-байтные UTF-8 символы не поддерживаются.
 * При обнаружении 4х-байтного символа обработка строки будет прекращена
 * (итоговая строка все равно будет нуль-терминирована).
 *
 * @note Символы, не подходящие под целевую кодировку, будут преобразованы в пробелы.
 */
size_t convert_Utf8ToCp1251(const char* pUtf8String, char *pOutputBuffer, size_t outputBufferLength);

/**
 * Преобразовать строку в кодировке CP-1251 в строку в кодировке UTF-8.
 * @param pCp1251String Исходная строка в кодировке CP-1251.
 * @param pOutputBuffer Буфер для вывода.
 * @param outputBufferLength Размер буфера для вывода.
 * @return Кол-во записанных в буфер для вывода байтов.
 *
 * @details При недостаточном размере выводного буфера обработка исходной строки будет прекращена. Итоговая строка всегда будет нуль-терминирована.
 */
size_t convert_Cp1251ToUtf8(const char *pCp1251String, char *pOutputBuffer, size_t outputBufferLength);



#ifdef __cplusplus
}
#endif
#endif // ifndef UNICODE_H_
